!------------------------------------------------------------------------------------------------!
! Module of Earth quantities                                                                     !
!                                                                                                !
! orginal version from Ulrike Seiler , ulrike.seiler@zmaw.de                                     !
!                                                                                                !
! (c) Petra Nerge 2012 , petra.nerge@informatik.uni-hamburg.de                                   !
!------------------------------------------------------------------------------------------------!
!                                                                                                !
! Description                                                                                    !
!                                                                                                !
!   Module of Earth quantities                                                                   !


MODULE mod_earth_quantities

    USE mod_kind
    USE mod_constants
    USE mod_grid
    USE mod_time_stepping , ONLY : delta_time

    IMPLICIT none

    !--- GLOBAL VARIABLES CONTAIN ---------------------------------------------------------------!
    !
    !--- Earth constants ------------------------------------------------------------------------!
    !--- Earth rotation period (day length) 
    !    in secs
    REAL (KIND=wp) , PARAMETER                      :: earth_rotation_period  = 86164._wp
    !--- Earth angular velocity in rad/secs
    REAL (KIND=wp) , PARAMETER                      :: earth_angular_velocity = zweipi                & 
                                                                              / earth_rotation_period 
    !--- Parameter of coriolis force 
    !    in rad/secs
    REAL (KIND=wp) , PARAMETER                      :: coriolis_parameter     = 2._wp                  &   !--- org: omecor
                                                                              * earth_angular_velocity 
    !--- Acceleration due to gravity
    !    in cm/secs^2 
    REAL (KIND=wp) , PARAMETER                      :: gravity_acceleration   = 980.6_wp
    !--------------------------------------------------------------------------------------------!

    !--- Earth quantities parametrization -------------------------------------------------------!
    !--- Parameter for tidal perturbation
    !    potential due to the loading and 
    !    self-attraction 
    REAL (KIND=wp) , PARAMETER                      :: lsa_parameter          = 0.1_wp  
    !--- Maximum coefficient of 
    !    horizontal eddy viscosity
    REAL (KIND=wp) , PARAMETER                      :: eddy_horizontal_coef   = 1.e5    
    !--- Coefficient of bottom friction
    REAL (KIND=wp) , PARAMETER                      :: bottom_friction_coef   = 0.003_wp    
    !--------------------------------------------------------------------------------------------!

    !--- Earth variables ------------------------------------------------------------------------!
    !--- Coriolis operator for  
    !    zonal velocities (u-points)
    REAL (KIND=wp) , DIMENSION (lats_start_u:lats_end_u)  :: coriolis_operator_u                 !--- org: fxcs
    !--- Coriolis operator for  
    !    meridional velocities (v-points)
    REAL (KIND=wp) , DIMENSION (lats_start_v:lats_end_v)  :: coriolis_operator_v                 !--- org: fycs 
    !--------------------------------------------------------------------------------------------!


    !--- INTERFACE CONTAIN ----------------------------------------------------------------------!
    !
    !--- Coriolis operator for zonal velocities (u-points) of type REAL(KIND=wp)
    INTERFACE earth_quantities_coriolis_u
            module procedure earth_quantities_coriolis_u
    END INTERFACE earth_quantities_coriolis_u
    !--- Coriolis operator for meridional velocities (v-points) of type REAL(KIND=wp)
    INTERFACE earth_quantities_coriolis_v
            module procedure earth_quantities_coriolis_v
    END INTERFACE earth_quantities_coriolis_v


  !---------------------------------------------------------------------------------------------!

  CONTAINS

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION earth_quantities_coriolis_u( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     coriolis operator for zonal momentum (u-points) of type REAL(KIND=wp)                 !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (lats_start_u:lats_end_u)  :: earth_quantities_coriolis_u  

        !--- Local variables
        REAL (KIND=wp)                                        :: coriolis

        !--- Help variables
        !--- Latitudes loop counter
        INTEGER                                               :: lat


        !---------------------------------------------------------------------------------------!
        !--- Coriolis operator for zonal momentum (u-points) -----------------------------------!

        DO lat = lats_start_u , lats_end_u
           coriolis                         = coriolis_parameter * sin_lat_u(lat) 
           earth_quantities_coriolis_u(lat) = SIN( coriolis * delta_time ) / delta_time
        END DO

        !---------------------------------------------------------------------------------------!

    END FUNCTION earth_quantities_coriolis_u

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION earth_quantities_coriolis_v( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     coriolis operator for meridional momentum (v-points) of type REAL(KIND=wp)            !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (lats_start_v:lats_end_v)  :: earth_quantities_coriolis_v  

        !--- Local variables
        REAL (KIND=wp)                                        :: coriolis

        !--- Help variables
        !--- Latitudes loop counter
        INTEGER                                               :: lat


        !---------------------------------------------------------------------------------------!
        !--- Coriolis operator for meridional momentum (v-points) ------------------------------!

        DO lat = lats_start_v , lats_end_v
           coriolis                         = coriolis_parameter * sin_lat_v(lat) 
           earth_quantities_coriolis_v(lat) = SIN( coriolis * delta_time ) / delta_time
        END DO

        !---------------------------------------------------------------------------------------!

    END FUNCTION earth_quantities_coriolis_v

    !-------------------------------------------------------------------------------------------!

END MODULE mod_earth_quantities


