!------------------------------------------------------------------------------------------------!
! Module of sea surface elevation                                                                !
!                                                                                                !
! orginal version from Ulrike Seiler , ulrike.seiler@zmaw.de                                     !
!                                                                                                !
! (c) Petra Nerge 2012 , petra.nerge@informatik.uni-hamburg.de                                   !
!------------------------------------------------------------------------------------------------!
!                                                                                                !
! Description                                                                                    !
!                                                                                                !
!   Module of sea surface elevation                                                              !


MODULE mod_elevation

    USE mod_kind
    USE mod_grid
    USE mod_time_stepping     , ONLY : delta_time
    USE mpi

    IMPLICIT none

    !--- GLOBAL CONSTANTS CONTAIN ---------------------------------------------------------------!
    !
    !--- Iteration of sea surface elevation -----------------------------------------------------!
    !--- First guess of sea surface elevation: 
    !    zeta_first_guess * zeta 
    REAL (KIND=wp) , PARAMETER                          :: zeta_new_first_guess_factor = 1._wp   !--- org: fiap
    !--- Maximum number of convergence checks
    !    for the iteration of sea surface elevation
    INTEGER        , PARAMETER                          :: max_convergence_checks      = 300     !--- org: mpscan
    !--- Number of iterations to be performed before 
    !    convergence is checked
    INTEGER        , PARAMETER                          :: number_iterations           = 100     !--- org: mxscan
    !--- Convergence criterion of iteration
    !    Criterion to stop iteration
    REAL (KIND=wp) , PARAMETER                          :: convergence_criterion       = 0.01_wp !--- org: crit
    !--------------------------------------------------------------------------------------------!


    !--- GLOBAL VARIABLES CONTAIN ---------------------------------------------------------------!
    !
    !--- State variables ------------------------------------------------------------------------!
    !--- Field of sea surface elevation
    REAL (KIND=wp) , DIMENSION (longs_start_sub_1:longs_end_add_1 , lats_start_zeta:lats_end_zeta) &
    :: zeta 
    !--------------------------------------------------------------------------------------------!

    !--- Iteration of sea surface elevation -----------------------------------------------------!
    !--- Field of sea surface elevation 
    !    semi-implicit iteration coefficients 
    REAL (KIND=wp) , DIMENSION (longs_start:longs_end , lats_start_zeta:lats_end_zeta) &
    :: zeta_iteration_coef                                                                       !--- org: rhsi
    !--- Field of sea surface elevation 
    !    semi-implicit iteration coefficients 
    !    easterly of elevation
    REAL (KIND=wp) , DIMENSION (longs_start:longs_end ,    lats_start_u:lats_end_u   ) &
    :: zeta_iteration_coef_east                                                                  !--- org: cfe    
    !--- Field of sea surface elevation 
    !    semi-implicit iteration coefficients 
    !    westerly of elevation 
    REAL (KIND=wp) , DIMENSION (longs_start:longs_end ,    lats_start_u:lats_end_u   ) &
    :: zeta_iteration_coef_west                                                                  !--- org: cfw 
    !--- Field of sea surface elevation 
    !    semi-implicit iteration coefficients 
    !    southerly of elevation 
    REAL (KIND=wp) , DIMENSION (longs_start:longs_end ,    lats_start_v:lats_end_v   ) &
    :: zeta_iteration_coef_south                                                                 !--- org: cfs
    !--- Field of sea surface elevation 
    !    semi-implicit iteration coefficients 
    !    northerly of elevation 
    REAL (KIND=wp) , DIMENSION (longs_start:longs_end ,    lats_start_u:lats_end_u   ) &
    :: zeta_iteration_coef_north                                                                 !--- org_ cfn
    !--- Number of convergence checks be done
    INTEGER                                                                            &   
    :: zeta_new_convergence_checks
    !--- Maximum of resdiuum
    REAL (KIND=wp)                                                                     &
    :: zeta_new_residuum_max                                                                     !--- org: resmap
    !--------------------------------------------------------------------------------------------!
 

    !--- INTERFACE CONTAIN
    !
    !--- Prepare of semi-implicit calculation of elevation
    !    elevation due to mass-retention equation for the sum of explicit calculated velocities
    !    at time step and velocities at time step - 1:                                         
    INTERFACE elevation_iteration_zeta_coefficient
            module procedure elevation_iteration_zeta_coefficient
    END INTERFACE elevation_iteration_zeta_coefficient


  !---------------------------------------------------------------------------------------------!

  CONTAINS

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION elevation_iteration_zeta_coefficient( u_new_exp             , &               !--- org: rhsi 
                                                        u_last                , &
                                                        total_depth_u_points  , &
                                                        v_new_exp             , &
                                                        v_last                , &
                                                        total_depth_v_points    &
                                                      )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value of type REAL(KIND=wp)                                                    !
    !     iteration matrix coefficient at zeta-points based on                                  !
    !     elevation due to mass-retention equation for the sum of explicit calculated velocities!
    !     at time step and velocities at time step - 1:                                         !
    !     delta(elevation)/delta(timestep) =                                                    !
    !                     - 1/(earthradius*cos(lat)) *                                          !
    !                     ( delta((total depth)*(sum zonal velocity))/delta(lon)                !
    !                     +delta((total depth)*(sum meridional velocity)*cos(lat))/delta(lat) ) ! 
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp)               , DIMENSION (       longs_start:longs_end , lats_start_zeta:lats_end_zeta) &
        :: elevation_iteration_zeta_coefficient  
 
        !--- Function variables
        !--- Field of explicit calculated zonal velocities
        !    at time step
        REAL (KIND=wp) , INTENT (IN) , DIMENSION (      longs_start:longs_end ,      lats_start_u:lats_end_u  ) &
        :: u_new_exp 
        !--- Field of zonal velocity
        !    at time step - 1 
        REAL (KIND=wp) , INTENT (IN) , DIMENSION (      longs_start:longs_end ,      lats_start_u:lats_end_u  ) &
        :: u_last
        !--- Field of explicit calculated meridional velocities
        !    at time step
        REAL (KIND=wp) , INTENT (IN) , DIMENSION (      longs_start:longs_end ,      lats_start_v:lats_end_v  ) &
        :: v_new_exp 
        !--- Field of meridional velocity
        !    at time step - 1 
        REAL (KIND=wp) , INTENT (IN) , DIMENSION (      longs_start:longs_end ,      lats_start_v:lats_end_v  ) &
        :: v_last
        !--- Field of total depth at u-points
        !    at time step - 1
        REAL (KIND=wp) , INTENT (IN) , DIMENSION (      longs_start:longs_end ,      lats_start_u:lats_end_u  ) &
        :: total_depth_u_points
        !--- Field of total depth at u-points
        !    at time step - 1
        REAL (KIND=wp) , INTENT (IN) , DIMENSION (      longs_start:longs_end ,      lats_start_v:lats_end_v  ) &
        :: total_depth_v_points

        !--- Local variables
        !--- Field of mass flow due to zonal velocity 
        REAL (KIND=wp)               , DIMENSION (longs_start_sub_1:longs_end ,      lats_start_u:lats_end_u  ) &
        :: zonal_vel_mul_depth
        !--- Field of mass flow due to meridional velocity 
        REAL (KIND=wp)               , DIMENSION (      longs_start:longs_end ,      lats_start_v:lats_end_v  ) &
        :: merid_vel_mul_depth
        REAL (KIND=wp)                                                                                          &
        :: coefficient_central
        REAL (KIND=wp)                                                                                          &
        :: coefficient_north
        REAL (KIND=wp)                                                                                          &
        :: coefficient_south
        REAL (KIND=wp)                                                                                          &
        :: constant 

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitudes loop counter
        INTEGER                       :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate iteration matrix coefficients at zeta-points ----------------------------!

        !--- Calculate zonal velocity (u) related term of mass-retention 
        !    (total depth at u-points)*(sum of u(last time step) and explizit part of u(time step) 
        !--- org: cfw
        zonal_vel_mul_depth(longs_start:longs_end , lats_start_u:lats_end_u) &
        = total_depth_u_points                                               &
        * (          u_last(longs_start:longs_end , lats_start_u:lats_end_u) &  
                + u_new_exp(longs_start:longs_end , lats_start_u:lats_end_u) &
          )
        !--- cyclic condition
        zonal_vel_mul_depth(longs_start_sub_1 , lats_start_u:lats_end_u) =  &
        zonal_vel_mul_depth(longs_end         , lats_start_u:lats_end_u)

        !--- Calculate meridional velocity (v) related term of mass-retention 
        !    (total depth at v-points)*(sum of v(time step) and explizit part of v(time step + 1) 
        !--- org: cfn
        merid_vel_mul_depth(longs_start:longs_end , lats_start_v:lats_end_v) &
        = total_depth_v_points                                               &
        * (          v_last(longs_start:longs_end , lats_start_v:lats_end_v) &
                + v_new_exp(longs_start:longs_end , lats_start_v:lats_end_v) &
          )

        !--- Calculate iteration coefficients due to zonal velocity related term of mass-retention 
        !--- set constant (time step)/earthradius*delta(lon) of zonal velocity related term 
        constant = delta_time * delta_x_inv_half
        DO lat = lats_start_u , lats_end_u
           !--- set coefficient: constant*1/cos(lat(zeta)) of zonal velocity related term
           !    cos(lat(zeta))=cos(lat(u)): zeta and u-points are on same latitudes
           coefficient_central = constant * cos_lat_u_inv(lat)
           !--- calcutate iteration coefficients 
           DO lon = longs_start , longs_end 
              elevation_iteration_zeta_coefficient(lon   , lat) &
                         = coefficient_central                  &
                         * (   zonal_vel_mul_depth(lon-1 , lat) &
                             - zonal_vel_mul_depth(lon   , lat) &
                           ) 
           END DO
        END DO

        !--- Calculate coefficient due to meridional velocity related term of mass-retention 
        !--- set constant (time step)/earthradius*delta(lat) of meridional velocity related term 
        constant = delta_time * delta_y_inv_half
        !--- calculate of first latitude (lats_start_zeta) 
        !--- set coefficient: constant*1/cos(lat(zeta))*cos(lat(v)) of meridional velocity related term
        !    cos(lat(zeta))=cos(lat(u)): zeta and u-points are on same latitudes
        coefficient_north = constant *  weight_v_north(lats_start_zeta)
        !--- calculate iteration coefficients
        DO lon = longs_start , longs_end
           elevation_iteration_zeta_coefficient(lon , lats_start_zeta) & 
           =                                                           &
           elevation_iteration_zeta_coefficient(lon , lats_start_zeta) &
           - coefficient_north                                         &
           *                merid_vel_mul_depth(lon , lats_start_v   )  
        END DO
        !--- calculate remaining latitudes up to last latitude of zonal velocity
        DO lat = lats_start_v , lats_end_u
           !--- set coefficient: constant*1/cos(lat(zeta))*cos(lat(v)) of meridional velocity related term
           !    cos(lat(zeta))=cos(lat(u)): zeta and u-points are on same latitudes
           coefficient_north = constant *  weight_v_north(lat)
           coefficient_south = constant *  weight_v_south(lat)
           !--- calculate iteration coefficients
           DO lon = longs_start , longs_end
              elevation_iteration_zeta_coefficient(lon , lat) &
              =                                               &
              elevation_iteration_zeta_coefficient(lon , lat) &
              + coefficient_south                             &
              *                merid_vel_mul_depth(lon , lat)  
           END DO
           DO lon = longs_start , longs_end
              elevation_iteration_zeta_coefficient(lon , lat)   &
              =                                                 &
              elevation_iteration_zeta_coefficient(lon , lat)   &
              - coefficient_north                               &
              *                merid_vel_mul_depth(lon , lat+1)   
           END DO
        END DO 
        
        !--- Calculate iteration coefficients at last latitude
        merid_vel_mul_depth(longs_start , lats_end_v) = SUM( merid_vel_mul_depth(: , lats_end_v) )
        elevation_iteration_zeta_coefficient(longs_start , lats_end_zeta) &
        = 0.5_wp                                                          &  
        *                merid_vel_mul_depth(longs_start , lats_end_v  )  &
        * weight_polar                                                    &
        * delta_time                                                      &
        +                               zeta(longs_start , lats_end_zeta)              

        !---------------------------------------------------------------------------------------!

    END FUNCTION elevation_iteration_zeta_coefficient

    !-------------------------------------------------------------------------------------------!

    PURE SUBROUTINE elevation_iteration_matrix_prepare( coefficient_east                 , & 
                                                        coefficient_west                 , &
                                                        coefficient_south                , &
                                                        coefficient_north                , &
                                                        coefficient_zeta                 , & 
                                                        u_new_exp                        , & 
                                                        u_last                           , &
                                                        v_new_exp                        , &
                                                        v_last                           , &
                                                        total_depth_u_points             , &
                                                        total_depth_v_points             , & 
                                                        zonal_pressure_gradient_coef_imp , &
                                                        merid_pressure_gradient_coef_imp   &
                                                      )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - determine iteration matrix for the semi-implicit calculation of the sea surface       !
    !     elevation                                                                             !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Subroutine variables
        !--- Field of sea surface elevation 
        !    semi-implicit iteration coefficients 
        !    easterly of elevation 
        REAL (KIND=wp) , INTENT (INOUT) , DIMENSION (longs_start:longs_end ,    lats_start_u:lats_end_u   ) &
        :: coefficient_east 
        !--- Field of sea surface elevation 
        !    semi-implicit iteration coefficients 
        !    westerly of elevation 
        REAL (KIND=wp) , INTENT (INOUT) , DIMENSION (longs_start:longs_end ,    lats_start_u:lats_end_u   ) &
        :: coefficient_west 
        !--- Field of sea surface elevation 
        !    semi-implicit iteration coefficients 
        !    southerly of elevation 
        REAL (KIND=wp) , INTENT (INOUT) , DIMENSION (longs_start:longs_end ,    lats_start_v:lats_end_v   ) & 
        :: coefficient_south
        !--- Field of sea surface elevation 
        !    semi-implicit iteration coefficients 
        !    northerly of elevation 
        REAL (KIND=wp) , INTENT (INOUT) , DIMENSION (longs_start:longs_end ,    lats_start_u:lats_end_u   ) & 
        :: coefficient_north
        !--- Field of semi-implicit sea surface elevation  
        REAL (KIND=wp) , INTENT (INOUT) , DIMENSION (longs_start:longs_end , lats_start_zeta:lats_end_zeta) & 
        :: coefficient_zeta
        !--- Field of explicit calculated zonal velocities
        !    at time step
        REAL (KIND=wp) , INTENT (IN)    , DIMENSION (longs_start:longs_end ,    lats_start_u:lats_end_u   ) & 
        :: u_new_exp 
        !--- Field of zonal velocity
        !    at time step - 1 
        REAL (KIND=wp) , INTENT (IN)    , DIMENSION (longs_start:longs_end ,    lats_start_u:lats_end_u   ) & 
        :: u_last
        !--- Field of explicit calculated meridional velocities
        !    at time step
        REAL (KIND=wp) , INTENT (IN)    , DIMENSION (longs_start:longs_end ,    lats_start_v:lats_end_v   ) & 
        :: v_new_exp 
        !--- Field of meridional velocity
        !    at time step - 1 
        REAL (KIND=wp) , INTENT (IN)    , DIMENSION (longs_start:longs_end ,    lats_start_v:lats_end_v   ) & 
        :: v_last
        !--- Field of total depth at u-points
        !    at time step - 1
        REAL (KIND=wp) , INTENT (IN)    , DIMENSION (longs_start:longs_end ,    lats_start_u:lats_end_u   ) & 
        :: total_depth_u_points
        !--- Field of total depth at u-points
        !    at time step - 1
        REAL (KIND=wp) , INTENT (IN)    , DIMENSION (longs_start:longs_end ,    lats_start_v:lats_end_v   ) & 
        :: total_depth_v_points
        !--- Field of implicit part of zonal pressure gradient  
        REAL (KIND=wp) , INTENT (IN)    , DIMENSION (longs_start:longs_end ,    lats_start_u:lats_end_u   ) & 
        :: zonal_pressure_gradient_coef_imp
        !--- Field of implicit part of meridional pressure gradient  
        REAL (KIND=wp) , INTENT (IN)    , DIMENSION (longs_start:longs_end ,    lats_start_v:lats_end_v   ) & 
        :: merid_pressure_gradient_coef_imp

        !--- Local variables
        REAL (KIND=wp)                :: coefficient
        REAL (KIND=wp)                :: constant 
        REAL (KIND=wp)                :: cennor
        REAL (KIND=wp)                :: censou
        REAL (KIND=wp)                :: center

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitudes loop counter
        INTEGER                       :: lat 


        !---------------------------------------------------------------------------------------!
        !--- determine iteration matrix for the semi-implicit calculation of the sea surface ---!
        !    elevation                                                                          !

        !--- Calculate central/zeta coefficients 
        coefficient_zeta        (longs_start:longs_end , lats_start_zeta:lats_end_zeta)    &
        = elevation_iteration_zeta_coefficient                                             &
          (                                                                                & 
                       u_new_exp(longs_start:longs_end ,    lats_start_u:lats_end_u   ) ,  & 
                          u_last(longs_start:longs_end ,    lats_start_u:lats_end_u   ) ,  &
            total_depth_u_points(longs_start:longs_end ,    lats_start_u:lats_end_u   ) ,  &
                       v_new_exp(longs_start:longs_end ,    lats_start_v:lats_end_v   ) ,  &
                          v_last(longs_start:longs_end ,    lats_start_v:lats_end_v   ) ,  &
            total_depth_v_points(longs_start:longs_end ,    lats_start_v:lats_end_v   )    &
          )

        !--- Calculate easterly coefficients
        !--- set constant (time step)/earthradius*delta(lon) of zonal velocity related term 
        constant = delta_time * delta_x_inv_half
        DO lat = lats_start_u , lats_end_u
           !--- set coefficient: constant*1/cos(lat(zeta)) of zonal velocity related term
           !    cos(lat(zeta))=cos(lat(u)): zeta and u-points are on same latitudes
           coefficient = constant * cos_lat_u_inv(lat)
           !--- calcutate iteration coefficient
           DO lon = longs_start , longs_end
              coefficient_east                  (lon , lat) &
              = coefficient                                 &
              *             total_depth_u_points(lon , lat) &
              * zonal_pressure_gradient_coef_imp(lon , lat)  
           END DO
        END DO

        !--- Calculate westerly coefficients
        coefficient_west(longs_start , lats_start_u:lats_end_u) = &
        coefficient_east(longs_end   , lats_start_u:lats_end_u)

        DO lat = lats_start_u , lats_end_u
           DO lon = longs_start_add_1 , longs_end
              coefficient_west(lon   , lat) = &
              coefficient_east(lon-1 , lat)
           END DO
        END DO
         
        !--- Calculate unweighted southerly coefficients
        !--- set constant (time step)/earthradius*delta(lat) of meridional velocity related term 
        coefficient = delta_time * delta_y_inv_half
        !--- calcutate iteration coefficient
        DO lat = lats_start_v , lats_end_v
           DO lon = longs_start , longs_end
              coefficient_south                 (lon , lat) &
              = coefficient                                 &
              * total_depth_v_points            (lon , lat) &
              * merid_pressure_gradient_coef_imp(lon , lat)  
           END DO
        END DO

        !--- Calculate weighted northerly coefficients
        DO lat = lats_start_u , lats_end_u
           DO lon = longs_start , longs_end
              coefficient_north(lon , lat  ) = &
              coefficient_south(lon , lat+1) * &
                 weight_v_north(      lat  )
           END DO
        END DO

        !--- Calculate weighted southerly coefficients
        DO lat = lats_start_v , lats_end_v
           DO lon = longs_start , longs_end
              coefficient_south(lon , lat) = &
              coefficient_south(lon , lat) * &
                 weight_v_south(      lat)
           END DO
        END DO
         
        !--- Boundary condition of southerly coeffients for northernmost latitude
        constant = weight_polar * delta_time * delta_y                                          !--- org: polfac
        coefficient_south(longs_start:longs_end , lats_end_v) = & 
        coefficient_south(longs_start:longs_end , lats_end_v) * &
        constant

        !--- Normalization of coefficient arrays
        DO lon = longs_start , longs_end
           censou                                = &
           1._wp                                 + &
           coefficient_north(lon , lats_start_u) + &
           coefficient_west (lon , lats_start_u) + &
           coefficient_east (lon , lats_start_u)
           censou = 1._wp / censou

           coefficient_north(lon , lats_start_u) = &
           coefficient_north(lon , lats_start_u) * &
           censou

           coefficient_west (lon , lats_start_u) = &
           coefficient_west (lon , lats_start_u) * &
           censou

           coefficient_east (lon , lats_start_u) = &
           coefficient_east (lon , lats_start_u) * &
           censou

           coefficient_zeta (lon , lats_start_u) = &
           coefficient_zeta (lon , lats_start_u) * &
           censou
        END DO

        cennor = 1._wp
        DO lon = longs_start , longs_end
           cennor = cennor + coefficient_south(lon , lats_end_v)
        END DO
        cennor = 1._wp / cennor

        coefficient_zeta (longs_start , lats_end_zeta) = &
        coefficient_zeta (longs_start , lats_end_zeta) * &
        cennor

        coefficient_south(longs_start:longs_end , lats_end_v ) = &
        coefficient_south(longs_start:longs_end , lats_end_v ) * &
        cennor

        DO lat = lats_start_v , lats_end_u
           DO lon = longs_start , longs_end
              center =                       &
              1._wp                        + & 
              coefficient_west (lon , lat) + &
              coefficient_east (lon , lat) + &
              coefficient_south(lon , lat) + &
              coefficient_north(lon , lat)  
              center = 1._wp / center

              coefficient_west (lon , lat) = &
              coefficient_west (lon , lat) * &
              center
 
              coefficient_east (lon , lat) = &
              coefficient_east (lon , lat) * &
              center
 
              coefficient_south(lon , lat) = &
              coefficient_south(lon , lat) * &
              center
 
              coefficient_north(lon , lat) = &
              coefficient_north(lon , lat) * &
              center 
              
              coefficient_zeta (lon , lat) = &
              coefficient_zeta (lon , lat) * &
              center 
           END DO
        END DO
        
        !---------------------------------------------------------------------------------------!

    END SUBROUTINE elevation_iteration_matrix_prepare

    !-------------------------------------------------------------------------------------------!

    SUBROUTINE elevation_iteration_relaxation( zeta_new          , &
                                                    coefficient_east  , &
                                                    coefficient_west  , &
                                                    coefficient_south , &
                                                    coefficient_north , &
                                                    coefficient_zeta  , &
                                                    residuum_max      , &
                                                    count_checks      , &
                                                    stat                &
                                                  )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - iteration of sea surface elevation by relaxation                                      !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Subroutine variables
        REAL (KIND=wp) , INTENT (INOUT) , DIMENSION (longs_start_sub_1:longs_end_add_1 , lats_start_zeta:lats_end_zeta) &
        :: zeta_new                                                                             !--- org: zetaa
        !--- Field of sea surface elevation 
        !    semi-implicit iteration coefficients 
        !    easterly of elevation 
        REAL (KIND=wp) , INTENT (IN)    , DIMENSION (      longs_start:longs_end       ,    lats_start_u:lats_end_u   ) &
        :: coefficient_east 
        !--- Field of sea surface elevation 
        !    semi-implicit iteration coefficients 
        !    westerly of elevation 
        REAL (KIND=wp) , INTENT (IN)    , DIMENSION (      longs_start:longs_end       ,    lats_start_u:lats_end_u   ) &
        :: coefficient_west 
        !--- Field of sea surface elevation 
        !    semi-implicit iteration coefficients 
        !    southerly of elevation 
        REAL (KIND=wp) , INTENT (IN)    , DIMENSION (      longs_start:longs_end       ,    lats_start_v:lats_end_v   ) &
        :: coefficient_south
        !--- Field of sea surface elevation 
        !    semi-implicit iteration coefficients 
        !    northerly of elevation 
        REAL (KIND=wp) , INTENT (IN)    , DIMENSION (      longs_start:longs_end       ,    lats_start_u:lats_end_u   ) &
        :: coefficient_north
        !--- Field of semi-implicit sea surface elevation  
        REAL (KIND=wp) , INTENT (IN)    , DIMENSION (      longs_start:longs_end       , lats_start_zeta:lats_end_zeta) &
        :: coefficient_zeta
        !--- Maximum of residuum
        REAL (KIND=wp) , INTENT (OUT)                                                                                   &      
        :: residuum_max                                                                         !--- org: resmap
        !--- Number of done convergence checks
        INTEGER        , INTENT (OUT)                                                                                   &
        :: count_checks
        !--- Status of subroutine
        LOGICAL        , INTENT (OUT)                                                                                   &
        :: stat

        !--- Local variables
        !--- Residuum of iteration
        REAL (KIND=wp)                  , DIMENSION (longs_start_sub_1:longs_end_add_1 , lats_start_zeta:lats_end_zeta) &
        :: residuum                                                                             !--- org: res
        !--- Polar residuum of iteration
        REAL (KIND=wp)                                                                                                  &
        :: residuum_polar                                                                       !--- org: respno
        !--- Maximum residuum of southerly hemisphere
        REAL (KIND=wp)                                                                                                  &
        :: residuum_south_max                                                                   !--- org: resmas
        !--- Maximaum residuum of northerly hemisphere
        REAL (KIND=wp)                                                                                                  &
        :: residuum_north_max                                                                   !--- org: resman
        !--- Latitude at which the iteration starts 
        !    for southerly hemisphere 
        INTEGER                                                                                                         &
        :: start_south_lat 
        !--- Latitude at which the iteration ends 
        !    for southerly hemisphere 
        INTEGER                                                                                                         &      
        :: end_south_lat                                                                        !--- org: jeqs 
        !--- Latitude at which the iteration starts 
        !    for northerly hemisphere 
        INTEGER                                                                                                         &
        :: start_north_lat                                                                      !--- org: jeqn 
        !--- Latitude at which the iteration ends 
        !    for northerly hemisphere 
        INTEGER                                                                                                         &
        :: end_north_lat 
        !--- Choosen start latitude for iterations  
        INTEGER                                                                                                         &
        :: start_lat
        !--- Choosen end latitude for iterations  
        INTEGER                                                                                                         &
        :: end_lat 
        !--- Choosen hemisphere for iterations   
        INTEGER                                                                                                         &
        :: hemisphere 
        
        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitudes loop counter
        INTEGER                       :: lat
        !--- Iteration loop counter 
        INTEGER                       :: count_iteration

        !!!!!! MPI !!!!!!!
        !---communicator for MPI
        INTEGER :: old_comm, cylinder_comm, ndims, reorder, IERROR, nprocs, rank, faktor, long_end, count
        INTEGER dim_size(2)
        LOGICAL periods (0:1)
        INTEGER :: mpi_end = -1
        
        CALL MPI_Comm_Size(MPI_COMM_WORLD, nprocs, IERROR)

        old_comm = MPI_COMM_WORLD
        ndims = 2
        dim_size(1) = 1            !longs = 360
        dim_size(2) = nprocs           !lats = 90
        periods(0)  = .true.        !row-periodic  
        periods(1)  = .false.       !column-nonperiodic
        reorder     = 1             !enable new structure
        
        CALL MPI_Cart_create(old_comm, ndims, dim_size, periods, reorder,  & 
                             cylinder_comm, IERROR)
        CALL MPI_COMM_SIZE(cylinder_comm, nprocs, IERROR)
        CALL MPI_COMM_RANK(cylinder_comm, rank, IERROR)
        
        !---------------------------------------------------------------------------------------!
        !--- iteration of sea surface elevation by relaxation ----------------------------------!

        !--- initialize status of subroutine to false
        !    iteration of sea surface elevation do not converge
        stat = .false.

        !--- first guess of zeta_new
        zeta_new(longs_start_sub_1:longs_end_add_1 , lats_start_zeta:lats_end_zeta) = &
        zeta_new_first_guess_factor                                                 * & 
            zeta(longs_start_sub_1:longs_end_add_1 , lats_start_zeta:lats_end_zeta)

        !--- set start latitude of all iterations for southerly hemisphere
        start_south_lat = lats_start_zeta 
        !--- set end latitude of all iterations for northerly hemisphere
        end_north_lat = lats_end_zeta
        
        !--- set end latitude of first iteration for southerly hemisphere
        end_south_lat = lats_equator_u
        !--- set start latitude of first iteration for northerly hemisphere
        start_north_lat = lats_equator_v

        !new loop options
        lon = longs_start + (rank * 360 / nprocs)
        mpi_end = lon + (360 / nprocs)
        
                 
        !--- start iteration
        DO count_checks = 0 , max_convergence_checks
           DO count_iteration = 1 , number_iterations
              !--- set southerly hemisphere to iterate first
              hemisphere = 1
              start_lat  = start_south_lat + 1
              end_lat    = end_south_lat
              DO WHILE ( hemisphere < 3 )
                 !--- latitudes without boundaries 
                 !lon = longs_start + (rank * 90 / nprocs)
                 !mpi_end = lon + (90 / nprocs)
                 !write (*,*) lon, lat, rank
                 DO lat = start_lat, end_lat
                    DO lon = longs_start , mpi_end 
                       
                       
                       residuum         (lon , lat)                           = &
                       coefficient_north(lon , lat) * zeta_new(lon   , lat+1) + &
                       coefficient_south(lon , lat) * zeta_new(lon   , lat-1) + &
                       coefficient_east (lon , lat) * zeta_new(lon+1 , lat  ) + &
                       coefficient_west (lon , lat) * zeta_new(lon-1 , lat  ) - &
                                                     zeta_new(lon   , lat  ) + &
                       coefficient_zeta (lon , lat) 
                       
                       count = count +1
                    END DO
                 END DO
                 !sending of calculates data
                 CALL MPI_Alltoall(residuum(lon, start_lat), lon, MPI_REAL, residuum, &
                                     count * (nprocs), MPI_REAL, cylinder_comm, IERROR)
                 !--- set northerly hemisphere to iterate next 
                 hemisphere = hemisphere + 1
                 start_lat  = start_north_lat
                 end_lat    = end_north_lat - 1
                 count = 0
              END DO

              !--- boundary condition for southernmost latitude
              !!!!!!!!!!!!!!!!!DO lon = longs_start , longs_end
              DO lon = longs_start , longs_end
                 residuum         (lon , start_south_lat)                                       = &
                 coefficient_north(lon , start_south_lat) * zeta_new(lon   , start_south_lat+1) + &
                 coefficient_east (lon , start_south_lat) * zeta_new(lon+1 , start_south_lat  ) + &
                 coefficient_west (lon , start_south_lat) * zeta_new(lon-1 , start_south_lat  ) - &
                                                            zeta_new(lon   , start_south_lat  ) + &
                 coefficient_zeta (lon , start_south_lat) 
              END DO

              !--- boundary condition for northernmost latitude
              residuum_polar = coefficient_zeta(longs_start , lats_end_zeta)
              DO lon = longs_start , longs_end
                 residuum_polar = residuum_polar                      &
                                + coefficient_south(lon , lats_end_v) &
                                *          zeta_new(lon , lats_end_u)                    
              END DO
              residuum(longs_start:longs_end , lats_end_zeta) = &
              residuum_polar                                  - & 
              zeta_new(longs_start:longs_end , lats_end_zeta)

              !--- cyclic boundary condition for longitudes on southerly hemisphere
              residuum(longs_start_sub_1 , start_south_lat:end_south_lat) = &
              residuum(longs_end         , start_south_lat:end_south_lat)

              residuum(longs_end_add_1   , start_south_lat:end_south_lat) = &
              residuum(longs_start       , start_south_lat:end_south_lat) 

              !--- cyclic boundary condition for longitudes on northerly hemisphere
              residuum(longs_start_sub_1 , start_north_lat:end_north_lat) = &
              residuum(longs_end         , start_north_lat:end_north_lat)

              residuum(longs_end_add_1   , start_north_lat:end_north_lat) = &
              residuum(longs_start       , start_north_lat:end_north_lat) 

              !--- new zeta on southerly hemisphere
              zeta_new(longs_start_sub_1:longs_end_add_1 , start_south_lat:end_south_lat) = &
              zeta_new(longs_start_sub_1:longs_end_add_1 , start_south_lat:end_south_lat) + & 
              residuum(longs_start_sub_1:longs_end_add_1 , start_south_lat:end_south_lat)

              !--- new zeta on northerly hemisphere
              zeta_new(longs_start_sub_1:longs_end_add_1 , start_north_lat:end_north_lat) = &
              zeta_new(longs_start_sub_1:longs_end_add_1 , start_north_lat:end_north_lat) + & 
              residuum(longs_start_sub_1:longs_end_add_1 , start_north_lat:end_north_lat)
           END DO

           !--- check convergence determination
           IF ( max_convergence_checks < 1 ) THEN 
                   stat = .true.
                   RETURN
           END IF

           !--- initialize maximum of residuum to false
           residuum_max = -99999._wp
 
           !--- set maximum of residuum to false for southern hemisphere
           residuum_south_max = -99999._wp
           !--- determine area on southern hemisphere where no further iteration is necessary
           !    set end latitude of next iteration for southerly hemisphere
           DO end_south_lat = end_south_lat , start_south_lat , -1
              DO lon = longs_start , longs_end
                 residuum_south_max = MAX( residuum_south_max                   , &
                                           ABS( residuum(lon , end_south_lat) )   &
                                         )
              END DO
              IF ( residuum_south_max > convergence_criterion) EXIT
              residuum_max = MAX( residuum_max , residuum_south_max )
           END DO
           IF ( end_south_lat >= start_south_lat .and. end_south_lat < lats_equator_u ) THEN 
                   end_south_lat = end_south_lat + 1
           END IF
        
           !--- set maximum of residuum to false for northern hemisphere
           residuum_north_max = -99999._wp
           !--- determine area on northern hemisphere where no further iteration is necessary
           !    set start latitude of next iteration for northerly hemisphere
           DO start_north_lat = start_north_lat , end_north_lat 
              DO lon = longs_start , longs_end
                 residuum_north_max = MAX( residuum_north_max                     , &
                                           ABS( residuum(lon , start_north_lat) )   &
                                         )
              END DO
              IF ( residuum_north_max > convergence_criterion) EXIT
              residuum_max = MAX( residuum_max , residuum_north_max)
           END DO
           IF ( start_north_lat > lats_equator_v .and. start_north_lat <= end_north_lat ) THEN 
                   start_north_lat = start_north_lat - 1
           !--- check convergence on both hemispheres
           ELSE IF ( start_north_lat > end_north_lat .and. end_south_lat < start_south_lat ) THEN
                   stat =.true. 
                   RETURN  
           END IF
        END DO

        !---------------------------------------------------------------------------------------!

    END SUBROUTINE elevation_iteration_relaxation

    !-------------------------------------------------------------------------------------------!

END MODULE mod_elevation


