!------------------------------------------------------------------------------------------------!
! Module of grid definition and initialize                                                       !
!                                                                                                !
! orginal version from Ulrike Seiler , ulrike.seiler@zmaw.de                                     !
!                                                                                                !
! (c) Petra Nerge 2012 , petra.nerge@informatik.uni-hamburg.de                                   !
!------------------------------------------------------------------------------------------------!
!                                                                                                !
! Description                                                                                    !
!                                                                                                !
!   Module of grid definition and initialize                                                     !


MODULE mod_grid

    USE mod_kind
    USE mod_nice_math
    USE mod_constants

    IMPLICIT none

    !--- GLOBAL VARIABLES CONTAIN ---------------------------------------------------------------!
    !
    !--- Grid constants -------------------------------------------------------------------------!
    !--- Earth radius in cm
    REAL (KIND=wp) , PARAMETER                :: radius     = 6371.e+05_wp
    !--- Inverse Earth radius in 1/cm
    REAL (KIND=wp) , PARAMETER                :: radius_inv = 1._wp / radius

    !--- 1° Arakawa C grid of the Earth      ----------------------------------------------------!
    !--- Grid spacing in x-direction (zonal) ----------------------------------------------------!
    !--- Grid spacing in degree
    REAL (KIND=wp) , PARAMETER                :: delta_x_deg        = 1._wp                            !--- org: dxdeg
    !--- Grid spacing in cm
    REAL (KIND=wp) , PARAMETER                :: delta_x            = delta_x_deg * radius * radian    !--- org: dx
    !--- Inverse grid spacing in 1/cm
    REAL (KIND=wp) , PARAMETER                :: delta_x_inv        = 1._wp / delta_x                  !--- org: dxr
    !--- Half of rekursive grid spacing
    !    in 1/cm
    REAL (KIND=wp) , PARAMETER                :: delta_x_inv_half   = 0.5_wp * delta_x_inv             !--- org: dx2r
    !--- Square of rekursive grid spacing
    !in 1/cm^2
    REAL (KIND=wp) , PARAMETER                :: delta_x_inv_square = delta_x_inv * delta_x_inv        !--- org: dxrq

    !--- Grid spacing in y-direction (meridional) -----------------------------------------------!
    !--- Grid spacing in degree
    REAL (KIND=wp) , PARAMETER                :: delta_y_deg        = 1._wp                            !--- org: dydeg
    !--- Grid spacing in cm
    REAL (KIND=wp) , PARAMETER                :: delta_y            = delta_y_deg * radius * radian    !--- org: dy
    !--- Rekursive grid spacing in 1/cm
    REAL (KIND=wp) , PARAMETER                :: delta_y_inv        = 1._wp / delta_y                  !--- org: dyr
    !--- Half of rekursive grid spacing
    !    in 1/cm
    REAL (KIND=wp) , PARAMETER                :: delta_y_inv_half   = 0.5_wp * delta_y_inv             !--- org: dy2r
    !--- Square of rekursive grid spacing
    !    in 1/cm^2
    REAL (KIND=wp) , PARAMETER                :: delta_y_inv_square = delta_y_inv * delta_y_inv        !--- org: dyrq

    !--- Polar cap represented for calculation by one point -------------------------------------!
    !--- Number of meridional points of the
    !    polar cap
    INTEGER        , PARAMETER                :: polar_points       = 4                                !--- org: ncap
    !--- Meridional spacing of the cap
    !    in degree
    REAL (KIND=wp) , PARAMETER                :: delta_y_deg_polar  = polar_points * delta_y_deg       !--- org: dysdeg
    !--- Meridional spacing of the cap
    !    in radian
    REAL (KIND=wp) , PARAMETER                :: delta_y_polar      = delta_y_deg_polar * radian       !--- org: dys
    !--- Area of polar circle 
    !    in cm^2
    REAL (KIND=wp) , PARAMETER                :: polar_circle_area  = zweipi * radius * radius       &
                                                                    * (1._wp - COS( delta_y_polar ))   !--- org: polar
    !--- Polar cap weighting  
    !    in 1/cm
    REAL (KIND=wp) , PARAMETER                :: weight_polar       = SIN( delta_y_polar ) & 
                                                                    * delta_x              &  
                                                                    / polar_circle_area                !--- org: pole / delt

    !--- Longitudes -----------------------------------------------------------------------------!
    !--- Number of longitudes
!    INTEGER        , PARAMETER                :: longs         = INT( 360 / delta_x_deg )
    INTEGER        , PARAMETER                :: inter_longs         = INT( 360 / delta_x_deg )
    !--------------------------------------------------------------------------------------------!

    !--- Latitudes ------------------------------------------------------------------------------!
    !--- Number of latidudes
    INTEGER        , PARAMETER                :: lats         = INT( 180 / delta_y_deg )
    !--- Number of meridional points of
    !    zeta (sea surface elevation)
    INTEGER        , PARAMETER                :: lats_zeta    = lats                             !--- org: lamz
    !--- Number of meridional points of
    !    u (zonal velocity)
    INTEGER        , PARAMETER                :: lats_u       = lats_zeta - 1                    !--- org: lamu
    !--- Number of meridional points of
    !    u (zonal velocity) + 1
    INTEGER        , PARAMETER                :: lats_u_add_1 = lats_u + 1                       !--- org: lamup
    !--- Number of meridional points of
    !    v (meridional velocity)
    INTEGER        , PARAMETER                :: lats_v       = lats_zeta                        !--- org: lamv
    !--- Number of meridional points of
    !    v (meridional velocity) + 1
    INTEGER        , PARAMETER                :: lats_v_add_1 = lats_v + 1                       !--- org: lamvp
    !--------------------------------------------------------------------------------------------!

    !--- Counter --------------------------------------------------------------------------------!
    !--- Start of longitudes counting -----------------------------------------------------------!
    INTEGER        , PARAMETER                :: longs_start        = 1   
    !--- Start of longitudes counting + 1
    INTEGER        , PARAMETER                :: longs_start_add_1  = longs_start + 1   
    !--- Start of longitudes counting - 1
    INTEGER        , PARAMETER                :: longs_start_sub_1  = longs_start - 1   
    !--- End of longitudes counting 
    INTEGER        , PARAMETER                :: longs_end          = inter_longs   
    !--- End of longitudes counting + 1
    INTEGER        , PARAMETER                :: longs_end_add_1    = longs_end + 1              !--- org: longp
    !--- Half of number of longitudes
    INTEGER        , PARAMETER                :: longs_end_half     = longs_end / 2              !--- org: lonh
    !--- Quarter of number of longitudes
    INTEGER        , PARAMETER                :: longs_end_quarter  = longs_end / 4              !--- org: lonv
    !--------------------------------------------------------------------------------------------!

    !--- Start of latitudes counting ------------------------------------------------------------!
!    INTEGER        , PARAMETER                :: lats_start        = 1                           !--- org: lafi
    INTEGER        , PARAMETER                :: inter_lats_start   = 1                          !--- org: lafi
    !--------------------------------------------------------------------------------------------!

    !--- Latitudes counter for sea surface elevation (zeta-points) ------------------------------!
    !--- Start counting
    INTEGER        , PARAMETER                :: lats_start_zeta    = inter_lats_start                  
    !--- End counting
    INTEGER        , PARAMETER                :: lats_end_zeta      = lats_zeta - polar_points + 1 !--- org: latz              
    !--------------------------------------------------------------------------------------------!

    !--- Latitudes counter for zonal momentum/velocity (u-points) -------------------------------!
    !--- Start counting
    INTEGER        , PARAMETER                :: lats_start_u       = lats_start_zeta                  
    !--- Start counting + 1
    INTEGER        , PARAMETER                :: lats_start_u_add_1 = lats_start_u + 1                  
    !--- End counting
    INTEGER        , PARAMETER                :: lats_end_u         = lats_end_zeta - 1          !--- org: latu
    !--- End of latitudes counting + 1
    INTEGER        , PARAMETER                :: lats_end_u_add_1   = lats_end_u + 1             !--- org: latup
    !--- Equatorial u-point
    INTEGER        , PARAMETER                :: lats_equator_u     = lats_zeta / 2              !--- org: lateu
    !--------------------------------------------------------------------------------------------!

    !--- Latitudes counter for meridional momentum/velocity (v-points) --------------------------!
    !--- Start counting
    INTEGER        , PARAMETER                :: lats_start_v       = lats_start_zeta + 1        !--- org: lafip
    !--- Start counting + 1
    INTEGER        , PARAMETER                :: lats_start_v_add_1 = lats_start_v + 1                  
    !--- End counting
    INTEGER        , PARAMETER                :: lats_end_v         = lats_end_zeta              !--- org: latv
    !--- End of latitudes counting + 1
    INTEGER        , PARAMETER                :: lats_end_v_add_1   = lats_end_v + 1             !--- org: latvp
    !--- Equatorial v-point
    INTEGER        , PARAMETER                :: lats_equator_v     = lats_equator_u + 1         !--- org: latev
    !--------------------------------------------------------------------------------------------!

    !--- Grid variables related to latitudes ----------------------------------------------------!
    !--- Cosine and sine of latitudes at
    !    u-points
    REAL (KIND=wp) , DIMENSION (lats_start_u:lats_end_u_add_1)  :: cos_lat_u , sin_lat_u         !--- org: cosu , sinu
    !--- Inverse cosine of latitudes at
    !    u-points
    REAL (KIND=wp) , DIMENSION (lats_start_u:lats_end_u_add_1)  :: cos_lat_u_inv                 !--- org: cosur
    !--- Tangens of latitudes at 
    !    u-points
    REAL (KIND=wp) , DIMENSION (lats_start_u:lats_end_u_add_1)  :: tan_lat_u                     !--- org: fxt
    !--- Cosine and sine of latitudes at
    !    v-points
    REAL (KIND=wp) , DIMENSION (lats_start_v:lats_end_v      )  :: cos_lat_v , sin_lat_v         !--- cosv , sinv
    !--- Inverse cosine of latitudes at
    !    v-points
    REAL (KIND=wp) , DIMENSION (lats_start_v:lats_end_v      )  :: cos_lat_v_inv                 !--- org: cosvr
    !--- Tangens of latitudes at 
    !    v-points
    REAL (KIND=wp) , DIMENSION (lats_start_v:lats_end_v      )  :: tan_lat_v                     !--- org: fyt
    !--- Northern weighting coefficients for 
    !    interpolation of meridional 
    !    velocities at u-points  
    REAL (KIND=wp) , DIMENSION (lats_start_u:lats_end_u      )  :: weight_v_north                !--- org: gun
    !--- Southern weighting coefficients for 
    !    interpolation of meridional 
    !    velocities at u-points  
    REAL (KIND=wp) , DIMENSION (lats_start_v:lats_end_v      )  :: weight_v_south                !--- org: gus
    !--- Northern weighting coefficients for 
    !    interpolation of zonal 
    !    velocities at v-points  
    REAL (KIND=wp) , DIMENSION (lats_start_v:lats_end_v      )  :: weight_u_north                !--- org: gvn
    !--- Southern weighting coefficients for 
    !    interpolation of zonal 
    !    velocities at v-points  
    REAL (KIND=wp) , DIMENSION (lats_start_v:lats_end_v      )  :: weight_u_south                !--- org: gvs
    !--------------------------------------------------------------------------------------------!

    !--- Grid variables related to longitudes ---------------------------------------------------!
    !--- Cosine and sine of longitudes at
    !    u-points
    REAL (KIND=wp) , DIMENSION (longs_start:longs_end)    :: cos_lon_u , sin_lon_u               !--- org: cosul , sinul
    !--- Cosine and sine of longitudes at
    !    v-points
    REAL (KIND=wp) , DIMENSION (longs_start:longs_end)    :: cos_lon_v , sin_lon_v               !--- org: cosvl, sinvl
    !--- Grid variables related to tidal
    !    forcing and diagnostic functions
    REAL (KIND=wp) , DIMENSION (longs_start:longs_end)    :: acsul , asnul
    REAL (KIND=wp) , DIMENSION (longs_start:longs_end)    :: acsvl , asnvl
    !--- Grid variables related to
    !    diagnostic functions
    REAL (KIND=wp) , DIMENSION (longs_start:longs_end)    :: acc , ass , acs
    !--------------------------------------------------------------------------------------------!

    !--- Grid variables related to depth --------------------------------------------------------!    
    !--- Ocean's non-pertubated depth at zeta-points in cm
    REAL (KIND=wp) , DIMENSION (longs_start:longs_end       ,               1:lats          ) &
    :: depth                                                                                     !--- org: dep                    
    !--- Ocean's pertubated depth at zeta-points in cm
    REAL (KIND=wp) , DIMENSION (longs_start:longs_end_add_1 , lats_start_zeta:lats_end_zeta ) &
    :: depth_add_zeta                                                                            !--- org: res
    !--- Ocean's pertubated depth at u-points in cm
    REAL (KIND=wp) , DIMENSION (longs_start:longs_end       ,    lats_start_u:lats_end_u    ) &
    :: depth_add_zeta_u_points                                                                   !--- org: cfe
    !--- Ocean's pertubated depth at v-points in cm
    REAL (KIND=wp) , DIMENSION (longs_start:longs_end       ,    lats_start_v:lats_end_v    ) &
    :: depth_add_zeta_v_points                                                                   !--- org: cfs
    !--------------------------------------------------------------------------------------------!


    !--- INTERFACE CONTAIN ----------------------------------------------------------------------!
    !
    !--- Initialize of the grid variables
    INTERFACE grid_initialize
            module procedure grid_initialize
    END INTERFACE grid_initialize
    !--- Cosine of latitudes at the grid points of u (zonal velocity) of type REAL(KIND=wp)
    INTERFACE grid_initialize_cos_lat_u
            module procedure grid_initialize_cos_lat_u
    END INTERFACE grid_initialize_cos_lat_u
    !--- Cosine of latitudes at the grid points of v (meridional velocity) of type REAL(KIND=wp)
    INTERFACE grid_initialize_cos_lat_v
            module procedure grid_initialize_cos_lat_v
    END INTERFACE grid_initialize_cos_lat_v
    !--- Sine of latitudes at the grid points of u (zonal velocity) of type REAL(KIND=wp)
    INTERFACE grid_initialize_sin_lat_u
            module procedure grid_initialize_sin_lat_u
    END INTERFACE grid_initialize_sin_lat_u
    !--- Sine of latitudes at the grid points of v (meridional velocity) of type REAL(KIND=wp)
    INTERFACE grid_initialize_sin_lat_v
            module procedure grid_initialize_sin_lat_v
    END INTERFACE grid_initialize_sin_lat_v
    !--- Inverse cosine of latitudes at the grid points of u (zonal velocity) of type REAL(KIND=wp)
    INTERFACE grid_initialize_cos_lat_u_inv
            module procedure grid_initialize_cos_lat_u_inv
    END INTERFACE grid_initialize_cos_lat_u_inv
    !--- Inverse cosine of latitudes at the grid points of v (meridional velocity) of type REAL(KIND=wp)
    INTERFACE grid_initialize_cos_lat_v_inv
            module procedure grid_initialize_cos_lat_v_inv
    END INTERFACE grid_initialize_cos_lat_v_inv
    !--- Cosine of longitudes at the grid points of u (zonal velocity) of type REAL(KIND=wp)
    INTERFACE grid_initialize_cos_lon_u
            module procedure grid_initialize_cos_lon_u
    END INTERFACE grid_initialize_cos_lon_u
    !--- Cosine of longitudes at the grid points of v (meridional velocity) of type REAL(KIND=wp)
    INTERFACE grid_initialize_cos_lon_v
            module procedure grid_initialize_cos_lon_v
    END INTERFACE grid_initialize_cos_lon_v
    !--- Sine of longitudes at the grid points of u (zonal velocity) of type REAL(KIND=wp)
    INTERFACE grid_initialize_sin_lon_u
            module procedure grid_initialize_sin_lon_u
    END INTERFACE grid_initialize_sin_lon_u
    !--- Sine of longitudes at the grid points of v (meridional velocity) of type REAL(KIND=wp)
    INTERFACE grid_initialize_sin_lon_v
            module procedure grid_initialize_sin_lon_v
    END INTERFACE grid_initialize_sin_lon_v
    !--- Cosine of longitudes at the grid points of u (zonal velocity) of type REAL(KIND=wp) 
    !    for tidal forcing and diagnostic functions                         
    INTERFACE grid_initialize_cos_lon_u_acs
            module procedure grid_initialize_cos_lon_u_acs
    END INTERFACE grid_initialize_cos_lon_u_acs
    !--- Cosine of longitudes at the grid points of v (meridional velocity) of type REAL(KIND=wp) 
    !    for tidal forcing and diagnostic functions                         
    INTERFACE grid_initialize_cos_lon_v_acs
            module procedure grid_initialize_cos_lon_v_acs
    END INTERFACE grid_initialize_cos_lon_v_acs
    !--- Sine of longitudes at the grid points of u (zonal velocity) of type REAL(KIND=wp) 
    !    for tidal forcing and diagnostic functions                         
    INTERFACE grid_initialize_sin_lon_u_asn
            module procedure grid_initialize_sin_lon_u_asn
    END INTERFACE grid_initialize_sin_lon_u_asn
    !--- Sine of longitudes at the grid points of v (meridional velocity) of type REAL(KIND=wp) 
    !    for tidal forcing and diagnostic functions                         
    INTERFACE grid_initialize_sin_lon_v_asn
            module procedure grid_initialize_sin_lon_v_asn
    END INTERFACE grid_initialize_sin_lon_v_asn
    !--- Rewrite the cosine of longitudes at the grid points of u (zonal velocity) of 
    !    type REAL(KIND=wp) 
    INTERFACE grid_initialize_cos_lon_u_rewrite
            module procedure grid_initialize_cos_lon_u_rewrite
    END INTERFACE grid_initialize_cos_lon_u_rewrite
    !--- Rewrite the cosine of longitudes at the grid points of v (meridional velocity) of 
    !    type REAL(KIND=wp) 
    INTERFACE grid_initialize_cos_lon_v_rewrite
            module procedure grid_initialize_cos_lon_v_rewrite
    END INTERFACE grid_initialize_cos_lon_v_rewrite
    !--- Rewrite the sine of longitudes at the grid points of u (zonal velocity) of 
    !    type REAL(KIND=wp) 
    INTERFACE grid_initialize_sin_lon_u_rewrite
            module procedure grid_initialize_sin_lon_u_rewrite
    END INTERFACE grid_initialize_sin_lon_u_rewrite
    !--- Rewrite the sine of longitudes at the grid points of v (meridional velocity) of 
    !    type REAL(KIND=wp) 
    INTERFACE grid_initialize_sin_lon_v_rewrite
            module procedure grid_initialize_sin_lon_v_rewrite
    END INTERFACE grid_initialize_sin_lon_v_rewrite
    !--- Total depth at u-points               
    INTERFACE grid_depth_add_zeta_u_points
            module procedure grid_depth_add_zeta_u_points
    END INTERFACE grid_depth_add_zeta_u_points
    !--- Total depth at v-points               
    INTERFACE grid_depth_add_zeta_v_points
            module procedure grid_depth_add_zeta_v_points
    END INTERFACE grid_depth_add_zeta_v_points


  !---------------------------------------------------------------------------------------------!

  CONTAINS

    !-------------------------------------------------------------------------------------------!

    SUBROUTINE grid_initialize()

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - initialize of the grid variables                                                      !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none


        !---------------------------------------------------------------------------------------!
        !--- Initialize of the grid variables --------------------------------------------------!

        !--- Cosine and sine of latitudes
        !--- cosine of latitudes at u-points
        cos_lat_u = grid_initialize_cos_lat_u( )
        !--- cosine of latitudes at v-points
        cos_lat_v = grid_initialize_cos_lat_v( )
        !--- sine of latitudes at u-points
        sin_lat_u = grid_initialize_sin_lat_u( )
        !--- sine of latitudes at v-points
        sin_lat_v = grid_initialize_sin_lat_v( )
        !--- inverse cosine of latitudes at u-points
        cos_lat_u_inv = grid_initialize_cos_lat_u_inv( cos_lat_u )
        !--- inverse cosine of latitudes at v-points
        cos_lat_v_inv = grid_initialize_cos_lat_v_inv( cos_lat_v )
        !--- tangens of latitudes at u-points
        tan_lat_u = nice_math_mul( sin_lat_u , cos_lat_u_inv )
        !--- tangens of latitudes at u-points
        tan_lat_v = nice_math_mul( sin_lat_v , cos_lat_v_inv )
        
        !--- Cosine and sine of longitudes
        !--- cosine of longitudes at u-points
        cos_lon_u = grid_initialize_cos_lon_u( )
        !--- cosine of longitudes at v-points
        cos_lon_v = grid_initialize_cos_lon_v( )
        !--- sine of longitudes at u-points
        sin_lon_u = grid_initialize_sin_lon_u( )
        !--- sine of longitudes at v-points
        sin_lon_v = grid_initialize_sin_lon_v( )
        !--- cosine of longitudes at u-points for tidal forcing at diagnostic functions
        acsul = grid_initialize_cos_lon_u_acs( ) 
        !--- cosine of longitudes at v-points for tidal forcing at diagnostic functions
        acsvl = grid_initialize_cos_lon_v_acs( )
        !--- sine of longitudes at u-points for tidal forcing at diagnostic functions
        asnul = grid_initialize_sin_lon_u_asn( ) 
        !--- sine of longitudes at v-points for tidal forcing at diagnostic functions
        asnvl = grid_initialize_sin_lon_v_asn( )
        !--- rewrite the cosine of longitudes at u-points
        cos_lon_u = grid_initialize_cos_lon_u_rewrite( )
        !--- rewrite the cosine of longitudes at v-points
        cos_lon_v = grid_initialize_cos_lon_v_rewrite( )
        !--- rewrite the sine of longitudes at u-points
        sin_lon_u = grid_initialize_sin_lon_u_rewrite( )
        !--- rewrite the sine of longitudes at v-points
        sin_lon_v = grid_initialize_sin_lon_u_rewrite( )

        !--- Cosines and sines for diagnostic functions
        acc = nice_math_square( acsvl )
        ass = nice_math_square( asnvl )
        acs = nice_math_mul( acsvl , asnvl ) 

        !--- Weighting coefficients
        !--- northern weighting coefficients for interpolation of meridional velocities 
        !    on u-points 
        weight_v_north(lats_start_u:lats_end_u) = &
        nice_math_mul                             &
        (                                         &  
         cos_lat_v    (lats_start_v:lats_end_v) , &
         cos_lat_u_inv(lats_start_u:lats_end_u)   &
        )
        IF ( lats_end_v == lats ) weight_v_north(lats_end_u) = 0._wp
        !--- southern weighting coefficients for interpolation of meridional velocities 
        !    on u-points
        weight_v_south(lats_start_v:lats_end_v) = & 
        nice_math_mul                             & 
        (                                         &
         cos_lat_v    (lats_start_v:lats_end_v) , &
         cos_lat_u_inv(lats_start_v:lats_end_v)   &
        )                                                       
        !--- northern weighting coefficients for interpolation of zonal velocities 
        !    on v-points
        weight_u_north(lats_start_v:lats_end_v) = &
        nice_math_mul                             &
        (                                         &
         cos_lat_u    (lats_start_v:lats_end_v) , &
         cos_lat_v_inv(lats_start_v:lats_end_v)   &
        )
        !--- southern weighting coefficients for interpolation of zonal velocities 
        !    on v-points 
        weight_u_south(lats_start_v:lats_end_v) = &
        nice_math_mul                             &
        (                                         &
         cos_lat_u    (lats_start_u:lats_end_u) , &
         cos_lat_v_inv(lats_start_v:lats_end_v)   &
        )
        !--- topography
        depth = 500._wp

        !---------------------------------------------------------------------------------------!

    END SUBROUTINE grid_initialize

    !-------------------------------------------------------------------------------------------!

    FUNCTION grid_initialize_cos_lat_u( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     cosine of latitudes at the grid points of u (zonal velocity) of type REAL(KIND=wp)    !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (lats_start_u:lats_end_u_add_1)  :: grid_initialize_cos_lat_u  
 
        !--- Local variables
        !--- Latitude in radian
        REAL (KIND=wp)                                              :: phi

        !--- Help variables
        !--- Latitudes loop counter
        INTEGER                                                     :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Determine the cosine of latitudes at u-points, u is located at half of delta_y_deg !

        !--- set latitude in radian
        phi = 0.5_wp * delta_y_deg * radian
        DO lat = lats_equator_u + 1 , lats_end_u_add_1
           !--- northerly hemisphere
           grid_initialize_cos_lat_u(lat) = COS( phi )
           !--- southerly hemisphere
           IF ( lats_u+2-lat >= lats_start_u_add_1 ) THEN
                   grid_initialize_cos_lat_u(lats_u+2-lat) = &
                   grid_initialize_cos_lat_u(lat         )
           END IF
           !--- set latitude in radian
           phi =  phi + delta_y_deg * radian
        END DO
        !--- southerly hemisphere
        IF ( lats_u+2-lat >= lats_start_u ) THEN
                DO lat = lats_u+2-lat , lats_start_u , -1 
                   grid_initialize_cos_lat_u(lat) = COS( phi )
                   !--- set latitude in radian
                   phi =  phi + delta_y_deg * radian
                END DO
        END IF        
        
        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_initialize_cos_lat_u

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_initialize_cos_lat_v( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     cosine of latitudes at the grid points of v (meridional velocity) of                  !
    !     type REAL(KIND=wp)                                                                    !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (lats_start_v:lats_end_v)  :: grid_initialize_cos_lat_v  
 
        !--- Local variables
        !--- Latitude in radian
        REAL (KIND=wp)                                        :: phi

        !--- Help variables
        !--- Latitudes loop counter
        INTEGER                                               :: lat


        !---------------------------------------------------------------------------------------!
        !--- Determine the cosine of latitudes at v-points -------------------------------------!

        !--- set latitude in radian
        phi = delta_y_deg * radian
        DO lat = lats_equator_v + 1 , lats_end_v
           !--- northerly hemisphere
           grid_initialize_cos_lat_v(lat) = COS( phi )
           !--- southerly hemisphere
           IF ( lats_v+2-lat >= lats_start_v ) THEN
                   grid_initialize_cos_lat_v(lats_v+2-lat) = &
                   grid_initialize_cos_lat_v(lat         )
           END IF
           !--- set latitude in radian
           phi =  phi + delta_y_deg * radian
        END DO
        !--- southerly hemisphere
        IF ( lats_v+2-lat >= lats_start_v ) THEN
                DO lat = lats_v+2-lat , lats_start_v , -1 
                   grid_initialize_cos_lat_v(lat) = COS( phi )
                   !--- set latitude in radian
                   phi =  phi + delta_y_deg * radian
                END DO
        END IF        
        !--- set cosine of equator
        grid_initialize_cos_lat_v(lats_equator_v) = 1._wp
        !--- set cosine of north and south pole
        IF ( lats_start_v == 0    ) grid_initialize_cos_lat_v(lats_start_v) = 0._wp
        IF ( lats_end_v   == lats ) grid_initialize_cos_lat_v(lats_end_v  ) = 0._wp

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_initialize_cos_lat_v

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_initialize_sin_lat_u( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     sine of latitudes at the grid points of u (zonal velocity) of type REAL(KIND=wp)      !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (lats_start_u:lats_end_u_add_1)  :: grid_initialize_sin_lat_u  
 
        !--- Local variables
        !--- Latitude in radian
        REAL (KIND=wp)                                              :: phi

        !--- Help variables
        !--- Latitudes loop counter
        INTEGER                                                     :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Determine the sine of latitudes at u-points, u is located at half of delta_y_deg --!

        !--- set latitude in radian
        phi = 0.5_wp * delta_y_deg * radian
        DO lat = lats_equator_u + 1 , lats_end_u_add_1
           !--- northerly hemisphere
           grid_initialize_sin_lat_u(lat) = SIN( phi )
           !--- southerly hemisphere
           IF ( lats_u+2-lat >= lats_start_u_add_1 ) THEN
                   grid_initialize_sin_lat_u(lats_u+2-lat) = &
                  -grid_initialize_sin_lat_u(lat         )
           END IF
           !--- set latitude in radian
           phi =  phi + delta_y_deg * radian
        END DO
        !--- southerly hemisphere
        IF ( lats_u+2-lat >= lats_start_u ) THEN
                DO lat = lats_u+2-lat , lats_start_u , -1 
                   grid_initialize_sin_lat_u(lat) = -SIN( phi )
                   !--- set latitude in radian
                   phi =  phi + delta_y_deg * radian
                END DO
        END IF        

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_initialize_sin_lat_u

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_initialize_sin_lat_v( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     sine of latitudes at the grid points of v (meridional velocity) of type REAL(KIND=wp) !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (lats_start_v:lats_end_v)  :: grid_initialize_sin_lat_v  
 
        !--- Local variables
        !--- Latitude in radian
        REAL (KIND=wp)                                        :: phi

        !--- Help variables
        !--- Latitudes loop counter
        INTEGER                                               :: lat


        !---------------------------------------------------------------------------------------!
        !--- Determine the sine of latitudes at v-points ---------------------------------------!

        !--- set latitude in radian
        phi = delta_y_deg * radian
        DO lat = lats_equator_v + 1 , lats_end_v
           !--- northerly hemisphere
           grid_initialize_sin_lat_v(lat) = SIN( phi )
           !--- southerly hemisphere
           IF ( lats_v+2-lat >= lats_start_v ) THEN
                   grid_initialize_sin_lat_v(lats_v+2-lat) = &
                  -grid_initialize_sin_lat_v(lat         )
           END IF
           !--- set latitude in radian
           phi =  phi + delta_y_deg * radian
        END DO
        !--- southerly hemisphere
        IF ( lats_v+2-lat >= lats_start_v ) THEN
                DO lat = lats_v+2-lat , lats_start_v , -1 
                   grid_initialize_sin_lat_v(lat) = -SIN( phi )
                   !--- set latitude in radian
                   phi =  phi + delta_y_deg * radian
                END DO
        END IF        
        !--- set cosine of equator
        grid_initialize_sin_lat_v(lats_equator_v) = 0._wp
        !--- set cosine of north and south pole
        IF ( lats_start_v == 0    ) grid_initialize_sin_lat_v(lats_start_v) = 1._wp
        IF ( lats_end_v   == lats ) grid_initialize_sin_lat_v(lats_end_v  ) = 1._wp

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_initialize_sin_lat_v

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_initialize_cos_lat_u_inv( var )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     inverse cosine of latitudes at the grid points of u (zonal velocity) of               !
    !     type REAL(KIND=wp)                                                                    !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp)               , DIMENSION (lats_start_u:lats_end_u_add_1)  :: grid_initialize_cos_lat_u_inv  

        !--- Function variable
        !--- Cosine of latitudes at u-points
        REAL (KIND=wp) , INTENT (IN) , DIMENSION (lats_start_u:lats_end_u_add_1)  :: var
 
        !--- Help variables
        !--- Latitudes loop counter
        INTEGER                                                                   :: lat


        !---------------------------------------------------------------------------------------!
        !--- Determine the inverse cosine of latitudes at u-points -----------------------------!

        !--- northerly hemisphere
        grid_initialize_cos_lat_u_inv(lats_equator_u+1 : lats_end_u_add_1)   = &
                nice_math_invert( var(lats_equator_u+1 : lats_end_u_add_1) )
        !--- southerly hemisphere
        !    first case: end of northerly calculation is less than start of southerly calculation 
        IF ( lats_u + 2 - lats_end_u_add_1 >= lats_start_u) THEN
                !--- copy symmetric part of the northerly hemisphere
                DO lat = lats_equator_u + 1 , lats_end_u_add_1
                   grid_initialize_cos_lat_u_inv(lats_u+2-lat) = &
                   grid_initialize_cos_lat_u_inv(lat         )
                END DO
                !--- inverse elements of the rest of the southerly hemisphere
                IF ( lats_u+2-lat > lats_start_u) THEN
                        grid_initialize_cos_lat_u_inv(lats_start_u : lats_u+2-lat)   = &
                                nice_math_invert( var(lats_start_u : lats_u+2-lat) )
                END IF
        !--- second case: end of northerly calculation is greater than start of southerly calculation 
        ELSE
                !--- copy symmetric part of the northerly hemisphere
                DO lat = lats_equator_u + 1 , lats_u + 2 - lats_start_u      
                   grid_initialize_cos_lat_u_inv(lats_u+2-lat) = &
                   grid_initialize_cos_lat_u_inv(lat         )
                END DO
        END IF

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_initialize_cos_lat_u_inv

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_initialize_cos_lat_v_inv( var )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     inverse cosine of latitudes at the grid points of v (meridional velocity) of          !
    !     type REAL(KIND=wp)                                                                    !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp)               , DIMENSION (lats_start_v:lats_end_v)  :: grid_initialize_cos_lat_v_inv  

        !--- Function variable
        !--- Cosine of latitudes at u-points
        REAL (KIND=wp) , INTENT (IN) , DIMENSION (lats_start_v:lats_end_v)  :: var
 
        !--- Help variables
        !--- Start of latitudes counting
        INTEGER                                                             :: lats_start 
        !--- End of latitudes counting
        INTEGER                                                             :: lats_end 
        !--- Latitudes loop counter
        INTEGER                                                             :: lat


        !---------------------------------------------------------------------------------------!
        !--- Determine the inverse cosine of latitudes at v-points -----------------------------!

        !--- set start of latitudes counting
        IF ( lats_start_v /= 0 ) THEN
                lats_start = lats_start_v 
        ELSE 
                lats_start = lats_start_v + 1
        END IF
        !--- set end of latitudes counting
        IF ( lats_end_v /= lats ) THEN
                lats_end = lats_end_v
        ELSE
                lats_end = lats_end_v - 1 
        END IF
        !--- northerly hemisphere
        grid_initialize_cos_lat_v_inv(lats_equator_v+1 : lats_end)   = &
                nice_math_invert( var(lats_equator_v+1 : lats_end) )
        !--- southerly hemisphere
        !    first case: end of northerly calculation is less than start of southerly calculation 
        IF ( lats_v + 2 - lats_end >= lats_start) THEN
                !--- copy symmetric part of the northerly hemisphere
                DO lat = lats_equator_v + 1 , lats_end
                   grid_initialize_cos_lat_v_inv(lats_v+2-lat) = &
                   grid_initialize_cos_lat_v_inv(lat         )
                END DO
                !--- inverse elements of the rest of the southerly hemisphere
                IF ( lats_v+2-lat > lats_start) THEN
                        grid_initialize_cos_lat_v_inv(lats_start : lats_v+2-lat) = &
                                nice_math_invert( var(lats_start : lats_v+2-lat) )
                END IF
        !--- second case: end of northerly calculation is greater than start of southerly calculation 
        ELSE
                !--- copy symmetric part of the northerly hemisphere
                DO lat = lats_equator_v + 1 , lats_v + 2 - lats_start      
                   grid_initialize_cos_lat_v_inv(lats_v+2-lat) = & 
                   grid_initialize_cos_lat_v_inv(lat         )
                END DO
        END IF
        !--- set inverse cosine of equator
        grid_initialize_cos_lat_v_inv(lats_equator_v) = 1._wp
        !--- set cosine of north and south pole
        IF ( lats_start_v == 0    ) grid_initialize_cos_lat_v_inv(lats_start_v) = 0._wp
        IF ( lats_end_v   == lats ) grid_initialize_cos_lat_v_inv(lats_end_v  ) = 0._wp

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_initialize_cos_lat_v_inv

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_initialize_cos_lon_u( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     cosine of longitudes at the grid points of u (zonal velocity) of type REAL(KIND=wp)   !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end)  :: grid_initialize_cos_lon_u  
 
        !--- Local variables
        !--- Longitude in radian
        REAL (KIND=wp)                                      :: lam

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                                             :: lon


        !---------------------------------------------------------------------------------------!
        !--- Determine the cosine of longitudes at u-points ------------------------------------!

        !--- set longitude in radian
        lam = delta_x_deg * radian
        !--- first quarter
        DO lon = longs_start , longs_end_quarter - 1
           grid_initialize_cos_lon_u(lon) = COS( lam )
           !--- set latitude in radian
           lam = lam + delta_x_deg * radian
        END DO
        !--- second quarter
        DO lon = longs_start , longs_end_quarter - 1
           grid_initialize_cos_lon_u(longs_end_half-lon) = &
          -grid_initialize_cos_lon_u(lon               )
        END DO
        !--- for 90°
        grid_initialize_cos_lon_u(longs_end_quarter) =  0._wp
        !--- for 180°
        grid_initialize_cos_lon_u(longs_end_half)    = -1._wp
        !--- second hemisphere
        DO lon = longs_end_half + 1 , longs_end - 1
           grid_initialize_cos_lon_u(lon          ) = &
           grid_initialize_cos_lon_u(longs_end-lon)
        END DO
        !--- for 360°
        grid_initialize_cos_lon_u(longs_end) = 1._wp

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_initialize_cos_lon_u

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_initialize_cos_lon_v( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     cosine of longitudes at the grid points of v (meridional velocity) of                 !
    !     type REAL(KIND=wp)                                                                    !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end)  :: grid_initialize_cos_lon_v  
 
        !--- Local variables
        !--- Longitude in radian
        REAL (KIND=wp)                                      :: lam

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                                             :: lon


        !---------------------------------------------------------------------------------------!
        !--- Determine the cosine of longitudes at v-points, v is located at half of delta_x_deg!

        !--- set longitude in radian
        lam = 0.5_wp * delta_x_deg * radian
        !--- first quarter
        DO lon = 1 , longs_end_quarter
           grid_initialize_cos_lon_v(lon) = COS( lam )
           !--- set latitude in radian
           lam = lam + delta_x_deg * radian
        END DO
        !--- second quarter
        DO lon = 1 , longs_end_quarter
           grid_initialize_cos_lon_v(longs_end_half+1-lon) = &
          -grid_initialize_cos_lon_v(lon                 )
        END DO
        !--- second hemisphere
        DO lon = longs_end_half + 1 , longs_end 
           grid_initialize_cos_lon_v(lon            ) = &
           grid_initialize_cos_lon_v(longs_end+1-lon)
        END DO

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_initialize_cos_lon_v

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_initialize_sin_lon_u( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     sine of longitudes at the grid points of u (zonal velocity) of type REAL(KIND=wp)     !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end)  :: grid_initialize_sin_lon_u  
 
        !--- Local variables
        !--- Longitude in radian
        REAL (KIND=wp)                                      :: lam

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                                             :: lon


        !---------------------------------------------------------------------------------------!
        !--- Determine the cosine of longitudes at u-points ------------------------------------!

        !--- set longitude in radian
        lam = delta_x_deg * radian
        !--- first quarter
        DO lon = longs_start , longs_end_quarter - 1
           grid_initialize_sin_lon_u(lon) = SIN( lam )
           !--- set latitude in radian
           lam = lam + delta_x_deg * radian
        END DO
        !--- second quarter
        DO lon = longs_start , longs_end_quarter - 1
           grid_initialize_sin_lon_u(longs_end_half-lon) = &
           grid_initialize_sin_lon_u(lon               )
        END DO
        !--- for 90°
        grid_initialize_sin_lon_u(longs_end_quarter) = 1._wp
        !--- for 180°
        grid_initialize_sin_lon_u(longs_end_half)    = 0._wp
        !--- second hemisphere
        DO lon = longs_end_half + 1 , longs_end - 1
           grid_initialize_sin_lon_u(lon          ) = &
          -grid_initialize_sin_lon_u(longs_end-lon)
        END DO
        !--- for 360°
        grid_initialize_sin_lon_u(longs_end) = 0._wp

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_initialize_sin_lon_u

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_initialize_sin_lon_v( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     sine of longitudes at the grid points of v (meridional velocity) of type REAL(KIND=wp)!
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end)  :: grid_initialize_sin_lon_v  
 
        !--- Local variables
        !--- Longitude in radian
        REAL (KIND=wp)                                      :: lam

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                                             :: lon


        !---------------------------------------------------------------------------------------!
        !--- Determine the sine of longitudes at v-points, v is located at half of delta_x_deg -!

        !--- set longitude in radian
        lam = 0.5_wp * delta_x_deg * radian
        !--- first quarter
        DO lon = longs_start , longs_end_quarter
           grid_initialize_sin_lon_v(lon) = SIN( lam )
           !--- set latitude in radian
           lam = lam + delta_x_deg * radian
        END DO
        !--- second quarter
        DO lon = longs_start , longs_end_quarter
           grid_initialize_sin_lon_v(longs_end_half+1-lon) = &
           grid_initialize_sin_lon_v(lon                 )
        END DO
        !--- second hemisphere
        DO lon = longs_end_half + 1 , longs_end 
           grid_initialize_sin_lon_v(lon            ) = &
          -grid_initialize_sin_lon_v(longs_end+1-lon)
        END DO

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_initialize_sin_lon_v

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_initialize_cos_lon_u_acs( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     cosine of longitudes at the grid points of u (zonal velocity) of                      !
    !     type REAL(KIND=wp) for tidal forcing and diagnostic functions                         !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end)  :: grid_initialize_cos_lon_u_acs  
 
        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                                             :: lon , ish


        !---------------------------------------------------------------------------------------!
        !--- Determine the cosine of longitudes at u-points for tidal forcing and diagnostic ---!
        !    functions                                                                          !

        DO lon = longs_start , longs_end - 20
           ish = lon + 20
           grid_initialize_cos_lon_u_acs(lon) = cos_lon_u(ish)
        END DO
        DO lon = longs_end - 19 , longs_end 
           ish = lon -longs_end + 20
           grid_initialize_cos_lon_u_acs(lon) = cos_lon_u(ish)
        END DO

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_initialize_cos_lon_u_acs

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_initialize_cos_lon_v_acs( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     cosine of longitudes at the grid points of v (meridional velocity) of                 !
    !     type REAL(KIND=wp) for tidal forcing and diagnostic functions                         !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end)  :: grid_initialize_cos_lon_v_acs  
 
        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                                             :: lon , ish


        !---------------------------------------------------------------------------------------!
        !--- Determine the sine of longitudes at v-points for tidal forcing and diagnostic -----!
        !    functions                                                                          !

        DO lon = longs_start , longs_end - 20
           ish = lon + 20
           grid_initialize_cos_lon_v_acs(lon) = cos_lon_v(ish)
        END DO
        DO lon = longs_end - 19 , longs_end 
           ish = lon -longs_end + 20
           grid_initialize_cos_lon_v_acs(lon) = cos_lon_v(ish)
        END DO

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_initialize_cos_lon_v_acs

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_initialize_sin_lon_u_asn( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     sine of longitudes at the grid points of u (zonal velocity) of                        !
    !     type REAL(KIND=wp) for tidal forcing and diagnostic functions                         !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end)  :: grid_initialize_sin_lon_u_asn  
 
        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                                             :: lon , ish


        !---------------------------------------------------------------------------------------!
        !--- Determine the sine of longitudes at u-points for tidal forcing and diagnostic -----!
        !    functions                                                                          !

        DO lon = longs_start , longs_end - 20
           ish = lon + 20
           grid_initialize_sin_lon_u_asn(lon) = sin_lon_u(ish)
        END DO
        DO lon = longs_end - 19 , longs_end 
           ish = lon -longs_end + 20
           grid_initialize_sin_lon_u_asn(lon) = sin_lon_u(ish)
        END DO

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_initialize_sin_lon_u_asn

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_initialize_sin_lon_v_asn( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     sine of longitudes at the grid points of v (meridional velocity) of                   !
    !     type REAL(KIND=wp) for tidal forcing and diagnostic functions                         !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end)  :: grid_initialize_sin_lon_v_asn  
 
        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                                             :: lon , ish


        !---------------------------------------------------------------------------------------!
        !--- Determine the sine of longitudes at v-points for tidal forcing and diagnostic -----!
        !    functions                                                                          !

        DO lon = longs_start , longs_end - 20
           ish = lon + 20
           grid_initialize_sin_lon_v_asn(lon) = sin_lon_v(ish)
        END DO
        DO lon = longs_end - 19 , longs_end 
           ish = lon -longs_end + 20
           grid_initialize_sin_lon_v_asn(lon) = sin_lon_v(ish)
        END DO

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_initialize_sin_lon_v_asn

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_initialize_cos_lon_u_rewrite( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     rewrite the cosine of longitudes at the grid points of u (zonal velocity) of          !
    !     type REAL(KIND=wp)                                                                    !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end)  :: grid_initialize_cos_lon_u_rewrite  
 
        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                                             :: lon


        !---------------------------------------------------------------------------------------!
        !--- Rewrite the cosine of longitudes at u-points                                       !

        DO lon = longs_start , longs_end_half - 10
           grid_initialize_cos_lon_u_rewrite(lon) = acsul(2*lon+20)
        END DO
        DO lon = longs_start , longs_end_half - 10
           grid_initialize_cos_lon_u_rewrite(lon+longs_end_half) = & 
           grid_initialize_cos_lon_u_rewrite(lon               )
        END DO
        DO lon = longs_end_half - 9 , longs_end_half 
           grid_initialize_cos_lon_u_rewrite(lon) = acsul(2*lon+20-longs_end)
        END DO
        DO lon = longs_end_half - 9 , longs_end_half 
           grid_initialize_cos_lon_u_rewrite(lon+longs_end_half) = & 
           grid_initialize_cos_lon_u_rewrite(lon               )
        END DO

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_initialize_cos_lon_u_rewrite

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_initialize_cos_lon_v_rewrite( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     rewrite the sine of longitudes at the grid points of v (meridional velocity) of       !
    !     type REAL(KIND=wp)                                                                    !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end)  :: grid_initialize_cos_lon_v_rewrite  
 
        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                                             :: lon


        !---------------------------------------------------------------------------------------!
        !--- Rewrite the cosine of longitudes at v-points                                       !

        DO lon = longs_start , longs_end_half - 10
           grid_initialize_cos_lon_v_rewrite(lon) = acsvl(2*lon+19)
        END DO
        DO lon = longs_start , longs_end_half - 10
           grid_initialize_cos_lon_v_rewrite(lon+longs_end_half) = & 
           grid_initialize_cos_lon_v_rewrite(lon               )
        END DO
        DO lon = longs_end_half - 9 , longs_end_half 
           grid_initialize_cos_lon_v_rewrite(lon) = acsvl(2*lon+19-longs_end)
        END DO
        DO lon = longs_end_half - 9 , longs_end_half 
           grid_initialize_cos_lon_v_rewrite(lon+longs_end_half) = & 
           grid_initialize_cos_lon_v_rewrite(lon               )
        END DO

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_initialize_cos_lon_v_rewrite

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_initialize_sin_lon_u_rewrite( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     sine of longitudes at the grid points of u (zonal velocity) of                        !
    !     type REAL(KIND=wp) for tidal forcing and diagnostic functions                         !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end)  :: grid_initialize_sin_lon_u_rewrite  
 
        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                                             :: lon


        !---------------------------------------------------------------------------------------!
        !--- Rewrite the sine of longitudes at u-points                                         !

        DO lon = longs_start , longs_end_half - 10
           grid_initialize_sin_lon_u_rewrite(lon) = asnul(2*lon+20)
        END DO
        DO lon = longs_start , longs_end_half - 10
           grid_initialize_sin_lon_u_rewrite(lon+longs_end_half) = & 
           grid_initialize_sin_lon_u_rewrite(lon               )
        END DO
        DO lon = longs_end_half - 9 , longs_end_half 
           grid_initialize_sin_lon_u_rewrite(lon) = asnul(2*lon+20-longs_end)
        END DO
        DO lon = longs_end_half - 9 , longs_end_half 
           grid_initialize_sin_lon_u_rewrite(lon+longs_end_half) = & 
           grid_initialize_sin_lon_u_rewrite(lon               )
        END DO

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_initialize_sin_lon_u_rewrite

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_initialize_sin_lon_v_rewrite( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     rewrite the sine of longitudes at the grid points of v (meridional velocity) of       !
    !     type REAL(KIND=wp)                                                                    !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end)  :: grid_initialize_sin_lon_v_rewrite  
 
        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                                             :: lon


        !---------------------------------------------------------------------------------------!
        !--- Rewrite the sine of longitudes at v-points                                         !

        DO lon = longs_start , longs_end_half - 10
           grid_initialize_sin_lon_v_rewrite(lon) = asnvl(2*lon+19)
        END DO
        DO lon = longs_start , longs_end_half - 10
           grid_initialize_sin_lon_v_rewrite(lon+longs_end_half) = & 
           grid_initialize_sin_lon_v_rewrite(lon               )
        END DO
        DO lon = longs_end_half - 9 , longs_end_half 
           grid_initialize_sin_lon_v_rewrite(lon) = asnvl(2*lon+19-longs_end)
        END DO
        DO lon = longs_end_half - 9 , longs_end_half 
           grid_initialize_sin_lon_v_rewrite(lon+longs_end_half) = & 
           grid_initialize_sin_lon_v_rewrite(lon               )
        END DO

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_initialize_sin_lon_v_rewrite

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_depth_add_zeta_u_points( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     total depth at u-points                                                               !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end , lats_start_u:lats_end_u)            &
        :: grid_depth_add_zeta_u_points

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitutdes loop counter
        INTEGER                       :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate total depth at u-points -------------------------------------------------!

        DO lat = lats_start_u , lats_end_u
           DO lon = longs_start , longs_end
              grid_depth_add_zeta_u_points(lon  ,lat) = &
                        0.5_wp                        * &
                        (   depth_add_zeta(lon  ,lat)   &
                          + depth_add_zeta(lon+1,lat)   &
                        )
           END DO
        END DO 

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_depth_add_zeta_u_points

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION grid_depth_add_zeta_v_points( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     total depth at v-points                                                               !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end , lats_start_v:lats_end_v)            &
        :: grid_depth_add_zeta_v_points

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitutdes loop counter
        INTEGER                       :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate total depth at v-points -------------------------------------------------!

        DO lat = lats_start_v , lats_end_v
           DO lon = longs_start , longs_end
               !--- org
!              grid_depth_add_zeta_v_points(lon,lat) = 0.5_wp                                          &
!                                                * (   weight_u_north(lat) * depth_add_zeta(lon,lat  ) &
!                                                    + weight_u_south(lat) * depth_add_zeta(lon,lat-1) &
!                                                  )
              !--- Fix me: why weighting of the depth in org?
              grid_depth_add_zeta_v_points(lon,lat  ) = &
                        0.5_wp                        * &
                        (   depth_add_zeta(lon,lat  )   &
                          + depth_add_zeta(lon,lat-1)   &
                        )
           END DO
        END DO 

        !---------------------------------------------------------------------------------------!

    END FUNCTION grid_depth_add_zeta_v_points

    !-------------------------------------------------------------------------------------------!

END MODULE mod_grid

