!------------------------------------------------------------------------------------------------!
! Module of meridional velocity                                                                  !
!                                                                                                !
! orginal version from Ulrike Seiler , ulrike.seiler@zmaw.de                                     !
!                                                                                                !
! (c) Petra Nerge 2012 , petra.nerge@informatik.uni-hamburg.de                                   !
!------------------------------------------------------------------------------------------------!
!                                                                                                !
! Description                                                                                    !
!                                                                                                !
!   Module of meridional velocity                                                                !


MODULE mod_merid_velocity

    USE mod_kind
    USE mod_grid
    USE mod_time_stepping     , ONLY : delta_time
    USE mod_earth_quantities
 
    IMPLICIT NONE

    !--- GLOBAL VARIABLES CONTAIN ---------------------------------------------------------------!
    !
    !--- State variables ------------------------------------------------------------------------!
    !--- Field of meridional velocity for v-points
    REAL (KIND=wp) , DIMENSION (longs_start_sub_1:longs_end_add_1 , lats_start_v:lats_end_v_add_1) &
    :: v 
    !--- Field of meridional velocity for u-points
    REAL (KIND=wp) , DIMENSION (longs_start      :longs_end       , lats_start_u:lats_end_u      ) &
    :: v_u_points = 0._wp                                                                        !--- org: fxn         
    !--------------------------------------------------------------------------------------------!

    !--- Coefficients of the meridional momentum equation ---------------------------------------!
    !--- Explicit part of pressure gradient
    !--- org: fyg
    REAL (KIND=wp)                                                                               &
    :: v_pressure_gradient_coef_exp   
    !--- Implicit part of pressure gradient
    REAL (KIND=wp) , DIMENSION (longs_start:longs_end , lats_start_v:lats_end_v )                &
    :: v_pressure_gradient_coef_imp                                                              !--- org: fyn
    !--------------------------------------------------------------------------------------------!


    !--- INTERFACE CONTAIN ----------------------------------------------------------------------!
    !
    !--- Coefficient of explicit part of pressure gradient
    INTERFACE merid_vel_pressure_gradient_coef_exp
            module procedure merid_vel_pressure_gradient_coef_exp
    END INTERFACE merid_vel_pressure_gradient_coef_exp
    !--- Coefficient of implicit part of pressure gradient
    INTERFACE merid_vel_pressure_gradient_coef_imp
            module procedure merid_vel_pressure_gradient_coef_imp
    END INTERFACE merid_vel_pressure_gradient_coef_imp
    !--- Meridional velocity at northernmost latitudes by cross-polar averaging
    INTERFACE merid_vel_boundary_lat
            module procedure merid_vel_boundary_lat
    END INTERFACE merid_vel_boundary_lat
    !--- Meridional velocity for u-points
    INTERFACE merid_vel_u_points
            module procedure merid_vel_u_points
    END INTERFACE merid_vel_u_points
    !--- Meridional velocity change per time step due to coriolis acceleration              
    INTERFACE merid_vel_coriolis
            module procedure merid_vel_coriolis
    END INTERFACE merid_vel_coriolis
    !--- Meridional velocity change per time step due to eddy viscosity related 
    !    to delta(u)-term 
    INTERFACE merid_vel_eddy_du_dlon
            module procedure merid_vel_eddy_du_dlon
    END INTERFACE merid_vel_eddy_du_dlon
    !--- Meridional velocity change per time step due to eddy viscosity related 
    !    to v-term                                          
    INTERFACE merid_vel_eddy_v
            module procedure merid_vel_eddy_v
    END INTERFACE merid_vel_eddy_v
    !--- Meridional velocity change per time step due to eddy viscosity related 
    !    to delta(v)-term
    INTERFACE merid_vel_eddy_dv_dlat
            module procedure merid_vel_eddy_dv_dlat
    END INTERFACE merid_vel_eddy_dv_dlat
    !--- Meridional velocity change per time step due to eddy viscosity related 
    !    to delta^2(u)/delta^2(lat)-term 
    INTERFACE merid_vel_eddy_d2v_d2lat
            module procedure merid_vel_eddy_d2v_d2lat
    END INTERFACE merid_vel_eddy_d2v_d2lat
    !--- Meridional velocity change per time step due to eddy viscosity related 
    !    to delta^2(u)/delta^2(lon)-term 
    INTERFACE merid_vel_eddy_d2v_d2lon
            module procedure merid_vel_eddy_d2v_d2lon
    END INTERFACE merid_vel_eddy_d2v_d2lon
    !--- Meridional velocity change per time step due to advection                                         
    INTERFACE merid_vel_advection
            module procedure merid_vel_advection
    END INTERFACE merid_vel_advection
    !--- Meridional velocity due to explicit part of pressure gradient and 
    !    parametrization of tidal pertubation potential proportional 
    !    to the pressure gradient        
    INTERFACE merid_vel_pressure_gradient_exp
            module procedure merid_vel_pressure_gradient_exp
    END INTERFACE merid_vel_pressure_gradient_exp
    !--- Meridional velocity due to implicit part of pressure gradient and 
    !    parametrization of tidal pertubation potential proportional 
    !    to the pressure gradient        
    INTERFACE merid_vel_pressure_gradient_imp
            module procedure merid_vel_pressure_gradient_imp
    END INTERFACE merid_vel_pressure_gradient_imp
    !--- Meridional velocity due to bottom friction  
    INTERFACE merid_vel_bottom_friction
            module procedure merid_vel_bottom_friction
    END INTERFACE merid_vel_bottom_friction
 

  !---------------------------------------------------------------------------------------------!

  CONTAINS

    !-------------------------------------------------------------------------------------------!

    ELEMENTAL FUNCTION merid_vel_pressure_gradient_coef_exp( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     coefficient of the explicit part of pressure gradient within the meridional           !
    !     momentum/velocity equation:                                                           !
    !     gravity_acceleration(1-lsa_parameter)/(earthradius)*delta(zeta)/delta(lon)            !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp)  :: merid_vel_pressure_gradient_coef_exp
 

        !---------------------------------------------------------------------------------------!
 
        !--- set coefficient: 
        !    gravity_acceleration(1-lsa_parameter)*timestep*1/(earthradius*delta(lon))
        merid_vel_pressure_gradient_coef_exp = (1._wp - lsa_parameter) &
                                             * gravity_acceleration    &
                                             * delta_time              &
                                             * delta_y_inv_half             

        !---------------------------------------------------------------------------------------!

    END FUNCTION merid_vel_pressure_gradient_coef_exp

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION merid_vel_pressure_gradient_coef_imp( friction_function )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     coefficient of the implicit part of pressure gradient within the meridional           ! 
    !     momentum/velocity equation:                                                           !
    !     gravity_acceleration(1-lsa_parameter)/(earthradius)*delta(zeta)/delta(lon) *          !
    !     bottom friction function                                                              !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp)              , DIMENSION (longs_start:longs_end , lats_start_v:lats_end_v ) &
        :: merid_vel_pressure_gradient_coef_imp
 
        !--- Function variables
        !--- Field of meridional bottom friction function 
        REAL (KIND=wp) , INTENT(IN) , DIMENSION (longs_start:longs_end , lats_start_v:lats_end_v ) &
        :: friction_function

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitudes loop counter
        INTEGER                       :: lat 


        !---------------------------------------------------------------------------------------!
 
        !--- set coefficient  
      DO lat = lats_start_v , lats_end_v
           DO lon = longs_start , longs_end
              merid_vel_pressure_gradient_coef_imp(lon , lat) = & 
                                 friction_function(lon , lat) * &
                                 v_pressure_gradient_coef_exp  
           END DO
        END DO

        !---------------------------------------------------------------------------------------!

    END FUNCTION merid_vel_pressure_gradient_coef_imp

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION merid_vel_boundary_lat( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     meridional velocity at northernmost latitudes by cross-polar averaging                !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start_sub_1:longs_end_add_1)  :: merid_vel_boundary_lat

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                                                         :: lon 


        !---------------------------------------------------------------------------------------!
        !--- Calculate j+1-values of meridional velocity at northernmost latitudes by ----------!
        !    cross-polar averaging                                                              !

        !--- first hemisphere
        DO lon = longs_start , longs_end_half
           merid_vel_boundary_lat(lon                            ) = &
                            0.5_wp                                 * &
                            (   v(lon                , lats_end_v)   &
                              - v(lon+longs_end_half , lats_end_v)   &
                            )
        END DO
        !--- second hemisphere
        DO lon = longs_end_half+1 , longs_end
           merid_vel_boundary_lat(lon                            ) = & 
                            0.5_wp                                 * &
                            (   v(lon                , lats_end_v)   &
                              - v(lon-longs_end_half , lats_end_v)   &
                            )
        END DO
        !--- cyclic boundary condition for longitudes
        merid_vel_boundary_lat(longs_start_sub_1) = &
        merid_vel_boundary_lat(longs_end  )

        merid_vel_boundary_lat(longs_end_add_1  ) = &
        merid_vel_boundary_lat(longs_start) 

        !---------------------------------------------------------------------------------------!

    END FUNCTION merid_vel_boundary_lat

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION merid_vel_u_points( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     meridional velocity for u-points                                                      !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end , lats_start_u:lats_end_u)            &
        :: merid_vel_u_points

        !--- Help variables
        !--- Start latitude for loop
        INTEGER                       :: lats_start
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitudes loop counter
        INTEGER                       :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate meridional velocity for u-points ----------------------------------------!

        !--- southernmost latitude
        IF ( lats_start_u < lats_start_v ) THEN
           DO lon = longs_start , longs_end
              merid_vel_u_points(lon   , lats_start_u  ) = &
                      0.25_wp                            * &
                      (    (   v(lon+1 , lats_start_u+1)   &
                             + v(lon   , lats_start_u+1)   &
                           )                               &
                        * weight_v_north(lats_start_u  )   & 
                      )
           END DO
           lats_start = lats_start_v
        ELSE 
           lats_start = lats_start_u
        END IF
        !--- remaining latitudes
        DO lat = lats_start , lats_end_u
           DO lon = longs_start , longs_end
              merid_vel_u_points(lon   , lat  ) = & 
                      0.25_wp                   * &
                      (    (   v(lon+1 , lat+1)   &
                             + v(lon   , lat+1)   &
                           )                      &
                        * weight_v_north(lat  )   &
                        +  (   v(lon+1 , lat  )   & 
                             + v(lon   , lat  )   &
                           )                      &
                        * weight_v_south(lat  )   & 
                      )
           END DO
        END DO 

        !---------------------------------------------------------------------------------------!

    END FUNCTION merid_vel_u_points

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION merid_vel_coriolis( zonal_vel )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     meridional momentum due to coriolis acceleration                                      !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp)              , DIMENSION (longs_start:longs_end , lats_start_v:lats_end_v) &
        :: merid_vel_coriolis

        !--- Function variables
        !--- Field of zonal velocity for v-points
        REAL (KIND=wp) , INTENT(IN) , DIMENSION (longs_start:longs_end , lats_start_v:lats_end_v) &
        :: zonal_vel

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitudes loop counter
        INTEGER                       :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate meridional momentum due to coriolis acceleration ------------------------!

        DO lat = lats_start_v , lats_end_v
           DO lon = longs_start , longs_end
              merid_vel_coriolis (lon , lat) = &
              coriolis_operator_v(      lat) * &
                        zonal_vel(lon , lat)
           END DO
        END DO 

        !---------------------------------------------------------------------------------------!

    END FUNCTION merid_vel_coriolis

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION merid_vel_eddy_du_dlon( zonal_vel )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     meridional momentum due to eddy viscosity related to                                  !
    !     u-term = -2*tan(lat)*delta(u)/delta(lon)*1/earthradius                                !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp)              , DIMENSION (      longs_start:longs_end , lats_start_v:lats_end_v) &
        :: merid_vel_eddy_du_dlon

        !--- Function variables
        !--- Field of zonal velocity for u-points 
        REAL (KIND=wp) , INTENT(IN) , DIMENSION (longs_start_sub_1:longs_end , lats_start_u:lats_end_v) &
        :: zonal_vel

        !--- Local variables
        REAL (KIND=wp)                :: coefficient    
        REAL (KIND=wp)                :: constant    

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitudes loop counter
        INTEGER                       :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate meridional momentum due to horizontal momentum diffusion ----------------!

        !--- set constant 1/delta(lon)*1/earthradius
        constant = delta_x_inv * radius_inv
        DO lat = lats_start_v , lats_end_v
           !--- set coefficient -2*tan(lat)*1/delta(lon)*1/earthradius
           coefficient = tan_lat_v(lat) * constant 
           !--- calculate meridional momentum  
           DO lon = longs_start , longs_end
              merid_vel_eddy_du_dlon(lon   , lat  ) = &
                   coefficient                      * &
                   (   (   zonal_vel(lon   , lat  )   &
                         - zonal_vel(lon-1 , lat  )   &
                       )                              &
                     * weight_u_north(       lat  )   & 
                   )                                + & 
                   coefficient                      * & 
                   (   (   zonal_vel(lon   , lat-1)   &
                         - zonal_vel(lon-1 , lat-1)   &
                       )                              &
                     * weight_u_south(       lat  )   &
                   )                                             
           END DO
        END DO 

        !---------------------------------------------------------------------------------------!

    END FUNCTION merid_vel_eddy_du_dlon

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION merid_vel_eddy_v( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     meridional momentum due to eddy viscosity related to                                  !
    !     v-term = -(1+tan^2(lat))*v*1/earthradius^2                                            !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end , lats_start_v:lats_end_v)            &
        :: merid_vel_eddy_v

        !--- Local variables
        REAL (KIND=wp)                :: coefficient    
        REAL (KIND=wp)                :: constant    

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitudes loop counter
        INTEGER                       :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate meridional momentum due to horizontal momentum diffusion ----------------!

        !--- set constant 1/earthradius^2
        constant = radius_inv * radius_inv
        DO lat = lats_start_v , lats_end_v
           !--- set coefficient -(1+tan^2(lat))*1/earthradius^2
           coefficient = -(1._wp + tan_lat_v(lat) * tan_lat_v(lat)) * constant 
           !--- calculate meridional momentum 
           DO lon = longs_start , longs_end
              merid_vel_eddy_v(lon , lat) = &
               coefficient * v(lon , lat)    
           END DO
        END DO 

        !---------------------------------------------------------------------------------------!

    END FUNCTION merid_vel_eddy_v

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION merid_vel_eddy_dv_dlat( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     meridional momentum due to eddy viscosity related to                                  !
    !     delta(v)-term = -tan(lat)*delta(v)/delta(lat)*1/earthradius                           !
    !     by central differences method                                                         !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end , lats_start_v:lats_end_v)            &
        :: merid_vel_eddy_dv_dlat

        !--- Local variables
        REAL (KIND=wp)                :: coefficient    
        REAL (KIND=wp)                :: constant    

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitudes loop counter
        INTEGER                       :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate meridional momentum due to horizontal momentum diffusion ----------------!

        !--- Calculate of first latitude (lats_start_v) by forward differences method
        !--- set coefficient -tan(lat)*1/delta(lat)*1/earthradius 
        coefficient = -tan_lat_v(lats_start_v) * delta_y_inv * radius_inv
        !--- calculate meridional momentum 
        DO lon = longs_start , longs_end
           merid_vel_eddy_dv_dlat(lon , lats_start_v      ) = &
                            coefficient                     * &
                            (   v(lon , lats_start_v_add_1)   & 
                              - v(lon , lats_start_v      )   &
                            )    
        END DO 
        !--- Calculate of remaining latitudes by central differences method
        !--- set constant 1/(2*delta(lat))*1/earthradius
        constant = delta_y_inv_half * radius_inv
        DO lat = lats_start_v_add_1 , lats_end_v
           !--- set coefficient -tan(lat)*1/(2*delta(lat))*1/earthradius 
           coefficient = -tan_lat_v(lat) * constant 
           !--- calculate meridional momentum 
           DO lon = longs_start , longs_end
              merid_vel_eddy_dv_dlat(lon , lat  ) = &
                               coefficient        * &
                               (   v(lon , lat+1)   &
                                 - v(lon , lat-1)   &
                               )    
           END DO
        END DO 

        !---------------------------------------------------------------------------------------!

    END FUNCTION merid_vel_eddy_dv_dlat

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION merid_vel_eddy_d2v_d2lat( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     meridional momentum due to eddy viscosity related to                                  !
    !     delta^2(v)-term = delta^2(v)/delta(lat)^2                                             !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end , lats_start_v:lats_end_v)            &
        :: merid_vel_eddy_d2v_d2lat

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitudes loop counter
        INTEGER                       :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate meridional momentum due to horizontal momentum diffusion ----------------!

        !--- Calculate of first latitude (lats_start_v) 
        DO lon = longs_start , longs_end
           merid_vel_eddy_d2v_d2lat(lon , lats_start_v      ) = &
                      delta_y_inv_square                      * &
                      (           v(lon , lats_start_v_add_1)   &
                        - 2._wp * v(lon , lats_start_v      )   & 
                      )    
        END DO 
        !--- Calculate of remaining latitudes by central differences method
        DO lat = lats_start_v_add_1 , lats_end_v
           DO lon = longs_start , longs_end
              merid_vel_eddy_d2v_d2lat(lon , lat  ) = &
                         delta_y_inv_square         * &   
                         (           v(lon , lat+1)   & 
                           +         v(lon , lat-1)   &
                           - 2._wp * v(lon , lat  )   &
                         )    
           END DO
        END DO 

        !---------------------------------------------------------------------------------------!

    END FUNCTION merid_vel_eddy_d2v_d2lat

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION merid_vel_eddy_d2v_d2lon( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     meridional momentum due to eddy viscosity related to                                  !
    !     delta^2(v)-term = 1/cos(lat)*delta^2(v)/delta(lon)^2                                  !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end , lats_start_v:lats_end_v)            &
        :: merid_vel_eddy_d2v_d2lon

        !--- Local variables
        REAL (KIND=wp)                :: coefficient    

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitudes loop counter
        INTEGER                       :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate meridional momentum due to horizontal momentum diffusion ----------------!

        !--- Calculate of remaining latitudes by central differences method
        DO lat = lats_start_v , lats_end_v
           !--- set coefficient 1/cos(lat)*1/delta(lon)^2 
           coefficient = cos_lat_v_inv(lat) * delta_x_inv_square 
           !--- calculate meridional momentum 
           DO lon = longs_start , longs_end
              merid_vel_eddy_d2v_d2lon(lon   , lat) = & 
                         coefficient                * &   
                         (           v(lon+1 , lat)   & 
                           +         v(lon-1 , lat)   &
                           - 2._wp * v(lon   , lat)   &
                         )    
           END DO
        END DO 

        !---------------------------------------------------------------------------------------!

    END FUNCTION merid_vel_eddy_d2v_d2lon

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION merid_vel_advection( zonal_vel )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     meridional momentum due to advective zonal velocity related term                      !
    !     u/(earthradius*cos(lat))*delta(v)/delta(lon)                                          !
    !     and                                                                                   !
    !     meridional momentum due to advective meridional velocity related term                 !
    !     v/(earthradius*delta(lat))*delta(v)/delta(lat)                                        !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp)              , DIMENSION (longs_start:longs_end , lats_start_v:lats_end_v) &
        :: merid_vel_advection

        !--- Function variables
        !--- Field of zonal velocity for v-points
        REAL (KIND=wp) , INTENT(IN) , DIMENSION (longs_start:longs_end , lats_start_v:lats_end_v) &
        :: zonal_vel

        !--- Local variables
        REAL (KIND=wp)                :: coefficient    

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitudes loop counter
        INTEGER                       :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate meridional momentum due to advection ------------------------------------!

        !--- Calculate zonal velocity related term 
        !    u/(earthradius*cos(lat))*delta(v)/delta(lon)
        DO lat = lats_start_v , lats_end_v
           !--- set coefficient 1/(earthradius*cos(lat))*1/delta(lon) 
           coefficient = cos_lat_v_inv(lat) * delta_x_inv_half 
           !--- calculate meridional momentum 
           DO lon = longs_start , longs_end
              merid_vel_advection(lon   , lat) = &
                        coefficient            * &
                        zonal_vel(lon   , lat) * &
                        (       v(lon+1 , lat)   & 
                              - v(lon-1 , lat)   &
                        )    
           END DO
        END DO 

        !--- Calculate meridional velocity related term 
        !    v/(earthradius*delta(lat))*delta(v)/delta(lat)
        !--- calculate of first latitude (lats_start_v) by forward differences method
        DO lon = longs_start , longs_end
           merid_vel_advection(lon , lats_start_v      ) = &
           merid_vel_advection(lon , lats_start_v      ) + &
                       (     v(lon , lats_start_v      ) * &
                             delta_y_inv                 * &
                         (   v(lon , lats_start_v_add_1)   &
                           - v(lon , lats_start_v      )   & 
                         )                                 &
                       )
        END DO 
        !--- Calculate of remaining latitudes by central differences method
        DO lat = lats_start_v_add_1 , lats_end_v
           !--- calculate meridional momentum
           DO lon = longs_start , longs_end
              merid_vel_advection(lon , lat  ) = &
              merid_vel_advection(lon , lat  ) + &
                          (     v(lon , lat  ) * &
                              delta_y_inv_half * &
                            (   v(lon , lat+1)   & 
                              - v(lon , lat-1)   &
                            )                    &
                          )
           END DO
        END DO 

        !---------------------------------------------------------------------------------------!

    END FUNCTION merid_vel_advection

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION merid_vel_pressure_gradient_exp( elevation )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     meridional velocity due to explicit part of pressure gradient and parametrization of  !
    !     tidal pertubation potential proportional to the pressure gradient                     !
    !     gravity_acceleration(1-lsa_parameter)/(earthradius)*delta(zeta)/delta(lon)            !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp)              , DIMENSION (longs_start:longs_end , lats_start_v:lats_end_v) &
        :: merid_vel_pressure_gradient_exp

        !--- Function variables
        !--- Field of sea surface elevation 
        REAL (KIND=wp) , INTENT(IN) , DIMENSION (longs_start:longs_end , lats_start_u:lats_end_v) &
        :: elevation

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitudes loop counter
        INTEGER                       :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate meridional velocity due to explicit part of pressure gradient and -------!
        !    parametrization of tidal pertubation potential proportional to the                 !
        !    pressure gradient                                                                  !
  
        !--- calculate meridional velocity 
        DO lat = lats_start_v , lats_end_v
           DO lon = longs_start , longs_end
              merid_vel_pressure_gradient_exp(lon , lat  ) = &
                              v_pressure_gradient_coef_exp * &
                                (   elevation(lon , lat  )   & 
                                  - elevation(lon , lat-1)   &
                                )    
           END DO
        END DO 
 
        !---------------------------------------------------------------------------------------!

    END FUNCTION merid_vel_pressure_gradient_exp

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION merid_vel_pressure_gradient_imp( elevation )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     meridional velocity due to implicit part of pressure gradient and parametrization of  !
    !     tidal pertubation potential proportional to the pressure gradient                     !
    !     gravity_acceleration(1-lsa_parameter)/(earthradius)*delta(zeta)/delta(lon)            !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp)              , DIMENSION (longs_start:longs_end , lats_start_v:lats_end_v) &
        :: merid_vel_pressure_gradient_imp

        !--- Function variables
        !--- Field of sea surface elevation 
        REAL (KIND=wp) , INTENT(IN) , DIMENSION (longs_start:longs_end , lats_start_u:lats_end_v) &
        :: elevation

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitudes loop counter
        INTEGER                       :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate meridional velocity due to implicit part of pressure gradient and -------!
        !    parametrization of tidal pertubation potential proportional to the                 !
        !    pressure gradient                                                                  !
  
        !--- calculate meridional velocity 
        DO lat = lats_start_v , lats_end_v
            DO lon = longs_start , longs_end
              merid_vel_pressure_gradient_imp(lon , lat  ) = &
                 v_pressure_gradient_coef_imp(lon , lat  ) * &
                                (   elevation(lon , lat  )   & 
                                  - elevation(lon , lat-1)   &
                                )    
           END DO
        END DO 
 
        !---------------------------------------------------------------------------------------!

    END FUNCTION merid_vel_pressure_gradient_imp

    !-------------------------------------------------------------------------------------------!

    ELEMENTAL FUNCTION merid_vel_bottom_friction( merid_vel , zonal_vel , total_depth )         !--- org: fyn

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     meridional velocity due to bottom friction                                            !
    !     (bottom friction coefficient)/(total depth)*v*sqrt(u^2+v^2)                           !
    !     expressed by the function                                                             !
    !     (total depth)/(total depth+(bottom friction coefficient)*timestep*sqrt(u^2+v^1)       !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp)                                  :: merid_vel_bottom_friction

        !--- Function variables
        !--- Field of meridional velocity for v-points
        REAL (KIND=wp) , INTENT (IN)                    :: merid_vel
        !--- Field of zonal velocity for v-points
        REAL (KIND=wp) , INTENT (IN)                    :: zonal_vel
        !--- Field of total depth at v-points
        REAL (KIND=wp) , INTENT (IN)                    :: total_depth

        !--- Local variables
        !--- Cofficient of bottom friction * time step
        REAL (KIND=wp)                                  :: coefficient    


        !---------------------------------------------------------------------------------------!
        !--- Calculate meridional velocity due to bottom friction ------------------------------!

        !--- set coefficient
        coefficient = bottom_friction_coef * delta_time
        !--- calculate u^2+v^2
        merid_vel_bottom_friction = zonal_vel * zonal_vel  &
                                  + merid_vel * merid_vel
        !--- calculate sqrt(u^2+v^2)
        merid_vel_bottom_friction = SQRT( merid_vel_bottom_friction )
        !--- calculate bottom friction function
        merid_vel_bottom_friction     = &
                          total_depth / &
        (                 total_depth   &
          +               coefficient   &
          * merid_vel_bottom_friction   &
        )

        !---------------------------------------------------------------------------------------!

    END FUNCTION merid_vel_bottom_friction

    !-------------------------------------------------------------------------------------------!

END MODULE mod_merid_velocity


