!-----------------------------------------------------------------------------------------------!
! Module for nice to have routines                                                              !
!                                                                                               ! 
! (c) Petra Nerge 2012, petra.nerge@informatik.uni-hamburg.de                                   !  
!-----------------------------------------------------------------------------------------------!
!                                                                                               ! 
!   Description                                                                                 ! 
!                                                                                               !
!   This module contains subroutines to easy often used intrinsic procedures.                   ! 
!                                                                                               ! 
!   Interface contain                                                                           ! 
!   - nice_allocate:                                                                            !
!     - allocates arrays of standard data types and initialize them                             !
!                                                                                               !   
!-----------------------------------------------------------------------------------------------!

MODULE mod_nice 

    USE mod_kind

    IMPLICIT none
    !--- Unit number of write error messages
    INTEGER , PARAMETER :: nice_eunit = 0

    INTERFACE nice_allocate
            MODULE PROCEDURE nice_allocate_int_1d
            MODULE PROCEDURE nice_allocate_int_2d
            MODULE PROCEDURE nice_allocate_float_1d
            MODULE PROCEDURE nice_allocate_float_2d
            MODULE PROCEDURE nice_allocate_float_3d
            MODULE PROCEDURE nice_allocate_float_4d
            MODULE PROCEDURE nice_allocate_double_1d
            MODULE PROCEDURE nice_allocate_double_2d
            MODULE PROCEDURE nice_allocate_double_4d
            MODULE PROCEDURE nice_allocate_string_1d
            MODULE PROCEDURE nice_allocate_string_2d
            MODULE PROCEDURE nice_allocate_logical_1d
    END INTERFACE nice_allocate

    INTERFACE nice_allocate_lu
            MODULE PROCEDURE nice_allocate_lu_int_1d
            MODULE PROCEDURE nice_allocate_lu_float_1d
            MODULE PROCEDURE nice_allocate_lu_double_1d
    END INTERFACE nice_allocate_lu


  !---------------------------------------------------------------------------------------------!

  CONTAINS

    !-------------------------------------------------------------------------------------------!

    INTEGER FUNCTION nice_allocate_int_1d( var , lenDim1 )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - allocates integer array of rank one by length lenDim1                                 ! 
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - allocatable integer array with one rank                                             !  
    !     - dimension length of the array                                                       ! 
    !                                                                                           !  
    !   - Output                                                                                !  
    !     - allocated integer array of rank one and of length lenDim1                           ! 
    !     - initialized array by 0                                                              ! 
    !                                                                                           !
    !   - Return value                                                                          !      
    !     -   0 : allocate o.k.                                                                 !
    !     - /=0 : allocate error: not enough memory                                             !  
    !     -   2 : allocate error: length of dimensions is less than ONE                         ! 
    !     -   3 : allocate error: variable already allocated                                    !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------! 

        !--- Function Variables
        !--- Array with one rank of type integer
        INTEGER (KIND=i4)          , INTENT (inout) , ALLOCATABLE , DIMENSION(:)  :: var 
        !--- Upper Bound of the array 
        INTEGER (KIND=i4)          , INTENT (in)                                  :: lenDim1

        !--- Help Variables
        INTEGER                                                                   :: ierror                     

        
        !---------------------------------------------------------------------------------------!
        !--- Allocate integer array of one rank and of length lenDim1 and initialize by O ------! 
        
        IF ( .NOT. ALLOCATED( var ) ) THEN
                !--- Get lower Bound of the array
                IF ( lenDim1 > 0 ) THEN
                        ALLOCATE ( var( 1 : lenDim1 ) , STAT = nice_allocate_int_1d )
                        IF ( nice_allocate_int_1d /= 0 ) THEN
                                WRITE(nice_eunit, *, IOSTAT = ierror)                  &   
                                         "nice_allocate_int_1d :: Allcoate ERROR = " , & 
                                         nice_allocate_int_1d  
                                RETURN 
                        END IF
                        var = 0_i4
                ELSE
                        nice_allocate_int_1d = 2
                        WRITE(nice_eunit, *, IOSTAT = ierror)                               & 
                                 "nice_allocate_int_1d :: ERROR dimension less than ONE " , &
                                 "length of dimension 1 = ", lenDim1
                        RETURN 
                END IF
        ELSE
                nice_allocate_int_1d = 3
                WRITE(nice_eunit, *, IOSTAT = ierror)                               &
                         "nice_allocate_int_1d :: ERROR variable already allocated"
                RETURN 
        END IF
        
    END FUNCTION nice_allocate_int_1d

    !-------------------------------------------------------------------------------------------!

    INTEGER FUNCTION nice_allocate_int_2d( var , lenDim1 , lenDim2 )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - allocates integer array of rank two by length lenDim1 and lenDim2                     ! 
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - allocatable integer array of rank twonk                                             !  
    !     - first dimension length of the array                                                 ! 
    !     - second dimension length of the array                                                ! 
    !                                                                                           !  
    !   - Output                                                                                !  
    !     - allocated integer array of rank two and of length lenDim1 and lenDim2               ! 
    !     - initialized array by 0                                                              ! 
    !                                                                                           !
    !   - Return value                                                                          !      
    !     -   0 : allocate o.k.                                                                 !
    !     - /=0 : allocate error: not enough memory                                             !  
    !     -   2 : allocate error: length of dimensions is less than ONE                         ! 
    !     -   3 : allocate error: variable already allocated                                    !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------! 

        !--- Function Variables
        !--- Array with two dimensions of type integer
        INTEGER (KIND=i4)          , INTENT (inout), ALLOCATABLE, DIMENSION(:,:)  :: var 
        !--- Dimensions length
        INTEGER (KIND=i4)          , INTENT (in)                                  :: lenDim1
        !--- Dimensions length
        INTEGER (KIND=i4)          , INTENT (in)                                  :: lenDim2

        !--- Help variables
        INTEGER                                                                   :: ierror                     
       

        !---------------------------------------------------------------------------------------!
        !--- Allocate integer array of rank two and of length lenDim1 and lenDim2 and ----------!
        !    initialize by O

        IF ( .NOT. ALLOCATED( var ) ) THEN
                IF ( lenDim1 > 0 .AND. &
                     lenDim2 > 0       &
                   ) THEN
                        ALLOCATE ( var( 1 : lenDim1 , &
                                        1 : lenDim2     )           , &
                                        STAT = nice_allocate_int_2d   &
                                 )
                        IF ( nice_allocate_int_2d /= 0 ) THEN
                                WRITE(nice_eunit, *, IOSTAT = ierror)                  &   
                                         "nice_allocate_int_2d :: Allocate ERROR = " , &  
                                          nice_allocate_int_2d 
                                RETURN 
                        END IF
                        var = 0_i4
                ELSE
                        nice_allocate_int_2d = 2
                        WRITE(nice_eunit, *, IOSTAT = ierror)                                & 
                                 "nice_allocate_int_2d :: ERROR dimensions less than ONE " , &
                                 "length of dimension 1 = "  , lenDim1                     , &
                                 " lenght of dimension 2 = " , lenDim2
                        RETURN 
                END IF
        ELSE
                nice_allocate_int_2d = 3
                WRITE(nice_eunit, *, IOSTAT = ierror)                               &  
                         "nice_allocate_int_2d :: ERROR variable already allocated"
                RETURN 
        END IF
       
    END FUNCTION nice_allocate_int_2d

    !-------------------------------------------------------------------------------------------!

    INTEGER FUNCTION nice_allocate_float_1d( var , lenDim1 )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - allocates and initialize float array of rank and of dimension length lenDim1          ! 
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - allocatable float array with one rank                                               !  
    !     - dimension length of the array                                                       ! 
    !                                                                                           !  
    !   - Output                                                                                !  
    !     - allocated float array of rank one and of dimension length lenDim1                   ! 
    !     - initialized array by 0                                                              ! 
    !                                                                                           !
    !   - Return value                                                                          !      
    !     -   0 : allocate o.k.                                                                 !
    !     - /=0 : allocate error: not enough memory                                             !  
    !     -   2 : allocate error: length of dimensions is less than ONE                         ! 
    !     -   3 : allocate error: variable already allocated                                    ! 
    !                                                                                           !
    !-------------------------------------------------------------------------------------------! 
       
        !--- Function Variables
        !--- Array with four dimensions of type float
        REAL (KIND=sp)             , INTENT (inout), ALLOCATABLE, DIMENSION(:)    :: var 
        !--- Upper Bound of the array 
        INTEGER (KIND=i4)          , INTENT (in)                                  :: lenDim1

        !--- Help variables
        INTEGER                                                                   :: ierror                  

        
        !---------------------------------------------------------------------------------------!
        !--- Allocate float array of one rank and of size lenDim1 and initialize ---------------!

        IF ( .NOT. ALLOCATED( var ) ) THEN
                IF ( lenDim1  > 0 ) THEN
                        ALLOCATE ( var( 1 : lenDim1 ) , STAT = nice_allocate_float_1d )
                        IF ( nice_allocate_float_1d /= 0 ) THEN
                                WRITE(nice_eunit, *, IOSTAT = ierror)                    & 
                                         "nice_allocate_float_1d :: Allocate ERROR = " , &
                                         nice_allocate_float_1d  
                                RETURN 
                        END IF
                        var = 0._sp
                ELSE
                        nice_allocate_float_1d = 2
                        WRITE(nice_eunit, *, IOSTAT = ierror)                                  &   
                                 "nice_allocate_float_1d :: ERROR dimensions less than ONE " , &
                                 "length of dimension 1 = ", lenDim1
                        RETURN 
                END IF
        ELSE
                nice_allocate_float_1d = 3
                WRITE(nice_eunit, *, IOSTAT = ierror)                                 & 
                         "nice_allocate_float_1d :: ERROR variable already allocated"
                RETURN 
        END IF
        
    END FUNCTION nice_allocate_float_1d

    !-------------------------------------------------------------------------------------------!

    INTEGER FUNCTION nice_allocate_float_2d( var , lenDim1 , lenDim2 )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - allocates float array of rank two by length lenDim1 and lenDim2                       ! 
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - allocatable float array of rank two                                                 !  
    !     - first dimension length of the array                                                 ! 
    !     - second dimension length of the array                                                ! 
    !                                                                                           !  
    !   - Output                                                                                !  
    !     - allocated float array of rank two and of length lenDim1 and lenDim2                 ! 
    !     - initialized array by 0                                                              ! 
    !                                                                                           !
    !   - Return value                                                                          !      
    !     -   0 : allocate o.k.                                                                 !
    !     - /=0 : allocate error: not enough memory                                             !  
    !     -   2 : allocate error: length of dimensions is less than ONE                         ! 
    !     -   3 : allocate error: variable already allocated                                    !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        !--- Function Variables
        !--- Array with four dimensions of type float
        REAL (KIND=sp)         , INTENT (inout), ALLOCATABLE, DIMENSION(:,:)      :: var 
        !--- Dimensions length
        INTEGER (KIND=i4)      , INTENT (in)                                      :: lenDim1
        !--- Dimensions length
        INTEGER (KIND=i4)      , INTENT (in)                                      :: lenDim2

        !--- Help variables
        INTEGER                                                                   :: ierror                  

        
        !---------------------------------------------------------------------------------------!
        !--- Allocate float array of rank two and of length lenDim1 and lenDim2 and   ----------!
        !    initialize by O

        IF ( .NOT. ALLOCATED( var ) ) THEN
                IF ( lenDim1 > 0 .AND. &
                     lenDim2 > 0       &
                   ) THEN
                        ALLOCATE ( var( 1 : lenDim1 , &
                                        1 : lenDim2     )             , &
                                        STAT = nice_allocate_float_2d   &
                                 )
                        IF ( nice_allocate_float_2d /= 0 ) THEN
                                WRITE(nice_eunit, *, IOSTAT = ierror)                    &  
                                         "nice_allocate_float_2d :: Allocate ERROR = " , &  
                                         nice_allocate_float_2d 
                                RETURN 
                        END IF
                        var = 0._sp
                ELSE
                        nice_allocate_float_2d = 2
                        WRITE(nice_eunit, *, IOSTAT = ierror)                                  &     
                                 "nice_allocate_float_2d :: ERROR dimensions less than ONE " , &
                                 "length of dimension 1 = "  , lenDim1                       , &
                                 " lenght of dimension 2 = " , lenDim2
                        RETURN 
                END IF
        ELSE
                nice_allocate_float_2d = 3
                WRITE(nice_eunit, *, IOSTAT = ierror)                                 & 
                         "nice_allocate_float_2d :: ERROR variable already allocated"
                RETURN 
        END IF
        
    END FUNCTION nice_allocate_float_2d

    !-------------------------------------------------------------------------------------------!

    INTEGER FUNCTION nice_allocate_float_3d( var , lenDim1 , lenDim2 , lenDim3 )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - allocates float array of rank three by length lenDim1, lenDim2, and lenDim3           ! 
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - allocatable float array of rank three                                               !  
    !     - first dimension length of the array                                                 ! 
    !     - second dimension length of the array                                                ! 
    !     - third dimension length of the array                                                 ! 
    !                                                                                           !  
    !   - Output                                                                                !  
    !     - allocated float array of rank three and of length lenDim1, lenDim2, and lenDim3     ! 
    !     - initialized array by 0                                                              ! 
    !                                                                                           !
    !   - Return value                                                                          !      
    !     -   0 : allocate o.k.                                                                 !
    !     - /=0 : allocate error: not enough memory                                             !  
    !     -   2 : allocate error: length of dimensions is less than ONE                         ! 
    !     -   3 : allocate error: variable already allocated                                    !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        !--- Function Variables
        !--- Array with four dimensions of type float
        REAL (KIND=sp)         , INTENT (inout), ALLOCATABLE, DIMENSION(:,:,:)    :: var 
        !--- Dimensions length
        INTEGER (KIND=i4)      , INTENT (in)                                      :: lenDim1
        !--- Dimensions length
        INTEGER (KIND=i4)      , INTENT (in)                                      :: lenDim2
        !--- Dimensions length
        INTEGER (KIND=i4)      , INTENT (in)                                      :: lenDim3

        !--- Help variables
        INTEGER                                                                   :: ierror                  

        
        !---------------------------------------------------------------------------------------!
        !--- Allocate float array of rank three and of length lenDim1, lenDim2, and lenDim3 and !
        !    initialize by O

        IF ( .NOT. ALLOCATED( var ) ) THEN
                IF ( lenDim1 > 0 .AND. &
                     lenDim2 > 0 .AND. &
                     lenDim3 > 0       &
                   ) THEN
                        ALLOCATE ( var( 1 : lenDim1 , &
                                        1 : lenDim2 , &
                                        1 : lenDim3   &
                                      )                               , &
                                        STAT = nice_allocate_float_3d   &
                                 )
                        IF ( nice_allocate_float_3d /= 0 ) THEN
                                WRITE(nice_eunit, *, IOSTAT = ierror)                    &   
                                         "nice_allocate_float_3d :: Allocate ERROR = " , &  
                                         nice_allocate_float_3d 
                                RETURN 
                        END IF
                        var = 0._sp
                ELSE
                        nice_allocate_float_3d = 2
                        WRITE(nice_eunit, *, IOSTAT = ierror)                                  &     
                                 "nice_allocate_float_3d :: ERROR dimensions less than ONE " , &
                                 " length of dimension 1 = " , lenDim1                       , &
                                 " lenght of dimension 2 = " , lenDim2                       , &
                                 " lenght of dimension 3 = " , lenDim3
                        RETURN 
                END IF
        ELSE
                nice_allocate_float_3d = 3
                WRITE(nice_eunit, *, IOSTAT = ierror)                                 & 
                         "nice_allocate_float_3d :: ERROR variable already allocated"
                RETURN 
        END IF
        
    END FUNCTION nice_allocate_float_3d

    !-------------------------------------------------------------------------------------------!

    INTEGER FUNCTION nice_allocate_float_4d( var , lenDim1 , lenDim2 , lenDim3 , lenDim4 )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - allocates float array of rank four by length lenDim1, lenDim2, lenDim3, and lenDim4   ! 
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - allocatable float array of rank four                                                !  
    !     - first  dimension length of the array                                                ! 
    !     - second dimension length of the array                                                ! 
    !     - third  dimension length of the array                                                ! 
    !     - fourth dimension length of the array                                                ! 
    !                                                                                           !  
    !   - Output                                                                                !  
    !     - allocated float array of rank three and of length lenDim1, lenDim2, lenDim3, and    !
    !       lenDim4                                                                             ! 
    !     - initialized array by 0                                                              ! 
    !                                                                                           !
    !   - Return value                                                                          !      
    !     -   0 : allocate o.k.                                                                 !
    !     - /=0 : allocate error: not enough memory                                             !  
    !     -   2 : allocate error: length of dimensions is less than ONE                         ! 
    !     -   3 : allocate error: variable already allocated                                    !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        !--- Function Variables
        !--- Array with four dimensions of type float
        REAL (KIND=sp)         , INTENT (inout), ALLOCATABLE, DIMENSION(:,:,:,:)  :: var 
        !--- Dimensions length
        INTEGER (KIND=i4)      , INTENT (in)                                      :: lenDim1
        !--- Dimensions length
        INTEGER (KIND=i4)      , INTENT (in)                                      :: lenDim2
        !--- Dimensions length
        INTEGER (KIND=i4)      , INTENT (in)                                      :: lenDim3
        !--- Dimensions length
        INTEGER (KIND=i4)      , INTENT (in)                                      :: lenDim4

        !--- Help variables
        INTEGER                                                                   :: ierror                  

        
        !---------------------------------------------------------------------------------------!
        !--- Allocate float array of rank four and of length lenDim1, lenDim2, lenDim3, and ----!
        !    and lenDim4 and initialize by O                                                    !    

        IF ( .NOT. ALLOCATED( var ) ) THEN
                IF ( lenDim1 > 0 .AND. &
                     lenDim2 > 0 .AND. &
                     lenDim3 > 0 .AND. &
                     lenDim4 > 0       &
                   ) THEN
                        ALLOCATE ( var( 1 : lenDim1 , &
                                        1 : lenDim2 , &
                                        1 : lenDim3 , &
                                        1 : lenDim4   &
                                      )                               , &
                                        STAT = nice_allocate_float_4d   &
                                 )
                        IF ( nice_allocate_float_4d /= 0 ) THEN
                                WRITE(nice_eunit, *, IOSTAT = ierror)                    &
                                         "nice_allocate_float_4d :: Allocate ERROR = " , &  
                                         nice_allocate_float_4d
                                RETURN 
                        END IF
                        var = 0._sp
                ELSE
                        nice_allocate_float_4d = 2
                        WRITE(nice_eunit, *, IOSTAT = ierror)                                  &    
                                 "nice_allocate_float_4d :: ERROR dimensions less than ONE " , &
                                 " length of dimension 1 = " , lenDim1                       , &
                                 " lenght of dimension 2 = " , lenDim2                       , &
                                 " lenght of dimension 3 = " , lenDim3                       , &
                                 " lenght of dimension 4 = " , lenDim4
                        RETURN 
                END IF
        ELSE
                nice_allocate_float_4d = 3
                WRITE(nice_eunit, *, IOSTAT = ierror)                                 & 
                         "nice_allocate_float_4d :: ERROR variable already allocated"
                RETURN 
        END IF
        
    END FUNCTION nice_allocate_float_4d

    !-------------------------------------------------------------------------------------------!

    INTEGER FUNCTION nice_allocate_double_1d( var , lenDim1 )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - allocates and initializes double array of rank and dimension length lenDim1           ! 
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - allocatable double array with one rank                                              !  
    !     - dimension length of the array                                                       ! 
    !                                                                                           !  
    !   - Output                                                                                !  
    !     - allocated double array of rank one and of dimension length lenDim1                  ! 
    !     - initialized array by 0                                                              ! 
    !                                                                                           !
    !   - Return value                                                                          !      
    !     -   0 : allocate o.k.                                                                 !
    !     - /=0 : allocate error: not enough memory                                             !  
    !     -   2 : allocate error: length of dimensions is less than ONE                         ! 
    !     -   3 : allocate error: variable already allocated                                    !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------! 

        !--- Function Variables
        !--- Array with four dimensions of type double
        REAL (KIND=dp)         , INTENT (inout), ALLOCATABLE, DIMENSION(:)        :: var 
        !--- Upper Bound of the array 
        INTEGER (KIND=i4)          , INTENT (in)                                  :: lenDim1

        !--- Help variables
        INTEGER                                                                   :: ierror                  

        
        !---------------------------------------------------------------------------------------!
        !--- Allocate double array of one rank and of size lenDim1 initialize by O -------------!   

        IF ( .NOT. ALLOCATED( var ) ) THEN
                IF ( lenDim1  > 0 ) THEN
                        ALLOCATE ( var( 1 : lenDim1 ) , STAT = nice_allocate_double_1d )
                        IF ( nice_allocate_double_1d /= 0 ) THEN 
                                WRITE(nice_eunit, *, IOSTAT = ierror)                     &  
                                         "nice_allocate_double_1d :: Allocate ERROR = " , &  
                                         nice_allocate_double_1d
                                RETURN 
                        END IF
                        var = 0._dp
                ELSE
                        nice_allocate_double_1d = 2
                        WRITE(nice_eunit, *, IOSTAT = ierror)                                   &  
                                 "nice_allocate_double_1d :: ERROR dimensions less than ONE " , & 
                                 "length of dimension 1 = " , lenDim1 
                        RETURN 
                END IF
        ELSE
                nice_allocate_double_1d = 3
                WRITE(nice_eunit, *, IOSTAT = ierror)                                  &
                         "nice_allocate_double_1d :: ERROR variable already allocated"
                RETURN 
        END IF
        
    END FUNCTION nice_allocate_double_1d

    !-------------------------------------------------------------------------------------------!

    INTEGER FUNCTION nice_allocate_double_2d( var , lenDim1 , lenDim2 )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - allocates double array of rank two by length lenDim1 and lenDim2                      ! 
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - allocatable double array of rank two                                                !  
    !     - first dimension length of the array                                                 ! 
    !     - second dimension length of the array                                                ! 
    !                                                                                           !  
    !   - Output                                                                                !  
    !     - allocated double array of rank two and of length lenDim1, and lenDim2               ! 
    !     - initialized array by 0                                                              ! 
    !                                                                                           !
    !   - Return value                                                                          !      
    !     -   0 : allocate o.k.                                                                 !
    !     - /=0 : allocate error: not enough memory                                             !  
    !     -   2 : allocate error: length of dimensions is less than ONE                         ! 
    !     -   3 : allocate error: variable already allocated                                    !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------! 

        !--- Function Variables
        !--- Array with four dimensions of type double
        REAL (KIND=dp)         , INTENT (inout), ALLOCATABLE, DIMENSION(:,:)      :: var 
        !--- Dimensions length
        INTEGER (KIND=i4)      , INTENT (in)                                      :: lenDim1
        !--- Dimensions length
        INTEGER (KIND=i4)      , INTENT (in)                                      :: lenDim2

        !--- Help variables
        INTEGER                                                                   :: ierror                  

        
        !---------------------------------------------------------------------------------------!
        !--- Allocate double array of rank two and of length lenDim1 and lenDim2 and -----------!
        !    initialize by O

        IF ( .NOT. ALLOCATED( var ) ) THEN
                IF ( lenDim1 > 0 .AND. & 
                     lenDim2 > 0       &
                   ) THEN
                        ALLOCATE ( var( 1 : lenDim1 , &
                                        1 : lenDim2   &
                                      )                                , &
                                        STAT = nice_allocate_double_2d   &
                                 )
                        IF ( nice_allocate_double_2d /= 0 ) THEN
                                WRITE(nice_eunit, *, IOSTAT = ierror)                           &   
                                         "nice_allocate_double_2d :: ERROR not enough memory" , &
                                         nice_allocate_double_2d 
                                RETURN 
                        END IF
                        var = 0._dp
                ELSE
                        nice_allocate_double_2d = 2
                        WRITE(nice_eunit, *, IOSTAT = ierror)                                   &   
                                 "nice_allocate_double_2d :: ERROR dimensions less than ONE " , &
                                 " length of dimension 1 = " , lenDim1                        , &
                                 " lenght of dimension 2 = " , lenDim2
                        RETURN 
                END IF
        ELSE
                nice_allocate_double_2d = 3
                WRITE(nice_eunit, *, IOSTAT = ierror)                                  & 
                         "nice_allocate_double_2d :: ERROR variable already allocated"
                RETURN 
        END IF
        
    END FUNCTION nice_allocate_double_2d

    !-------------------------------------------------------------------------------------------!

    INTEGER FUNCTION nice_allocate_double_4d( var , lenDim1 , lenDim2 , lenDim3 , lenDim4 )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - allocates double array of rank four by length lenDim1, lenDim2, lenDim3, and lenDim4  ! 
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - allocatable double array of rank four                                               !  
    !     - first  dimension length of the array                                                ! 
    !     - second dimension length of the array                                                ! 
    !     - third  dimension length of the array                                                ! 
    !     - fourth dimension length of the array                                                ! 
    !                                                                                           !  
    !   - Output                                                                                !  
    !     - allocated double array of rank three and of length lenDim1, lenDim2, lenDim3, and   !
    !       lenDim4                                                                             ! 
    !     - initialized array by 0                                                              ! 
    !                                                                                           !
    !   - Return value                                                                          !      
    !     -   0 : allocate o.k.                                                                 !
    !     - /=0 : allocate error: not enough memory                                             !  
    !     -   2 : allocate error: length of dimensions is less than ONE                         ! 
    !     -   3 : allocate error: variable already allocated                                    !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        !--- Function Variables
        !--- Array with four dimensions of type double
        REAL (KIND=dp)         , INTENT (inout), ALLOCATABLE, DIMENSION(:,:,:,:)  :: var 
        !--- Dimensions length
        INTEGER (KIND=i4)      , INTENT (in)                                      :: lenDim1
        !--- Dimensions length
        INTEGER (KIND=i4)      , INTENT (in)                                      :: lenDim2
        !--- Dimensions length
        INTEGER (KIND=i4)      , INTENT (in)                                      :: lenDim3
        !--- Dimensions length
        INTEGER (KIND=i4)      , INTENT (in)                                      :: lenDim4

        !--- Help variables
        INTEGER                                                                   :: ierror                  

        
        !---------------------------------------------------------------------------------------!
        !--- Allocate double array of rank four and of length lenDim1, lenDim2, lenDim3, and ---!
        !    and lenDim4 and initialize by O                                                    !  

        IF ( .NOT. ALLOCATED( var ) ) THEN
                IF ( lenDim1 > 0 .AND. &
                     lenDim2 > 0 .AND. &
                     lenDim3 > 0 .AND. &
                     lenDim4 > 0       &
                   ) THEN
                        ALLOCATE ( var( 1 : lenDim1 , &
                                        1 : lenDim2 , &
                                        1 : lenDim3 , &
                                        1 : lenDim4   &
                                      )                                , &
                                        STAT = nice_allocate_double_4d   &
                                 )
                        IF ( nice_allocate_double_4d /= 0 ) THEN
                                WRITE(nice_eunit, *, IOSTAT = ierror)                     &    
                                         "nice_allocate_double_4d :: Allocate ERROR = " , & 
                                         nice_allocate_double_4d
                                RETURN 
                        END IF
                        var = 0._dp
                ELSE
                        nice_allocate_double_4d = 2
                        WRITE(nice_eunit, *, IOSTAT = ierror)                                   &    
                                 "nice_allocate_double_4d :: ERROR dimensions less than ONE " , &
                                 " length of dimension 1 = " , lenDim1                        , &
                                 " lenght of dimension 2 = " , lenDim2                        , &
                                 " lenght of dimension 3 = " , lenDim3                        , &
                                 " lenght of dimension 4 = " , lenDim4
                        RETURN 
                END IF
        ELSE
                nice_allocate_double_4d = 3
                WRITE(nice_eunit, *, IOSTAT = ierror)                                  & 
                         "nice_allocate_double_4d :: ERROR variable already allocated"
                RETURN 
        END IF
        
    END FUNCTION nice_allocate_double_4d

    !-------------------------------------------------------------------------------------------!

    INTEGER FUNCTION nice_allocate_string_1d( var , lenDim1 )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - allocates string array of rank one by length lenDim1                                  ! 
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - allocatable string array with one rank                                              !  
    !     - dimension length of the array                                                       ! 
    !                                                                                           !  
    !   - Output                                                                                !  
    !     - allocated string array of rank one and of length lenDim1                            ! 
    !     - initialized array by ""                                                             ! 
    !                                                                                           !
    !   - Return value                                                                          !      
    !     -   0 : allocate o.k.                                                                 !
    !     - /=0 : allocate error: not enough memory                                             !  
    !     -   2 : allocate error: length of dimensions is less than ONE                         ! 
    !     -   3 : allocate error: variable already allocated                                    !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------! 

        !--- Function Variables
        !--- Array with one dimension of type string
        CHARACTER (LEN=*)          , INTENT (inout), ALLOCATABLE, DIMENSION(:)    :: var 
        !--- Dimensions length
        INTEGER (KIND=i4)          , INTENT (in)                                  :: lenDim1

        !--- Help variables
        INTEGER                                                                   :: ierror                     

        
        !---------------------------------------------------------------------------------------!
        !--- Allocate string array with one rank and of length lenDim1 and initialize by O -----! 

        IF ( .NOT. ALLOCATED( var ) ) THEN
                IF ( lenDim1 > 0 ) THEN
                        ALLOCATE ( var( 1 : lenDim1 ) , STAT = nice_allocate_string_1d )
                        IF ( nice_allocate_string_1d /= 0 ) THEN
                                WRITE(nice_eunit, *, IOSTAT = ierror)                     &    
                                         "nice_allocate_string_1d :: Allocate ERROR = " , &  
                                         nice_allocate_string_1d
                                RETURN 
                        END IF
                        var = ""
                ELSE
                        nice_allocate_string_1d = 2
                        WRITE(nice_eunit, *, IOSTAT = ierror)                               &
                                 "nice_allocate_string_1d :: ERROR dimension less than ONE"
                        RETURN 
                END IF
        ELSE
                nice_allocate_string_1d = 3
                WRITE(nice_eunit, *, IOSTAT = ierror)                                  & 
                         "nice_allocate_string_1d :: ERROR variable already allocated"
                RETURN 
        END IF
        
    END FUNCTION nice_allocate_string_1d

    !-------------------------------------------------------------------------------------------!

    INTEGER FUNCTION nice_allocate_string_2d( var , lenDim1 , lenDim2)

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - allocates string array of rank two by length lenDim1, and lenDim2                     ! 
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - allocatable string array of rank two                                                !  
    !     - first dimension length of the array                                                 ! 
    !     - second dimension length of the array                                                ! 
    !                                                                                           !  
    !   - Output                                                                                !  
    !     - allocated string array of rank two and of length lenDim1, and lenDim2               ! 
    !     - initialized array by 0                                                              ! 
    !                                                                                           !
    !   - Return value                                                                          !      
    !     -   0 : allocate o.k.                                                                 !
    !     - /=0 : allocate error: not enough memory                                             !  
    !     -   2 : allocate error: length of dimensions is less than ONE                         ! 
    !     -   3 : allocate error: variable already allocated                                    !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------! 

        !--- Function Variables
        !--- Array with two dimensions of type string
        CHARACTER (LEN=*)          , INTENT (inout), ALLOCATABLE, DIMENSION(:,:)  :: var 
        !--- Dimensions length
        INTEGER (KIND=i4)          , INTENT (in)                                  :: lenDim1
        !--- Dimensions length
        INTEGER (KIND=i4)          , INTENT (in)                                  :: lenDim2

        !--- Help variables
        INTEGER                                                                   :: ierror                     

        
        !---------------------------------------------------------------------------------------!
        !--- Allocate string array of rank two and of length lenDim1 and lenDim2 and -----------!
        !    initialize by O

        IF ( .NOT. ALLOCATED( var ) ) THEN
                IF ( lenDim1 > 0 .AND. &
                     lenDim2 > 0       &
                   ) THEN
                        ALLOCATE ( var( 1 : lenDim1 , &
                                        1 : lenDim2   &
                                      )                                , &
                                        STAT = nice_allocate_string_2d   &
                                 )
                        IF ( nice_allocate_string_2d /= 0 ) THEN
                                WRITE(nice_eunit, *, IOSTAT = ierror)                     &    
                                         "nice_allocate_string_2d :: Allocate ERROR = " , &  
                                         nice_allocate_string_2d
                                RETURN 
                        END IF
                        var = ""
                ELSE
                        nice_allocate_string_2d = 2
                        WRITE(nice_eunit, *, IOSTAT = ierror)                                   &      
                                 "nice_allocate_string_2d :: ERROR dimensions less than ONE " , & 
                                 " length of dimension 1 = " , lenDim1                        , &
                                 " lenght of dimension 2 = " , lenDim2
                        RETURN 
                END IF
        ELSE
                nice_allocate_string_2d = 3
                WRITE(nice_eunit, *, IOSTAT = ierror)                                   &
                         "nice_allocate_string_2d :: ERROR: variable already allocated"
                RETURN 
        END IF
        
    END FUNCTION nice_allocate_string_2d

    !-------------------------------------------------------------------------------------------!

    INTEGER FUNCTION nice_allocate_logical_1d( var , lenDim1 )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - allocates logical array of rank one by length lenDim1                                 ! 
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - allocatable logical array of rank one                                               !  
    !     - dimension length of the array                                                       ! 
    !                                                                                           !  
    !   - Output                                                                                !  
    !     - allocated logical array of rank one and of length lenDim1                           ! 
    !     - initialized array by false                                                          ! 
    !                                                                                           !
    !   - Return value                                                                          !      
    !     -   0 : allocate o.k.                                                                 !
    !     - /=0 : allocate error: not enough memory                                             !  
    !     -   2 : allocate error: length of dimensions is less than ONE                         ! 
    !     -   3 : allocate error: variable already allocated                                    !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------! 

        !--- Function Variables
        !--- Array with four dimensions of type double
        LOGICAL                , INTENT (inout), ALLOCATABLE, DIMENSION(:)        :: var 
        !--- Dimensions length
        INTEGER (KIND=i4)      , INTENT (in)                                      :: lenDim1

        !--- Help variables
        INTEGER                                                                   :: ierror                  

        
        !---------------------------------------------------------------------------------------!
        !--- Allocate double array with one rank and of length lenDim1 and initialize by O -----! 

        IF ( .NOT. ALLOCATED( var ) ) THEN
                IF ( lenDim1 > 0 ) THEN
                        ALLOCATE ( var( 1 : lenDim1 ) , STAT = nice_allocate_logical_1d )
                        IF ( nice_allocate_logical_1d /= 0 ) THEN
                                WRITE(nice_eunit, *, IOSTAT = ierror)                      &   
                                         "nice_allocate_logical_1d :: Allocate ERROR = " , &  
                                         nice_allocate_logical_1d
                                RETURN 
                        END IF
                        var = .false.
                ELSE
                        nice_allocate_logical_1d = 2
                        WRITE(nice_eunit, *, IOSTAT = ierror)                                    &  
                                 "nice_allocate_logical_1d :: ERROR dimensions less than ONE " , &
                                 "length of dimension 1 = ", lenDim1
                        RETURN 
                END IF
        ELSE
                nice_allocate_logical_1d = 3
                WRITE(nice_eunit, *, IOSTAT = ierror)                                  & 
                         "nice_allocate_logical_1d :: ERROR variable already allocated"
                RETURN 
        END IF
        
    END FUNCTION nice_allocate_logical_1d

    !-------------------------------------------------------------------------------------------!

    INTEGER FUNCTION nice_allocate_lu_int_1d( var , uBound1 , olBound1 )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - allocates integer array of rank one by                                                ! 
    !     - default  with length        1 : uBound1                                             ! 
    !     - optional with length olBound1 : uBound1                                             !  
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - allocatable integer array with one rank                                             !  
    !     - dimension length of the array                                                       ! 
    !                                                                                           !  
    !   - Output                                                                                !  
    !     - allocated integer array of rank one                                                 ! 
    !     - initialized array by 0                                                              ! 
    !                                                                                           !
    !   - Return value                                                                          !      
    !     -   0 : allocate o.k.                                                                 !
    !     - /=0 : allocate error: not enough memory                                             !  
    !     -   2 : allocate error: length of dimensions is less than ONE                         ! 
    !     -   3 : allocate error: variable already allocated                                    !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------! 

        !--- Function Variables
        !--- Array with one rank of type integer
        INTEGER (KIND=i4)          , INTENT (inout) , ALLOCATABLE , DIMENSION(:)  :: var 
        !--- Upper Bound of the array 
        INTEGER (KIND=i4)          , INTENT (in)                                  :: uBound1
        !--- Lower Bound of the array
        INTEGER (KIND=i4)          , INTENT (in)    , OPTIONAL                    :: olBound1 

        !--- Local Variables
        INTEGER (KIND=i4)                                                         :: lBound1 

        !--- Help Variables
        INTEGER                                                                   :: ierror                     

        
        !---------------------------------------------------------------------------------------!
        !--- Allocate integer array of one rank and initialize by O ----------------------------! 
        
        lbound1 = 1
        IF ( .NOT. ALLOCATED( var ) ) THEN
                !--- Get lower Bound of the array
                IF ( PRESENT( olBound1) ) lBound1 = olBound1
                IF ( uBound1 > lBound1 ) THEN
                        ALLOCATE ( var( lBound1 : uBound1 ) , STAT = nice_allocate_lu_int_1d )
                        IF ( nice_allocate_lu_int_1d /= 0 ) THEN
                                WRITE(nice_eunit, *, IOSTAT = ierror)                     &   
                                         "nice_allocate_lu_int_1d :: Allocate ERROR = " , &  
                                         nice_allocate_lu_int_1d
                                RETURN 
                        END IF
                        var = 0_i4
                ELSE
                        nice_allocate_lu_int_1d = 2
                        WRITE(nice_eunit, *, IOSTAT = ierror)                                 &  
                                 "nice_allocate_lu_int_1d :: ERROR dimension less than ONE" , &
                                 "length of dimension 1 = ", lBound1, uBound1 
                        RETURN 
                END IF
        ELSE
                nice_allocate_lu_int_1d = 3
                WRITE(nice_eunit, *, IOSTAT = ierror)                                  &  
                         "nice_allocate_lu_int_1d :: ERROR variable already allocated"
                RETURN 
        END IF
        
    END FUNCTION nice_allocate_lu_int_1d

    !-------------------------------------------------------------------------------------------!

    INTEGER FUNCTION nice_allocate_lu_float_1d( var , uBound1 , olBound1 )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - allocates and initialize float array of rank one by                                   ! 
    !     - default  with length        1 : uBound1                                             ! 
    !     - optional with length olBound1 : uBound1                                             !  
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - allocatable float array with one rank                                               !  
    !     - dimension length of the array                                                       ! 
    !                                                                                           !  
    !   - Output                                                                                !  
    !     - allocated float array of rank one                                                   ! 
    !     - initialized array by 0                                                              ! 
    !                                                                                           !
    !   - Return value                                                                          !      
    !     -   0 : allocate o.k.                                                                 !
    !     - /=0 : allocate error: not enough memory                                             !  
    !     -   2 : allocate error: length of dimensions is less than ONE                         ! 
    !     -   3 : allocate error: variable already allocated                                    !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------! 
       
        !--- Function Variables
        !--- Array with four dimensions of type float
        REAL (KIND=sp)             , INTENT (inout), ALLOCATABLE, DIMENSION(:)    :: var 
        !--- Upper Bound of the array 
        INTEGER (KIND=i4)          , INTENT (in)                                  :: uBound1
        !--- Lower Bound of the array
        INTEGER (KIND=i4)          , INTENT (in)    , OPTIONAL                    :: olBound1 

        !--- Local Variables
        INTEGER (KIND=i4)                                                         :: lBound1 

        !--- Help variables
        INTEGER                                                                   :: ierror                  
        
        !---------------------------------------------------------------------------------------!
        !--- Allocate float array of one rank and initialize by O ------------------------------!

        lbound1 = 1
        IF ( .NOT. ALLOCATED( var ) ) THEN
                IF ( PRESENT( olBound1) ) lBound1 = olBound1
                IF ( uBound1 > lBound1 ) THEN
                        ALLOCATE ( var( lBound1 : uBound1 ) , STAT = nice_allocate_lu_float_1d )
                        IF ( nice_allocate_lu_float_1d /= 0 ) THEN
                                WRITE(nice_eunit, *, IOSTAT = ierror)                       &  
                                         "nice_allocate_lu_float_1d :: Allocate ERROR = " , &  
                                         nice_allocate_lu_float_1d
                                RETURN 
                        END IF
                        var = 0._sp
                ELSE
                        nice_allocate_lu_float_1d = 2
                        WRITE(nice_eunit, *, IOSTAT = ierror)                                  &  
                                 "nice_allocate_float_1d :: ERROR dimensions less than ONE " , & 
                                 "length of dimension 1 = " , lBound1 , uBound1
                        RETURN 
                END IF
        ELSE
                nice_allocate_lu_float_1d = 3
                WRITE(nice_eunit, *, IOSTAT = ierror)                                 & 
                         "nice_allocate_float_1d :: ERROR variable already allocated"
                RETURN 
        END IF
        
    END FUNCTION nice_allocate_lu_float_1d

    !-------------------------------------------------------------------------------------------!

    INTEGER FUNCTION nice_allocate_lu_double_1d( var , uBound1 , olBound1 )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - allocates and initializes double array of rank one by                                 ! 
    !     - default  with length        1 : uBound1                                             ! 
    !     - optional with length olBound1 : uBound1                                             !  
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - allocatable double array with one rank                                              !  
    !     - dimension length of the array                                                       ! 
    !                                                                                           !  
    !   - Output                                                                                !  
    !     - allocated double array of rank one                                                  ! 
    !     - initialized array by 0                                                              ! 
    !                                                                                           !
    !   - Return value                                                                          !      
    !     -   0 : allocate o.k.                                                                 !
    !     - /=0 : allocate error: not enough memory                                             !  
    !     -   2 : allocate error: length of dimensions is less than ONE                         ! 
    !     -   3 : allocate error: variable already allocated                                    !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------! 

        !--- Function Variables
        !--- Array with four dimensions of type double
        REAL (KIND=dp)         , INTENT (inout), ALLOCATABLE, DIMENSION(:)        :: var 
        !--- Upper Bound of the array 
        INTEGER (KIND=i4)          , INTENT (in)                                  :: uBound1
        !--- Lower Bound of the array
        INTEGER (KIND=i4)          , INTENT (in)    , OPTIONAL                    :: olBound1 

        !--- Local Variables
        INTEGER (KIND=i4)                                                         :: lBound1 


        !--- Help variables
        INTEGER                                                                   :: ierror                  

        
        !---------------------------------------------------------------------------------------!
        !--- Allocate double array of one rank and initialize by O -----------------------------!   

        lbound1 = 1
        IF ( .NOT. ALLOCATED( var ) ) THEN
                IF ( PRESENT( olBound1) ) lBound1 = olBound1
                IF ( uBound1 > lBound1 ) THEN
                        ALLOCATE ( var( lBound1 : uBound1 ) , STAT = nice_allocate_lu_double_1d )
                        IF ( nice_allocate_lu_double_1d /= 0 ) THEN
                                WRITE(nice_eunit, *, IOSTAT = ierror)                        &    
                                         "nice_allocate_lu_double_1d :: Allocate ERROR = " , &  
                                         nice_allocate_lu_double_1d
                                RETURN 
                        END IF
                        var = 0._dp
                ELSE
                        nice_allocate_lu_double_1d = 2
                        WRITE(nice_eunit, *, IOSTAT = ierror)                                      &   
                                 "nice_allocate_lu_double_1d :: ERROR dimensions less than ONE " , &
                                 "length of dimension 1 = " , lBound1 , uBound1 
                        RETURN 
                END IF
        ELSE
                nice_allocate_lu_double_1d = 3
                WRITE(nice_eunit, *, IOSTAT = ierror)                                     & 
                         "nice_allocate_lu_double_1d :: ERROR variable already allocated"
                RETURN 
        END IF
        
    END FUNCTION nice_allocate_lu_double_1d

    !-------------------------------------------------------------------------------------------!

END MODULE mod_nice

 
