!------------------------------------------------------------------------------------------------!
! Module of external tidal forcing of individual partial tides                                   !
!                                                                                                !
! orginal version from Ulrike Seiler , ulrike.seiler@zmaw.de                                     !
!                                                                                                !
! (c) Petra Nerge 2012 , petra.nerge@informatik.uni-hamburg.de                                   !
!------------------------------------------------------------------------------------------------!
!                                                                                                !
! Description                                                                                    !
!                                                                                                !
!   Module of external tidal forcing of individual partial tides                                 !


MODULE mod_tide_force

    USE mod_kind
    USE mod_constants
    USE mod_grid

    IMPLICIT none

    !--- GLOBAL VARIABLES CONTAIN ---------------------------------------------------------------!
    !
    !--- Tidal band:
    !    diurnal       = 1
    !    semi-diurnal  = 2
    !    long-periodic = 3
    INTEGER                                    :: tide_band             !--- org: itide (namelist)
    !--- Doodsons k-factor for the amplitude
    !    of the partial tide
    REAL (KIND=wp)                             :: tide_amplitude_factor !--- org: amptid (namelist)
    !--- Tidal angular velocity
    !    in degrees per hour
    REAL (KIND=wp)                             :: tide_angular_vel      !--- org: degpeh (namelist)
    !--- Tidal angular velocity
    !    in radian/secs
    REAL (KIND=wp)                             :: tide_omega            !--- org: ometid
    !--- Tidal period in secs
    REAL (KIND=wp)                             :: tide_period           !--- org: tidper
    !--------------------------------------------------------------------------------------------!


    !--- PRIVATE VARIABLES CONTAIN --------------------------------------------------------------!
    !
    !--- Error Value
    REAL (KIND=wp) , PARAMETER                 :: errValue = -9999._wp
    !--------------------------------------------------------------------------------------------!


    !--- INTERFACE CONTAIN ----------------------------------------------------------------------!
    !
    !--- Initialize of the tides variables
    INTERFACE tide_force_initialize
            module procedure tide_force_initialize
    END INTERFACE tide_force_initialize
    !--- Tidal angular velocity in radian/secs of type REAL(KIND=wp)
    INTERFACE tide_force_omega
            module procedure tide_force_omega
    END INTERFACE tide_force_omega
    !--- Tidal period secs of type REAL(KIND=wp)
    INTERFACE tide_force_period
            module procedure tide_force_period
    END INTERFACE tide_force_period
    !--- Zonal tidal force, the band being determined by "tide_band" of type REAL(KIND=wp)
    INTERFACE tide_force_zonal
            module procedure tide_force_zonal
    END INTERFACE tide_force_zonal
    !--- Meridional tidal force, the band being determined by "tide_band" of type REAL(KIND=wp)
    INTERFACE tide_force_merid
            module procedure tide_force_merid
    END INTERFACE tide_force_merid


  !---------------------------------------------------------------------------------------------!

  CONTAINS

    !-------------------------------------------------------------------------------------------!

    SUBROUTINE tide_force_initialize( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - initializes the time stepping variables                                               !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none


        !---------------------------------------------------------------------------------------!
        !--- Initialize time stepping ----------------------------------------------------------!

        !--- Tidal band: diurnal = 1 , semi-diurnal = 2 , long-periodic = 3
        tide_band  = 2
        !--- Doodsons k-factor for the amplitude of the partial tide
        tide_amplitude_factor = 1._wp
        !--- Tidal angular velocity in degrees per hour
        !    M2 tide
        tide_angular_vel = 28.9841042_wp
        !--- Tidal angular velocity in radian/secs
        tide_omega = tide_force_omega( tide_angular_vel )
        !--- Tidal period in secs
        tide_period = tide_force_period( tide_omega )

        !---------------------------------------------------------------------------------------!

    END SUBROUTINE tide_force_initialize

    !-------------------------------------------------------------------------------------------!

    ELEMENTAL FUNCTION tide_force_omega( angular_vel )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     - tidal angular velocity in radian/secs of type REAL(KIND=wp)                         !
    !                                                                                           !
    !   - Input                                                                                 !
    !     - tidal angular velocity in degree/hour of type REAL(KIND=wp)                         !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function
        REAL (KIND=wp)                                   :: tide_force_omega

        !--- Function variables
        !--- Tidal angular velocity in degrees per hour
        REAL (KIND=wp) , INTENT (IN)                     :: angular_vel


        !---------------------------------------------------------------------------------------!
        !--- Calculates tidal angular velocity in radian/secs ----------------------------------!

        !--- Tidal angular velocity in radian/secs
        tide_force_omega = angular_vel * radian/3600._wp

        !---------------------------------------------------------------------------------------!

    END FUNCTION tide_force_omega

    !-------------------------------------------------------------------------------------------!

    ELEMENTAL FUNCTION tide_force_period( omega )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     - tidal period secs of type REAL(KIND=wp)                                             !
    !                                                                                           !
    !   - Input                                                                                 !
    !     - tidal angular velocity in radian/hour of type REAL(KIND=wp)                         !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function
        REAL (KIND=wp)                                   :: tide_force_period

        !--- Function variables
        !--- Tidal angular velocity in radian per secs
        REAL (KIND=wp) , INTENT (IN)                     :: omega


        !---------------------------------------------------------------------------------------!
        !--- Calculates tidal period in secs ---------------------------------------------------!

        !--- Tidal period secs
        tide_force_period = zweipi / omega

        !---------------------------------------------------------------------------------------!

    END FUNCTION tide_force_period

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION tide_force_zonal( cos_phase_lag , sin_phase_lag )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     - zonal tidal force, the band being determined by "tide_band"                         !
    !     - error Value on error                                                                !
    !                                                                                           !
    !   - Input                                                                                 !
    !     - cosine and sine of tide phase lag of type REAL(KIND=wp)                             !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end , lats_start_u:lats_end_u)  :: tide_force_zonal

        !--- Function variables
        !--- Cosine of tide phase lag
        REAL (KIND=wp) , INTENT (IN)                                                  :: cos_phase_lag
        !--- Sine of tide phase lag
        REAL (KIND=wp) , INTENT (IN)                                                  :: sin_phase_lag

        !--- Local variables
        !--- Amplitude of the tide
        REAL (KIND=wp)                                                                :: amplitude

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                                                                       :: lon
        !--- Latitudes loop counter
        INTEGER                                                                       :: lat


        !---------------------------------------------------------------------------------------!
        !--- Calculates zonal tidal force, the band being determined by "tide_band" ------------!

        SELECT CASE ( tide_band )
                   CASE (  1 )
                           !--- diurnal tides
                           DO lat = lats_start_u , lats_end_u
                              !--- set amplitude of tidal force
                              amplitude = tide_amplitude_factor * sin_lat_u(lat)
                              !--- calculate zonal tidal force
                              DO lon = longs_start , longs_end
                                 tide_force_zonal(lon,lat) = amplitude                       &
                                                           * ( asnul(lon) * cos_phase_lag +  &
                                                               acsul(lon) * sin_phase_lag    &
                                                             )
                              END DO
                           END DO
                   CASE (  2 )
                           !--- semi-diurnal tides
                           DO lat = lats_start_u , lats_end_u
                              !--- set amplitude of tidal force
                              amplitude = tide_amplitude_factor * cos_lat_u(lat)
                              !--- calculate zonal tidal force
                              DO lon = longs_start , longs_end
                                 tide_force_zonal(lon,lat) = amplitude                           &
                                                           * ( sin_lon_u(lon) * cos_phase_lag +  &
                                                               cos_lon_u(lon) * sin_phase_lag    &
                                                             )
                              END DO
                           END DO
                   CASE (  3 )
                           !--- long-period tides, tide_band = 3
                           !--- calculate zonal tidal force
                           tide_force_zonal(:,:) = 0._wp
                   CASE DEFAULT
                           !--- error handling
                           !--- set function status to not o.k.
                           !    wrong choice of tidal band -> tide_band
                           tide_force_zonal(:,:) = errValue
                   END SELECT

        !---------------------------------------------------------------------------------------!

    END FUNCTION tide_force_zonal

    !-------------------------------------------------------------------------------------------!

    FUNCTION tide_force_merid( cos_phase_lag , sin_phase_lag )


    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     - meridional tidal force, the band being determined by "tide_band"                    !
    !     - error Value on error                                                                !
    !                                                                                           !
    !   - Input                                                                                 !
    !     - cosine and sine of tide phase lag of type REAL(KIND=wp)                             !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end , lats_start_v:lats_end_v)  :: tide_force_merid

        !--- Function variables
        !--- Cosine of tide phase lag
        REAL (KIND=wp) , INTENT (IN)                                                  :: cos_phase_lag
        !--- Sine of tide phase lag
        REAL (KIND=wp) , INTENT (IN)                                                  :: sin_phase_lag

        !--- Local variables
        !--- Amplitude factor
        REAL (KIND=wp)                                                                :: factor
        !--- Amplitude of the tide
        REAL (KIND=wp)                                                                :: amplitude

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                                                                       :: lon
        !--- Latitudes loop counter
        INTEGER                                                                       :: lat


        !---------------------------------------------------------------------------------------!
        !--- Calculates meridional tidal force, the band being determined by "tide_band" -------!

        SELECT CASE ( tide_band )
                   CASE (  1 )
                           !--- diurnal tides
                           DO lat = lats_start_v , lats_end_v
                              !--- set amplitude of tidal force
                              amplitude = tide_amplitude_factor               &
                                        * ( sin_lat_v(lat) * sin_lat_v(lat) - &
                                            cos_lat_v(lat) * cos_lat_v(lat)   &
                                          )
                              !--- calculate meridional tidal force
                              DO lon = longs_start , longs_end
                                 tide_force_merid(lon,lat) = amplitude                      &
                                                           * ( acsvl(lon) * cos_phase_lag - &
                                                               asnvl(lon) * sin_phase_lag   &
                                                             )
                              END DO
                           END DO
                   CASE (  2 )
                           !--- semi-diurnal tides
                           DO lat = lats_start_v , lats_end_v
                              !--- set amplitude of tidal force
                              amplitude = tide_amplitude_factor * cos_lat_v(lat) * sin_lat_v(lat)
                              !--- calculate meridional tidal force
                              DO lon = longs_start , longs_end
                                 tide_force_merid(lon,lat) = amplitude                          &
                                                           * ( cos_lon_v(lon) * cos_phase_lag - &
                                                               sin_lon_v(lon) * sin_phase_lag   &
                                                             )
                              END DO
                           END DO
                   CASE (  3 )
                           !--- long-period tides, tide_band = 3
                           !--- set amplitude factor
                           factor = tide_amplitude_factor * cos_phase_lag * 1.5_wp
                           DO lat = lats_start_v , lats_end_v
                              !--- set amplitude of tidal force
                              amplitude = factor * cos_lat_v(lat) * sin_lat_v(lat)
                              !--- calculate meridional tidal force
                              DO lon = longs_start , longs_end
                                 tide_force_merid(lon,lat) = amplitude
                              END DO
                           END DO
                   CASE DEFAULT
                           !--- error handling
                           !--- set function status to not o.k.
                           !    wrong choice of tidal band -> tide_band
                           tide_force_merid(:,:) = errValue
                   END SELECT

    END FUNCTION tide_force_merid

    !-------------------------------------------------------------------------------------------!

END MODULE mod_tide_force

