!------------------------------------------------------------------------------------------------!
! Module of zonal velocity                                                                       !
!                                                                                                !
! orginal version from Ulrike Seiler , ulrike.seiler@zmaw.de                                     !
!                                                                                                !
! (c) Petra Nerge 2012 , petra.nerge@informatik.uni-hamburg.de                                   !
!------------------------------------------------------------------------------------------------!
!                                                                                                !
! Description                                                                                    !
!                                                                                                !
!   Module of zonal velocity                                                                     !


MODULE mod_zonal_velocity

    USE mod_kind
    USE mod_grid
    USE mod_time_stepping     , ONLY : delta_time
    USE mod_earth_quantities
 
    IMPLICIT NONE

    !--- GLOBAL VARIABLES CONTAIN
    !
    !--- State variables ------------------------------------------------------------------------!
    !--- Field of zonal velocity
    REAL (KIND=wp) , DIMENSION (longs_start_sub_1:longs_end_add_1 , lats_start_u:lats_end_u_add_1) &
    :: u 
    !--- Field of zonal velocity for v-points
    !--- org: fyn
    REAL (KIND=wp) , DIMENSION (longs_start      :longs_end       , lats_start_v:lats_end_v      ) &
    :: u_v_points = 0._wp 
    !--------------------------------------------------------------------------------------------!

    !--- Coefficients of the zonal momentum/velocity equation -----------------------------------!
    !--- Explicit part of pressure gradient
    REAL (KIND=wp) , DIMENSION (                        lats_start_u:lats_end_u)                 &
    :: u_pressure_gradient_coef_exp                                                              !--- org: fxgj 
    !--- Implicit part of pressure gradient
    REAL (KIND=wp) , DIMENSION (longs_start:longs_end , lats_start_u:lats_end_u  )               &
    :: u_pressure_gradient_coef_imp                                                              !--- org: fxn
    !--------------------------------------------------------------------------------------------!

    !--- INTERFACE CONTAIN ----------------------------------------------------------------------!
    !
    !--- Coefficient of explicit part of pressure gradient
    INTERFACE zonal_vel_pressure_gradient_coef_exp
            module procedure zonal_vel_pressure_gradient_coef_exp
    END INTERFACE zonal_vel_pressure_gradient_coef_exp
    !--- Coefficient of implicit part of pressure gradient
    INTERFACE zonal_vel_pressure_gradient_coef_imp
            module procedure zonal_vel_pressure_gradient_coef_imp
    END INTERFACE zonal_vel_pressure_gradient_coef_imp
    !--- Zonal velocity at northernmost latitudes by cross-polar averaging              
    INTERFACE zonal_vel_boundary_lat
            module procedure zonal_vel_boundary_lat
    END INTERFACE zonal_vel_boundary_lat
    !--- Zonal velocity for v-points              
    INTERFACE zonal_vel_v_points
            module procedure zonal_vel_v_points
    END INTERFACE zonal_vel_v_points
    !--- Zonal momentum due to coriolis acceleration              
    INTERFACE zonal_vel_coriolis
            module procedure zonal_vel_coriolis
    END INTERFACE zonal_vel_coriolis
    !--- Zonal momentum due to eddy viscosity related 
    !    to delta(v)-term 
    INTERFACE zonal_vel_eddy_dv_dlon
            module procedure zonal_vel_eddy_dv_dlon
    END INTERFACE zonal_vel_eddy_dv_dlon
    !--- Zonal momentum due to eddy viscosity related 
    !    to u-term                                         
    INTERFACE zonal_vel_eddy_u
            module procedure zonal_vel_eddy_u
    END INTERFACE zonal_vel_eddy_u
    !--- Zonal momentum due to eddy viscosity related 
    !    to delta(u)-term 
    INTERFACE zonal_vel_eddy_du_dlat
            module procedure zonal_vel_eddy_du_dlat
    END INTERFACE zonal_vel_eddy_du_dlat
    !--- Zonal momentum due to eddy viscosity related 
    !    to delta^2(u)/delta^2(lat)-term 
    INTERFACE zonal_vel_eddy_d2u_d2lat
            module procedure zonal_vel_eddy_d2u_d2lat
    END INTERFACE zonal_vel_eddy_d2u_d2lat
    !--- Zonal momentum due to eddy viscosity related 
    !    to delta^2(u)/delta^2(lon)-term 
    INTERFACE zonal_vel_eddy_d2u_d2lon
            module procedure zonal_vel_eddy_d2u_d2lon
    END INTERFACE zonal_vel_eddy_d2u_d2lon
    !--- Zonal momentum due to advection                                         
    INTERFACE zonal_vel_advection
            module procedure zonal_vel_advection
    END INTERFACE zonal_vel_advection
    !--- Zonal velocity due to explicit part of pressure gradient and 
    !    parametrization of tidal pertubation potential proportional 
    !    to the pressure gradient        
    INTERFACE zonal_vel_pressure_gradient_exp
            module procedure zonal_vel_pressure_gradient_exp
    END INTERFACE zonal_vel_pressure_gradient_exp
    !--- Zonal velocity due to implicit part of pressure gradient and 
    !    parametrization of tidal pertubation potential proportional 
    !    to the pressure gradient        
    INTERFACE zonal_vel_pressure_gradient_imp
            module procedure zonal_vel_pressure_gradient_imp
    END INTERFACE zonal_vel_pressure_gradient_imp
    !--- Zonal velocity due to bottom friction  
    INTERFACE zonal_vel_bottom_friction
            module procedure zonal_vel_bottom_friction
    END INTERFACE zonal_vel_bottom_friction
 

  !---------------------------------------------------------------------------------------------!

  CONTAINS

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION zonal_vel_pressure_gradient_coef_exp( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     coefficient of the explicit part of pressure gradient within the zonal                !
    !     momentum/velocity equation:                                                           !
    !     gravity_acceleration(1-lsa_parameter)/(earthradius*cos(lat))*delta(zeta)/delta(lon)   !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (lats_start_u:lats_end_u)  :: zonal_vel_pressure_gradient_coef_exp
 
        !--- Local variables
        REAL (KIND=wp)                                        :: constant                        !--- org: fxg 

        !--- Help variables
        !--- Latitudes loop counter
        INTEGER                                               :: lat


        !---------------------------------------------------------------------------------------!
 
        !--- set constant: 
        !    gravity_acceleration(1-lsa_parameter)*timestep*1/(earthradius*delta(lon))
        constant = (1._wp - lsa_parameter) &
                 * gravity_acceleration    &
                 * delta_time              &
                 * delta_x_inv_half             
        !--- set coefficient constant*1/cos(lat) 
        DO lat = lats_start_u , lats_end_u
           zonal_vel_pressure_gradient_coef_exp(lat) = & 
                                            constant * &
                                  cos_lat_u_inv(lat)  
        END DO

        !---------------------------------------------------------------------------------------!

    END FUNCTION zonal_vel_pressure_gradient_coef_exp

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION zonal_vel_pressure_gradient_coef_imp( friction_function )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     coefficient of the implicit part of pressure gradient within the zonal                !
    !     momentum/velocity equation:                                                           !
    !     gravity_acceleration(1-lsa_parameter)/(earthradius*cos(lat))*delta(zeta)/delta(lon)*  !
    !     bottom friction function                                                              !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp)              , DIMENSION (longs_start:longs_end , lats_start_u:lats_end_u) &
        :: zonal_vel_pressure_gradient_coef_imp
 
        !--- Function variables
        !--- Field of zonal bottom friction function 
        REAL (KIND=wp) , INTENT(IN) , DIMENSION (longs_start:longs_end , lats_start_u:lats_end_u) &
        :: friction_function

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitudes loop counter
        INTEGER                       :: lat 


        !---------------------------------------------------------------------------------------!
 
        !--- set coefficient  
        DO lat = lats_start_u , lats_end_u
           DO lon = longs_start , longs_end
              zonal_vel_pressure_gradient_coef_imp(lon , lat) = &
                                 friction_function(lon , lat) * & 
                      u_pressure_gradient_coef_exp(      lat)  
           END DO
        END DO

        !---------------------------------------------------------------------------------------!

    END FUNCTION zonal_vel_pressure_gradient_coef_imp

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION zonal_vel_boundary_lat( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     zonal velocity at northernmost latitudes by cross-polar averaging                     !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start_sub_1:longs_end_add_1)  :: zonal_vel_boundary_lat
 
        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                                                         :: lon
 

        !---------------------------------------------------------------------------------------!
        !--- Calculate j+1-values of zonal velocity at northernmost latitudes by cross-polar ---!
        !    averaging                                                                          !

        !--- first hemisphere
        DO lon = longs_start , longs_end_half
           zonal_vel_boundary_lat(lon                            ) = & 
                            0.5_wp                                 * &
                            (   u(lon                , lats_end_u)   &
                              - u(lon+longs_end_half , lats_end_u)   &
                            )
        END DO
        !--- second hemisphere
        DO lon = longs_end_half+1 , longs_end
           zonal_vel_boundary_lat(lon                            ) = &
                            0.5_wp                                 * &
                            (   u(lon                , lats_end_u)   &
                              - u(lon-longs_end_half , lats_end_u)   &
                            )
        END DO
        !--- cyclic boundary condition for longitudes
        zonal_vel_boundary_lat(longs_start_sub_1) = &
        zonal_vel_boundary_lat(longs_end        )

        zonal_vel_boundary_lat(longs_end_add_1  ) = &
        zonal_vel_boundary_lat(longs_start) 

        !---------------------------------------------------------------------------------------!

    END FUNCTION zonal_vel_boundary_lat

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION zonal_vel_v_points( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     zonal velocity for v-points                                                           !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end , lats_start_v:lats_end_v)            &
        :: zonal_vel_v_points

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitudes loop counter
        INTEGER                       :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate zonal velocity for v-points ---------------------------------------------!

        DO lat = lats_start_v , lats_end_v
           DO lon = longs_start , longs_end
              zonal_vel_v_points(lon   , lat  ) = & 
                      0.25_wp                   * &
                      (    (   u(lon-1 , lat-1)   &
                             + u(lon   , lat-1)   &
                           )                      &
                        * weight_u_south(lat  )   &
                        +  (   u(lon-1 , lat  )   &
                             + u(lon   , lat  )   &
                           )                      &
                        * weight_u_north(lat  )   & 
                      )
           END DO
        END DO 

        !---------------------------------------------------------------------------------------!

    END FUNCTION zonal_vel_v_points

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION zonal_vel_coriolis( merid_vel )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     zonal momentum due to coriolis acceleration                                           !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp)              , DIMENSION (longs_start:longs_end , lats_start_u:lats_end_u) &
        :: zonal_vel_coriolis

        !--- Function variables
        !--- Field of meridional velocity for u-points
        REAL (KIND=wp) , INTENT(IN) , DIMENSION (longs_start:longs_end , lats_start_u:lats_end_u) &
        :: merid_vel

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                       :: lon 
        !--- Latitudes loop counter
        INTEGER                       :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate zonal momentum due to coriolis acceleration -----------------------------!

        DO lat = lats_start_u , lats_end_u
           DO lon = longs_start , longs_end
              zonal_vel_coriolis (lon , lat) = &
              coriolis_operator_u(      lat) * &
                        merid_vel(lon , lat)
           END DO
        END DO 

        !---------------------------------------------------------------------------------------!

    END FUNCTION zonal_vel_coriolis

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION zonal_vel_eddy_dv_dlon( merid_vel )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     zonal momentum due to eddy viscosity related to                                       !
    !     delta(v)-term = -2*tan(lat)*delta(v)/delta(lon)*1/earthradius                         !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp)              , DIMENSION (longs_start:longs_end       , lats_start_u:lats_end_u      ) &
        :: zonal_vel_eddy_dv_dlon

        !--- Function variables
        !--- Field of meridional velocity for v-points 
        REAL (KIND=wp) , INTENT(IN) , DIMENSION (longs_start:longs_end_add_1 , lats_start_v:lats_end_u_add_1) &
        :: merid_vel

        !--- Local variables
        REAL (KIND=wp)                 :: coefficient    
        REAL (KIND=wp)                 :: constant    

        !--- Help variables
        !--- Start latitude for loop
        INTEGER                        :: lats_start
        !--- Longitudes loop counter
        INTEGER                        :: lon 
        !--- Latitudes loop counter
        INTEGER                        :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate zonal momentum due to horizontal momentum diffusion ---------------------!

        !--- set constant 1/delta(lon)*1/earthradius
        constant = delta_x_inv * radius_inv
        !--- southernmost latitude
        IF ( lats_start_u < lats_start_v ) THEN
           !--- set coefficient -2*tan(lat)*1/delta(lon)*1/earthradius
           coefficient = -tan_lat_u(lats_start_u) * constant 
           DO lon = longs_start , longs_end
              zonal_vel_eddy_dv_dlon(lon   , lats_start_u  ) = &
                coefficient                                  * &
                (   (      merid_vel(lon+1 , lats_start_u+1)   &
                        -  merid_vel(lon   , lats_start_u+1)   &
                    )                                          &
                  * weight_v_north  (        lats_start_u  )   & 
                )                            
           END DO
           lats_start = lats_start_v
        ELSE 
           lats_start = lats_start_u
        END IF
        !--- remaining latitudes
        DO lat = lats_start , lats_end_u
           !--- set coefficient -2*tan(lat)*1/delta(lon)*1/earthradius
           coefficient = -tan_lat_u(lat) * constant 
           !--- calculate zonal momentum 
           DO lon = longs_start , longs_end
              zonal_vel_eddy_dv_dlon(lon   , lat  ) = &
                coefficient                         * &
                (   (      merid_vel(lon+1 , lat+1)   &
                         - merid_vel(lon   , lat+1)   &
                    )                                 &
                  * weight_v_north  (        lat  )   & 
                )                                   + & 
                coefficient                         * & 
                (   (      merid_vel(lon+1 , lat  )   &
                         - merid_vel(lon   , lat  )   &
                    )                                 &
                  * weight_v_south  (        lat  )   &
                )                                             
           END DO
        END DO 

        !---------------------------------------------------------------------------------------!

    END FUNCTION zonal_vel_eddy_dv_dlon

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION zonal_vel_eddy_u( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     zonal momentum due to eddy viscosity related to                                       !
    !     u-term = -(1+tan^2(lat))*u*1/earthradius^2                                            !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end , lats_start_u:lats_end_u)            &
        :: zonal_vel_eddy_u

        !--- Local variables
        REAL (KIND=wp)                 :: coefficient    
        REAL (KIND=wp)                 :: constant    

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                        :: lon 
        !--- Latitudes loop counter
        INTEGER                        :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate zonal momentum due to horizontal momentum diffusion ---------------------!

        !--- set constant 1/earthradius^2
        constant = radius_inv * radius_inv
        DO lat = lats_start_u , lats_end_u
           !--- set coefficient -(1+tan^2(lat))*1/earthradius^2
           coefficient = -(   1._wp                           & 
                            + tan_lat_u(lat) * tan_lat_u(lat) &
                          )                                   &   
                       *  constant 
           !--- calculate zonal momentum 
           DO lon = longs_start , longs_end
              zonal_vel_eddy_u(lon , lat) = &
                              coefficient * &
                             u(lon , lat)    
           END DO
        END DO 

        !---------------------------------------------------------------------------------------!

    END FUNCTION zonal_vel_eddy_u

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION zonal_vel_eddy_du_dlat( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     zonal momentum due to eddy viscosity related to                                       !
    !     delta(u)-term = -tan(lat)*delta(u)/delta(lat)*1/earthradius                           !
    !     by central differences method                                                         !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end , lats_start_u:lats_end_u)            &
        :: zonal_vel_eddy_du_dlat

        !--- Local variables
        REAL (KIND=wp)                 :: coefficient    
        REAL (KIND=wp)                 :: constant    

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                        :: lon 
        !--- Latitudes loop counter
        INTEGER                        :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate zonal momentum due to horizontal momentum diffusion ---------------------!

        !--- Calculate of first latitude (lats_start_u) by forward differences method
        !--- set coefficient -tan(lat)*(1/delta(lat))*1/earthradius 
        coefficient = -tan_lat_u(lats_start_u) * delta_y_inv * radius_inv
        !--- calculate zonal momentum 
        DO lon = longs_start , longs_end
           zonal_vel_eddy_du_dlat(lon , lats_start_u  ) = &
                            coefficient                 * &
                            (   u(lon , lats_start_u+1)   &
                              - u(lon , lats_start_u  )   &
                            )    
        END DO
        !--- Calculate of remaining latitudes by central differences method
        !--- set constant 1/(2*delta(lat))*1/earthradius
        constant = delta_y_inv_half * radius_inv 
        DO lat = lats_start_v , lats_end_u
           !--- set coefficient -tan(lat)*1/(2*delta(lat))*1/earthradius 
           coefficient = -tan_lat_u(lat) * constant 
           !--- calculate zonal momentum 
           DO lon = longs_start , longs_end
              zonal_vel_eddy_du_dlat(lon , lat  ) = &
                               coefficient        * &
                               (   u(lon , lat+1)   & 
                                 - u(lon , lat-1)   &
                               )    
           END DO
        END DO 

        !---------------------------------------------------------------------------------------!

    END FUNCTION zonal_vel_eddy_du_dlat

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION zonal_vel_eddy_d2u_d2lat( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     zonal momentum due to eddy viscosity related to                                       !
    !     delta^2(u)-term = delta^2(u)/delta(lat)^2                                             !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end , lats_start_u:lats_end_u)            &
        :: zonal_vel_eddy_d2u_d2lat

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                        :: lon 
        !--- Latitudes loop counter
        INTEGER                        :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate zonal momentum due to horizontal momentum diffusion ---------------------!

        !--- Calculate of first latitude (lats_start_u) 
        DO lon = longs_start , longs_end
           zonal_vel_eddy_d2u_d2lat(lon , lats_start_u  ) = &
                      delta_y_inv_square                  * &
                      (           u(lon , lats_start_u+1)   &
                        - 2._wp * u(lon , lats_start_u  )   & 
                      )    
        END DO
        !--- Calculate of remaining latitudes 
        DO lat = lats_start_u_add_1 , lats_end_u
           DO lon = longs_start , longs_end
              zonal_vel_eddy_d2u_d2lat(lon , lat  ) = &
                         delta_y_inv_square         * &
                         (           u(lon , lat+1)   & 
                           +         u(lon , lat-1)   &
                           - 2._wp * u(lon , lat  )   &
                         )    
           END DO
        END DO 

        !---------------------------------------------------------------------------------------!

    END FUNCTION zonal_vel_eddy_d2u_d2lat

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION zonal_vel_eddy_d2u_d2lon( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     zonal momentum due to eddy viscosity related to                                       !
    !     delta^2(u)-term = 1/cos(lat)*delta^2(u)/delta(lon)^2                                  !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp) , DIMENSION (longs_start:longs_end , lats_start_u:lats_end_u)            &
        :: zonal_vel_eddy_d2u_d2lon

        !--- Local variables
        REAL (KIND=wp)                 :: coefficient    

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                        :: lon 
        !--- Latitudes loop counter
        INTEGER                        :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate zonal momentum due to horizontal momentum diffusion ---------------------!

        !--- Calculate of remaining latitudes 
        DO lat = lats_start_u , lats_end_u
           !--- set coefficient 1/cos(lat)*1/delta(lon)^2 
           coefficient = cos_lat_u_inv(lat) * delta_x_inv_square 
           !--- calculate zonal momentum 
           DO lon = longs_start , longs_end
              zonal_vel_eddy_d2u_d2lon(lon  , lat) = &
                         coefficient               * &
                         (           u(lon+1, lat)   & 
                           +         u(lon-1, lat)   &
                           - 2._wp * u(lon  , lat)   &
                         )    
           END DO
        END DO 

        !---------------------------------------------------------------------------------------!

    END FUNCTION zonal_vel_eddy_d2u_d2lon

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION zonal_vel_advection( merid_vel )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     zonal momentum due to advective zonal velocity related term                           !
    !     u/(earthradius*cos(lat))*delta(u)/delta(lon)                                          !
    !     and                                                                                   !
    !     zonal momentum due to advective meridional velocity related term                      !
    !     v/(earthradius*delta(lat))*delta(u)/delta(lat)                                        !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp)              , DIMENSION (longs_start:longs_end , lats_start_u:lats_end_u) &
        :: zonal_vel_advection

        !--- Function variables
        !--- Field of meridional velocity for u-points
        REAL (KIND=wp) , INTENT(IN) , DIMENSION (longs_start:longs_end , lats_start_u:lats_end_u) & 
        :: merid_vel

        !--- Local variables
        REAL (KIND=wp)                 :: coefficient    

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                        :: lon 
        !--- Latitudes loop counter
        INTEGER                        :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate zonal mometum due to advection ------------------------------------------!

        !--- Calculate zonal velocity related term 
        !    u/(earthradius*cos(lat))*delta(u)/delta(lon)
        DO lat = lats_start_u , lats_end_u
           !--- set coefficient 1/(earthradius*cos(lat))*1/delta(lon) 
           coefficient = cos_lat_u_inv(lat) * delta_x_inv_half 
           !--- calculate zonal momentum 
           DO lon = longs_start , longs_end
              zonal_vel_advection(lon   , lat) = &
                            coefficient        * &   
                                u(lon   , lat) * &
                            (   u(lon+1 , lat)   & 
                              - u(lon-1 , lat)   &
                            )    
           END DO
        END DO 

        !--- Calculate meridional velocity related term 
        !    v/(earthradius*delta(lat))*delta(u)/delta(lat)
        !--- calculate of first latitude (lats_start_u) by forward differences method 
        DO lon = longs_start , longs_end
           zonal_vel_advection(lon , lats_start_u      ) = &
           zonal_vel_advection(lon , lats_start_u      ) + &
                   ( merid_vel(lon , lats_start_u      ) * &
                     delta_y_inv                         * &
                     (       u(lon , lats_start_u_add_1)   &
                           - u(lon , lats_start_u      )   & 
                     )                                     &
                   )
        END DO
        !--- calculate of remaining latitudes by central differences method
        DO lat = lats_start_u_add_1 , lats_end_u
           DO lon = longs_start , longs_end
              zonal_vel_advection(lon , lat  ) = &
              zonal_vel_advection(lon , lat  ) + &
                      ( merid_vel(lon , lat  ) * &
                        delta_y_inv_half       * &
                        (       u(lon , lat+1)   & 
                              - u(lon , lat-1)   &
                        )                        & 
                      )
           END DO
        END DO 

        !---------------------------------------------------------------------------------------!

    END FUNCTION zonal_vel_advection

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION zonal_vel_pressure_gradient_exp( elevation )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     zonal velocity due to explicit part of pressure gradient and parametrization of       !
    !     tidal pertubation potential proportional to the pressure gradient                     !
    !     gravity_acceleration(1-lsa_parameter)/(earthradius*cos(lat))*delta(zeta)/delta(lon)   !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp)              , DIMENSION (longs_start:longs_end       , lats_start_u:lats_end_u) &
        :: zonal_vel_pressure_gradient_exp

        !--- Function variables
        !--- Field of sea surface elevation 
        REAL (KIND=wp) , INTENT(IN) , DIMENSION (longs_start:longs_end_add_1 , lats_start_u:lats_end_u) &
        :: elevation

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                        :: lon 
        !--- Latitudes loop counter
        INTEGER                        :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate zonal velocity due to explicit part of pressure gradient and ------------!
        !    parametrization of tidal pertubation potential proportional to the                 !
        !    pressure gradient                                                                  !
  
        !--- calculate zonal velocity 
        DO lat = lats_start_u , lats_end_u
           DO lon = longs_start , longs_end
              zonal_vel_pressure_gradient_exp(lon   , lat) = &
                 u_pressure_gradient_coef_exp(        lat) * &
                                (   elevation(lon+1 , lat)   & 
                                  - elevation(lon   , lat)   &
                                )    
           END DO
        END DO 
 
        !---------------------------------------------------------------------------------------!

    END FUNCTION zonal_vel_pressure_gradient_exp

    !-------------------------------------------------------------------------------------------!

    PURE FUNCTION zonal_vel_pressure_gradient_imp( elevation )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     zonal velocity due to implicit part of pressure gradient and parametrization of       !
    !     tidal pertubation potential proportional to the pressure gradient                     !
    !     gravity_acceleration(1-lsa_parameter)/(earthradius*cos(lat))*delta(zeta)/delta(lon)   !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp)              , DIMENSION (longs_start:longs_end       , lats_start_u:lats_end_u) &
        :: zonal_vel_pressure_gradient_imp

        !--- Function variables
        !--- Field of sea surface elevation 
        REAL (KIND=wp) , INTENT(IN) , DIMENSION (longs_start:longs_end_add_1 , lats_start_u:lats_end_u) &
        :: elevation

        !--- Help variables
        !--- Longitudes loop counter
        INTEGER                        :: lon 
        !--- Latitudes loop counter
        INTEGER                        :: lat 


        !---------------------------------------------------------------------------------------!
        !--- Calculate zonal velocity due to implicit part of pressure gradient and ------------!
        !    parametrization of tidal pertubation potential proportional to the                 !
        !    pressure gradient                                                                  !
  
        !--- calculate zonal velocity  
        DO lat = lats_start_u , lats_end_u
           DO lon = longs_start , longs_end
              zonal_vel_pressure_gradient_imp(lon   , lat) = &
                 u_pressure_gradient_coef_imp(lon   , lat) * &
                                (   elevation(lon+1 , lat)   & 
                                  - elevation(lon   , lat)   &
                                )    
           END DO
        END DO 
 
        !---------------------------------------------------------------------------------------!

    END FUNCTION zonal_vel_pressure_gradient_imp

    !-------------------------------------------------------------------------------------------!

    ELEMENTAL FUNCTION zonal_vel_bottom_friction( zonal_vel , merid_vel , total_depth )              !--- org: fxn

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !
    !     zonal velocity due to bottom friction                                                 !
    !     (bottom friction coefficient)/(total depth)*u*sqrt(u^2+v^2)                           !
    !     expressed by the function                                                             !
    !     (total depth)/(total depth+(bottom friction coefficient)*timestep*sqrt(u^2+v^1)       !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none

        !--- Function 
        REAL (KIND=wp)                                  :: zonal_vel_bottom_friction

        !--- Function variables
        !--- Field of zonal velocity for u-points
        REAL (KIND=wp) , INTENT (IN)                    :: zonal_vel
        !--- Field of meridional velocity for u-points
        REAL (KIND=wp) , INTENT (IN)                    :: merid_vel
        !--- Field of total depth at u-points
        REAL (KIND=wp) , INTENT (IN)                    :: total_depth

        !--- Local variables
        !--- Cofficient of bottom friction * time step
        REAL (KIND=wp)                                  :: coefficient    


        !---------------------------------------------------------------------------------------!
        !--- Calculate zonal velocity due to bottom friction -----------------------------------!

        !--- set coefficient
        coefficient = bottom_friction_coef * delta_time
        !--- calculate u^2+v^2
        zonal_vel_bottom_friction = zonal_vel * zonal_vel  &
                                  + merid_vel * merid_vel
        !--- calculate sqrt(u^2+v^2)
        zonal_vel_bottom_friction = SQRT( zonal_vel_bottom_friction )
        !--- calculate bottom friction function 
        zonal_vel_bottom_friction       = &
                            total_depth / &
          (                 total_depth   &
            +               coefficient   &
            * zonal_vel_bottom_friction   &
          )

        !---------------------------------------------------------------------------------------!

    END FUNCTION zonal_vel_bottom_friction

    !-------------------------------------------------------------------------------------------!

END MODULE mod_zonal_velocity


