!-----------------------------------------------------------------------------------------------!
!   SEILER TIDE MODEL                                                                           !
!                                                                                               !
! orginal version from Ulrike Seiler, ulrike.seiler@zmaw.de                                     !
!                                                                                               !
! (c) Anna Fuchs 2012  , anna.fuchsmakaeva@googlemail.com                                       !
! (c) Petra Nerge 2012 , petra.nerge@informatik.uni-hamburg.de                                  !
!-----------------------------------------------------------------------------------------------!

PROGRAM globe

   USE mod_constants
   USE mod_grid
   USE mod_tide_force
   USE mod_time_stepping
   USE mod_earth_quantities
   USE mod_zonal_velocity
   USE mod_merid_velocity
   USE mod_elevation
   
   USE mpi
   !INCLUDE 'mpif.h'
   

   IMPLICIT NONE

   !--- Workspace for calculation of state variables --------------------------------------------!
   !--- Workspace field of zonal velocities
   REAL (KIND=wp) , DIMENSION (longs_start:longs_end             ,    lats_start_u:lats_end_u   ) &
   :: u_new             = 0._wp                                                                  !--- org: ua
   !--- Workspace field of zonal bottom friction 
   REAL (KIND=wp) , DIMENSION (longs_start:longs_end             ,    lats_start_u:lats_end_u   ) &
   :: u_bottom_friction = 0._wp                                                                  !--- org: fxn
   !--- Workspace field of meridional velocities
   REAL (KIND=wp) , DIMENSION (longs_start:longs_end             ,    lats_start_v:lats_end_v   ) &
   :: v_new             = 0._wp                                                                  !--- org: va
   !--- Workspace field of meridional bottom friciton
   REAL (KIND=wp) , DIMENSION (longs_start:longs_end             ,    lats_start_v:lats_end_v   ) &
   :: v_bottom_friction = 0._wp                                                                  !--- org: fyn
   !--- Workspace field of sea surface elevations 
   REAL (KIND=wp) , DIMENSION (longs_start_sub_1:longs_end_add_1 , lats_start_zeta:lats_end_zeta) &
   :: zeta_new          = 0._wp                                                                  !--- org: zetaa
   !---------------------------------------------------------------------------------------------!


   !--- Program status variables ----------------------------------------------------------------!
   LOGICAL                              :: lstatus = .false.
!   INTEGER                              :: istatus
   !--- I/O status variable
   INTEGER                              :: iostatus = 0
   !--- Unit number of error messages
   INTEGER                              :: errUnit = 0

   !--- Help variables
   !--- Time stepping loop counter
   INTEGER                              :: timestep 


   !--------------------------------------------------------------------------------------------!
   !--- Initialize grid ------------------------------------------------------------------------!
   !    Fix me: set depth to constant                                                           !
   
   !MPI
   INTEGER :: IERROR
   CALL MPI_INIT(IERROR)
   
   CALL grid_initialize( )
   
   !--------------------------------------------------------------------------------------------!
   !--- Initialize state variables: sea surface elevation (zeta), zonal velocity (u), ----------!
   !    meridional velocity (v)                                                                 !
   !    Fix me: set to zero                                                                     !
   
   zeta = 0._wp
   u    = 0._wp
   v    = 0._wp

   !--------------------------------------------------------------------------------------------!
   !--- Initialize tide force ------------------------------------------------------------------!

   CALL tide_force_initialize( )

   !--------------------------------------------------------------------------------------------!
   !--- Initialize time stepping ---------------------------------------------------------------!

   CALL time_stepping_initialize( )

   !--------------------------------------------------------------------------------------------!
   !--- Initialize earth quantities ------------------------------------------------------------!

   !--- set coriolis operator for zonal velocities (u-points) 
   coriolis_operator_u(lats_start_u:lats_end_u) = earth_quantities_coriolis_u( ) 
   !--- set coriolis operator for meridional velocities (v-points) 
   coriolis_operator_v(lats_start_v:lats_end_v) = earth_quantities_coriolis_v( )

   !--------------------------------------------------------------------------------------------!
   !--- Initialize zonal momentum/velocity equation --------------------------------------------!

   !--- set coefficients of explicit part of pressure gradient
   u_pressure_gradient_coef_exp(lats_start_u:lats_end_u) = zonal_vel_pressure_gradient_coef_exp( )

   !--------------------------------------------------------------------------------------------!
   !--- Initialize meridional momentum/velocity equation ---------------------------------------!

   !--- set coefficients of explicit part of pressure gradient
   v_pressure_gradient_coef_exp = merid_vel_pressure_gradient_coef_exp( )

   !--------------------------------------------------------------------------------------------!
   !--- Time stepping --------------------------------------------------------------------------!

   DO timestep = 0 , timesteps_tide_period * number_tide_periods       !200 , step
      !--- set simulation time
      simulation_time = timestep * delta_time
      !--- set phase lag of the tide
      phase_lag_tide  = tide_omega * simulation_time
      !--- set cosine and sine of the tide phase lag
      cos_phase_lag_tide = COS( phase_lag_tide )
      sin_phase_lag_tide = SIN( phase_lag_tide )

      !--- Prepare calculation -----------------------------------------------------------------!
      !--- set total depths at zeta-points
      depth_add_zeta(longs_start:longs_end , lats_start_zeta:lats_end_zeta) = &
               depth(longs_start:longs_end , lats_start_zeta:lats_end_zeta) + &
                zeta(longs_start:longs_end , lats_start_zeta:lats_end_zeta)
      !--- cyclic boundary condition for longitudes
      depth_add_zeta(longs_end_add_1 , lats_start_zeta:lats_end_zeta) = &
      depth_add_zeta(longs_start     , lats_start_zeta:lats_end_zeta)
     
      !--- set total depth at u-points
      depth_add_zeta_u_points(longs_start:longs_end , lats_start_u:lats_end_u) = &
      grid_depth_add_zeta_u_points( )

      !--- set total depth at v-points
      depth_add_zeta_v_points(longs_start:longs_end , lats_start_v:lats_end_v) = &
      grid_depth_add_zeta_v_points( )

      !--- set meridional velocities (v) at u-points
      v_u_points(longs_start:longs_end , lats_start_u:lats_end_u) = merid_vel_u_points( )

      !--- set zonal velocities (u) at v-points
      u_v_points(longs_start:longs_end , lats_start_v:lats_end_v) = zonal_vel_v_points( )
      !-----------------------------------------------------------------------------------------!

      !--- Zonal momentum, explicit calculation ------------------------------------------------! 
      !--- due to tidal force
      u_new(longs_start:longs_end , lats_start_u:lats_end_u) = &
      tide_force_zonal( cos_phase_lag_tide , sin_phase_lag_tide )

      !--- due to horizontal diffusion of momentum
      u_new(longs_start:longs_end , lats_start_u:lats_end_u)                                     = &
      u_new(longs_start:longs_end , lats_start_u:lats_end_u)                                     + &
      eddy_horizontal_coef                                                                       * &
      (                                                                                            &
        zonal_vel_eddy_d2u_d2lon( )                                                              + &
        zonal_vel_eddy_d2u_d2lat( )                                                              + &
        zonal_vel_eddy_du_dlat( )                                                                + &
        zonal_vel_eddy_u( )                                                                      + &
        zonal_vel_eddy_dv_dlon( v(longs_start:longs_end_add_1 , lats_start_v:lats_end_u_add_1) )   &
      )

      !--- due to advection 
      u_new                          (longs_start:longs_end , lats_start_u:lats_end_u)   = &
      u_new                          (longs_start:longs_end , lats_start_u:lats_end_u)   - &
      zonal_vel_advection( v_u_points(longs_start:longs_end , lats_start_u:lats_end_u) )

      !--- due to coriolis acceleration
      u_new                         (longs_start:longs_end , lats_start_u:lats_end_u)   = &
      u_new                         (longs_start:longs_end , lats_start_u:lats_end_u)   + &
      zonal_vel_coriolis( v_u_points(longs_start:longs_end , lats_start_u:lats_end_u) )
      !-----------------------------------------------------------------------------------------!

      !--- Zonal velocity, explicit calculation ------------------------------------------------!
      !--- get explicit calculated zonal velocity
      u_new(longs_start:longs_end , lats_start_u:lats_end_u)              = &
      u_new(longs_start:longs_end , lats_start_u:lats_end_u) * delta_time + &
          u(longs_start:longs_end , lats_start_u:lats_end_u)    

      !--- due to explicit part of pressure gradient and 
      !    parametrization of tidal pertubation potential
      u_new                                (longs_start:longs_end       , lats_start_u:lats_end_u)   = &
      u_new                                (longs_start:longs_end       , lats_start_u:lats_end_u)   - &
      zonal_vel_pressure_gradient_exp( zeta(longs_start:longs_end_add_1 , lats_start_u:lats_end_u) ) 

      !--- get bottom friction function, is also needed by implicit part of pressure gradient
      u_bottom_friction                                 (longs_start:longs_end , lats_start_u:lats_end_u) = &
      zonal_vel_bottom_friction(                       u(longs_start:longs_end , lats_start_u:lats_end_u) , &
                                              v_u_points(longs_start:longs_end , lats_start_u:lats_end_u) , &
                                 depth_add_zeta_u_points(longs_start:longs_end , lats_start_u:lats_end_u)   &
                               )

      !--- due to bottom friction
      u_new            (longs_start:longs_end , lats_start_u:lats_end_u) = &
      u_new            (longs_start:longs_end , lats_start_u:lats_end_u) * &
      u_bottom_friction(longs_start:longs_end , lats_start_u:lats_end_u) 
      !-----------------------------------------------------------------------------------------!

      !--- Meridional momentum, explicit calculation -------------------------------------------! 
      !--- due to tidal force
      v_new(longs_start:longs_end , lats_start_v:lats_end_v)      = &
      tide_force_merid( cos_phase_lag_tide , sin_phase_lag_tide )

      !--- due to horizontal diffusion of momentum
      v_new(longs_start:longs_end , lats_start_v:lats_end_v)                               = &
      v_new(longs_start:longs_end , lats_start_v:lats_end_v)                               + &
      eddy_horizontal_coef                                                                 * &
      (                                                                                      &
        merid_vel_eddy_d2v_d2lon( )                                                        + &
        merid_vel_eddy_d2v_d2lat( )                                                        + &
        merid_vel_eddy_dv_dlat( )                                                          + &
        merid_vel_eddy_v( )                                                                + &
        merid_vel_eddy_du_dlon( u(longs_start_sub_1:longs_end , lats_start_u:lats_end_v) )   &
      )

      !--- due to advection 
      v_new                          (longs_start:longs_end , lats_start_v:lats_end_v)   = &
      v_new                          (longs_start:longs_end , lats_start_v:lats_end_v)   - &
      merid_vel_advection( u_v_points(longs_start:longs_end , lats_start_v:lats_end_v) )

      !--- due to coriolis acceleration
      v_new                         (longs_start:longs_end , lats_start_v:lats_end_v)   = &
      v_new                         (longs_start:longs_end , lats_start_v:lats_end_v)   - &
      merid_vel_coriolis( u_v_points(longs_start:longs_end , lats_start_v:lats_end_v) )
      !-----------------------------------------------------------------------------------------!

      !--- Meridional velocity, explicit calculation 
      !--- get explicit calculated meridional velocity
      v_new(longs_start:longs_end , lats_start_v:lats_end_v)              = &
      v_new(longs_start:longs_end , lats_start_v:lats_end_v) * delta_time + &
          v(longs_start:longs_end , lats_start_v:lats_end_v)  

      !--- due to explicit part of pressure gradient and 
      !    parametrization of tidal pertubation potential
      v_new                                (longs_start:longs_end ,    lats_start_v:lats_end_v)   = &
      v_new                                (longs_start:longs_end ,    lats_start_v:lats_end_v)   - &
      merid_vel_pressure_gradient_exp( zeta(longs_start:longs_end , lats_start_zeta:lats_end_v) ) 

      !--- get bottom friction function, is also needed by implicit part of pressure gradient
      v_bottom_friction                                 (longs_start:longs_end , lats_start_v:lats_end_v) = &
      merid_vel_bottom_friction(                       v(longs_start:longs_end , lats_start_v:lats_end_v) , &
                                              u_v_points(longs_start:longs_end , lats_start_v:lats_end_v) , &
                                 depth_add_zeta_v_points(longs_start:longs_end , lats_start_v:lats_end_v)   &
                               )

      !--- due to bottom friction 
      v_new            (longs_start:longs_end , lats_start_v:lats_end_v) = &
      v_new            (longs_start:longs_end , lats_start_v:lats_end_v) * &
      v_bottom_friction(longs_start:longs_end , lats_start_v:lats_end_v)
      !-----------------------------------------------------------------------------------------!

      !--- Sea surface elevation, implicit calculation -----------------------------------------!
      !--- get iteration matrix for implicit method
      CALL elevation_iteration_matrix_prepare                                                      &
           ( zeta_iteration_coef_east    (longs_start:longs_end ,    lats_start_u:lats_end_u   ) , & 
             zeta_iteration_coef_west    (longs_start:longs_end ,    lats_start_u:lats_end_u   ) , &
             zeta_iteration_coef_south   (longs_start:longs_end ,    lats_start_v:lats_end_v   ) , &
             zeta_iteration_coef_north   (longs_start:longs_end ,    lats_start_u:lats_end_u   ) , &
             zeta_iteration_coef         (longs_start:longs_end , lats_start_zeta:lats_end_zeta) , &
             u_new                       (longs_start:longs_end ,    lats_start_u:lats_end_u   ) , & 
             u                           (longs_start:longs_end ,    lats_start_u:lats_end_u   ) , &
             v_new                       (longs_start:longs_end ,    lats_start_v:lats_end_v   ) , &
             v                           (longs_start:longs_end ,    lats_start_v:lats_end_v   ) , &
             depth_add_zeta_u_points     (longs_start:longs_end ,    lats_start_u:lats_end_u   ) , &
             depth_add_zeta_v_points     (longs_start:longs_end ,    lats_start_v:lats_end_v   ) , & 
             u_pressure_gradient_coef_imp(longs_start:longs_end ,    lats_start_u:lats_end_u   ) , &
             v_pressure_gradient_coef_imp(longs_start:longs_end ,    lats_start_v:lats_end_v   )   &
           )

      !--- iteration of elevation by relaxation
      CALL elevation_iteration_relaxation                                                                   &
           ( zeta_new                 (longs_start_sub_1:longs_end_add_1 , lats_start_zeta:lats_end_zeta) , &
             zeta_iteration_coef_east (      longs_start:longs_end       ,    lats_start_u:lats_end_u   ) , & 
             zeta_iteration_coef_west (      longs_start:longs_end       ,    lats_start_u:lats_end_u   ) , &
             zeta_iteration_coef_south(      longs_start:longs_end       ,    lats_start_v:lats_end_v   ) , &
             zeta_iteration_coef_north(      longs_start:longs_end       ,    lats_start_u:lats_end_u   ) , &
             zeta_iteration_coef      (      longs_start:longs_end       , lats_start_zeta:lats_end_zeta) , &
             zeta_new_residuum_max                                                                        , &
             zeta_new_convergence_checks                                                                  , &
             lstatus                                                                                        &
           )
      !--- error handling
      IF ( .not. lstatus ) THEN
              WRITE( errUnit , * , IOSTAT = iostatus )                                         &
                     "stm ERROR :: Iteration of sea surface elvation do not converge = "     , &
                     lstatus
              WRITE( errUnit , * , IOSTAT = iostatus)                                          &
                     "convergence criterion = " , convergence_criterion                      , &
                     "maximum residuum = "      , zeta_new_residuum_max
              WRITE( errUnit , * , IOSTAT = iostatus)                                          &
                     "maximum number of convergence checks = " , max_convergence_checks      , &
                     "number of convergence checks = "         , zeta_new_convergence_checks  
              WRITE( errUnit , * , IOSTAT = iostatus)                                          &
                     "number of iterations before convergence check = " , number_iterations    
              STOP 
      END IF
      !-----------------------------------------------------------------------------------------!

      !--- Zonal velocity, implicit calculation ------------------------------------------------! 
      !--- get zonal implicit pressure gradient coefficients
      u_pressure_gradient_coef_imp(longs_start:longs_end , lats_start_u:lats_end_u) = &
      zonal_vel_pressure_gradient_coef_imp                                            &
               ( u_bottom_friction(longs_start:longs_end , lats_start_u:lats_end_u)   &
               ) 

      !--- due to implicit part of pressure gradient and 
      !    parametrization of tidal pertubation potential
      u_new                                    (longs_start:longs_end       , lats_start_u:lats_end_u)   = &
      u_new                                    (longs_start:longs_end       , lats_start_u:lats_end_u)   - &
      zonal_vel_pressure_gradient_imp( zeta_new(longs_start:longs_end_add_1 , lats_start_u:lats_end_u) ) 
      !-----------------------------------------------------------------------------------------!

      !--- Meridional velocity, implicit calculation -------------------------------------------!
      !--- Get meridional implicit pressure gradient coefficients
      v_pressure_gradient_coef_imp(longs_start:longs_end , lats_start_v:lats_end_v) = &
      merid_vel_pressure_gradient_coef_imp                                            &
               ( v_bottom_friction(longs_start:longs_end , lats_start_v:lats_end_v)   &
               )
 
      !--- due to implicit part of pressure gradient and 
      !    parametrization of tidal pertubation potential
      v_new                                    (longs_start:longs_end ,    lats_start_v:lats_end_v)   = &
      v_new                                    (longs_start:longs_end ,    lats_start_v:lats_end_v)   - &
      merid_vel_pressure_gradient_imp( zeta_new(longs_start:longs_end , lats_start_zeta:lats_end_v) ) 
      !-----------------------------------------------------------------------------------------!

      !--- set total depths at zeta-points
      depth_add_zeta(longs_start:longs_end , lats_start_zeta:lats_end_zeta) = &
               depth(longs_start:longs_end , lats_start_zeta:lats_end_zeta) + &
            zeta_new(longs_start:longs_end , lats_start_zeta:lats_end_zeta)
      !--- cyclic boundary condition for longitudes
      depth_add_zeta(longs_end_add_1 , lats_start_zeta:lats_end_zeta) = &
      depth_add_zeta(longs_start     , lats_start_zeta:lats_end_zeta)

      !--- Update seaface elevaton -------------------------------------------------------------!
      !--- sea surface elevation
      zeta    (longs_start_sub_1:longs_end_add_1 , lats_start_zeta:lats_end_zeta) = &
      zeta_new(longs_start_sub_1:longs_end_add_1 , lats_start_zeta:lats_end_zeta)
      !-----------------------------------------------------------------------------------------!

      !--- Update zonal velocity ---------------------------------------------------------------!
      u    (longs_start:longs_end , lats_start_u:lats_end_u) = &
      u_new(longs_start:longs_end , lats_start_u:lats_end_u)

      !--- cyclic boundary condition for longitudes
      u(longs_start_sub_1 , lats_start_u:lats_end_u) = &
      u(longs_end         , lats_start_u:lats_end_u)

      u(longs_end_add_1   , lats_start_u:lats_end_u) = &
      u(longs_start       , lats_start_u:lats_end_u) 

      !--- set j+1-values of zonal velocity at northernmost latitudes by cross-polar averaging
      u(longs_start_sub_1:longs_end_add_1 , lats_end_u_add_1) = zonal_vel_boundary_lat( )
      !-----------------------------------------------------------------------------------------!

      !--- Update meridional velocity ----------------------------------------------------------!
      !--- Fix me: fpe!!!  
!      v    (longs_start:longs_end , lats_start_v:lats_end_v) = & 
!      v_new(longs_start:longs_end , lats_start_v:lats_end_v)

      !--- cyclic boundary condition for longitudes
      v(longs_start_sub_1 , lats_start_v:lats_end_v) = &
      v(longs_end         , lats_start_v:lats_end_v)

      v(longs_end_add_1   , lats_start_v:lats_end_v) = &
      u(longs_start       , lats_start_v:lats_end_v) 
       
      !--- set j+1-values of meridional velocity at northernmost latitudes by cross-polar averaging
      v(longs_start_sub_1:longs_end_add_1 , lats_end_v_add_1) = merid_vel_boundary_lat( )
      !-----------------------------------------------------------------------------------------!

   END DO
   
   CALL MPI_FINALIZE(IERROR)

END PROGRAM





















